/*  Copyright (C) 1993 Wade R. Schauer  */
/*  All Rights Reserved                 */
/*  ----------------------------------  */
/**/
/* fortran call of the curses "waddstr" and "addstr" functions */

#include <curses.h>
#include "fcursdef.h"

INT4 wmove_();
INT4 move_();

/*******************************************************************/

INT4 twaddstr_( wnum, str, len )
INT4 *wnum;
char *str;
INT4 *len;
{
	WINDOW *win;
	int ntmp = *len;

	win = (WINDOW *) *wnum;

	/* use the waddnstr function and pass the Fortran "len" value */
	/* since the fortran string doesn't have a null terminator */

	return waddnstr( win, str, ntmp );
}

/**************************************************************/

/* as in C, this just calls 'waddstr' with "stdscr" as the window */

INT4 taddstr_( str, len )
char *str;
INT4 *len;
{
	return twaddstr_( (INT4 *) &stdscr, str, len );
}

/***************************************************************/

/* combine the "wmove" and "waddstr" functions */

INT4 tmvwaddstr_( wnum, y, x, str, len )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
{
	WINDOW *win;
	int ntmp = *len;

	win = (WINDOW *) *wnum;

	if ( wmove_( wnum, y, x ) != ERR )
		return waddnstr( win, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

/* combine the "move" and "waddstr" functions */

INT4 tmvaddstr_( y, x, str, len )
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
{
	if ( wmove_( (INT4 *) &stdscr, y, x ) != ERR )
		return twaddstr_( (INT4 *) &stdscr, str, len );
	else
		return ERR;
}

/***************************************************************/

/* "move" in a window and then print the string to n positions */

INT4 tmvwaddnstr_( wnum, y, x, str, len, n )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	WINDOW *win;
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	win = (WINDOW *) *wnum;

	if ( wmove_( wnum, y, x ) != ERR )
		return waddnstr( win, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

INT4 tmvaddnstr_( y, x, str, len, n )
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	if ( move_( y, x ) != ERR )
		return waddnstr( stdscr, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

INT4 twaddnstr_( wnum, str, len, n )
INT4 *wnum;
char *str;
INT4 *len;
INT4 *n;
{
	WINDOW *win;
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	win = (WINDOW *) *wnum;

	return waddnstr( win, str, ntmp );
}

/***************************************************************/

INT4 taddnstr_( str, len, n )
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	return waddnstr( stdscr, str, ntmp );
}
