/*  Copyright (C) 1993 Wade R. Schauer  */
/*  All Rights Reserved                 */
/*  ----------------------------------  */
/**/
/* fortran call of the curses "insstr" set of functions */

#include <curses.h>
#include "fcursdef.h"

INT4 wmove_();

/*******************************************************************/

INT4 twinsstr_( wnum, str, len )
INT4 *wnum;
char *str;
INT4 *len;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	/* use the winsnstr function and pass the Fortran "len" value */
	/* since the fortran string doesn't have a null terminator */

	return winsnstr( win, str, ntmp );
}

/**************************************************************/

/* as in C, this just calls 'winsstr' with "stdscr" as the window */

INT4 tinsstr_( str, len )
char *str;
INT4 *len;
{
	return twinsstr_( (INT4 *) &stdscr, str, len );
}

/***************************************************************/

/* combine the "wmove" and "winsstr" functions */

INT4 tmvwinsstr_( wnum, y, x, str, len )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len; 
{
	int ntmp = *len; 

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( wmove_( wnum, y, x ) != ERR ) 
		return winsnstr( win, str, ntmp ); 
	else 
		return ERR; 
}

/***************************************************************/

/* combine the "move" and "winsstr" functions */ 

INT4 tmvinsstr_( y, x, str, len )
INT4 *y;
INT4 *x;
char *str;
INT4 *len; 
{
	if ( wmove_( (INT4 *) &stdscr, y, x ) != ERR ) 
		return twinsstr_( (INT4 *) &stdscr, str, len );
	else
		return ERR;
}

/***************************************************************/

/* "move" in a window and then print the string to n positions */

INT4 tmvwinsnstr_( wnum, y, x, str, len, n )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( *n > 0 && *n <= *len )
		ntmp = *n;

	if ( wmove_( wnum, y, x ) != ERR )
		return winsnstr( win, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

INT4 tmvinsnstr_( y, x, str, len, n )
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	INT4 twinsnstr_();

	if ( wmove_( (INT4 *) &stdscr, y, x ) != ERR )
		return twinsnstr_( (INT4 *) &stdscr, str, len, n );
	else
		return ERR;
}

/***************************************************************/

INT4 twinsnstr_( wnum, str, len, n )
INT4 *wnum;
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	return winsnstr( win, str, ntmp );
}

/***************************************************************/

INT4 tinsnstr_( str, len, n )
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	return winsnstr( stdscr, str, ntmp );
}
