/*  Copyright (C) 1993 Wade R. Schauer  */
/*  All Rights Reserved                 */
/*  ----------------------------------  */
/**/
/* fortran call of the curses "instr" set of functions */

#include <curses.h>
#include "fcursdef.h"

INT4 wmove_();

/*******************************************************************/

INT4 twinstr_( wnum, str, len )
INT4 *wnum;
char *str;
INT4 *len;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	/* use the winnstr function and pass the Fortran "len" value */
	/* since the fortran string doesn't have a null terminator */

	return winnstr( win, str, ntmp );
}

/**************************************************************/

/* as in C, this just calls 'winstr' with "stdscr" as the window */

INT4 tinstr_( str, len )
char *str;
INT4 *len;
{
	return twinstr_( (INT4 *) &stdscr, str, len );
}

/***************************************************************/

/* combine the "wmove" and "winstr" functions */

INT4 tmvwinstr_( wnum, y, x, str, len )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( wmove_( wnum, y, x ) != ERR )
		return winnstr( win, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

/* combine the "move" and "winstr" functions */

INT4 tmvinstr_( y, x, str, len )
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
{
	if ( wmove_( (INT4 *) &stdscr, y, x ) != ERR )
		return twinstr_( (INT4 *) &stdscr, str, len );
	else
		return ERR;
}

/***************************************************************/

/* "move" in a window and then print the string to n positions */

INT4 tmvwinnstr_( wnum, y, x, str, len, n )
INT4 *wnum;
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	if ( wmove_( wnum, y, x ) != ERR )
		return winnstr( win, str, ntmp );
	else
		return ERR;
}

/***************************************************************/

INT4 tmvinnstr_( y, x, str, len, n )
INT4 *y;
INT4 *x;
char *str;
INT4 *len;
INT4 *n;
{
	INT4 twinnstr_();

	if ( wmove_( (INT4 *) &stdscr, y, x ) != ERR )
		return twinnstr_( (INT4 *) &stdscr, str, len, n );
	else
		return ERR;
}

/***************************************************************/

INT4 twinnstr_( wnum, str, len, n )
INT4 *wnum;
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	WINDOW *win;

	win = (WINDOW *) *wnum;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	return winnstr( win, str, ntmp );
}

/***************************************************************/

INT4 tinnstr_( str, len, n )
char *str;
INT4 *len;
INT4 *n;
{
	int ntmp = *len;

	if ( *n >= 0 && *n <= *len )
		ntmp = *n;

	return winnstr( stdscr, str, ntmp );
}
