* wrs (10/12/92) - a curses screen saver using F-Curses (fortran)
*			(simplified movebox)

	implicit none

	include '../../curses.fh'

* use "stdscr" as the output window -- copy other windows into this
* window and then write only this window ( facilitates pop-up-window action )
* to the phisical screen

*   screen response is much better if you read from stdscr as well, because
*   if you read from another window, that window will actually be written
*   to the screen if it hasn't been done manually.
*         (wgetch, wgetstr, etc.)

* "small" is the popup window (box)

	integer shadow
	common /bckground/ shadow

	integer small
	common /smallcom/ small

* current position of the popup *
	integer y, x
	common /position/ y,x

 	integer i, retval
 	integer ch

	integer    FIRST_X, FIRST_Y, W_LINES, W_COLS

	data FIRST_X / 5 /
	data FIRST_Y / 5 /
	data W_LINES / 5 /
	data W_COLS  / 11 /

*   initialize curses

	retval = initscr()

*  set certain input/output options **************************

*  don't echo keystrokes, get keystrokes immediately, 
*  interpret special keys, hide cursor if possible

	retval = noecho()
	retval = cbreak()
	retval = raw()
	retval = keypad(stdscr,.TRUE.)
*	retval = leaveok(stdscr, .TRUE.)
	retval = curs_set( 0 )

*  only get a character after the screen has been completely written since
*  the last key action (Disable typeahead checking).  THIS IS CRITICAL
*  FOR GETTING A SMOOTH APPEARANCE OF THE BOX WHEN IN MOTION.  Comment
*  out this call to see the difference, especially when holding down
*  one of the arrow keys.

	retval = typeahead(-1)

**************************************************************


	retval = wnoutrefresh(stdscr)

*	create a small pop-up window to move around

	y = FIRST_Y
	x = FIRST_X
	small = newwin(W_LINES, W_COLS, y, x)
	shadow = newwin(W_LINES, W_COLS, y, x)

*	if the terminal has color capabilities, make it stand out with a
*	white on red color pair.

*	if colors are not supported, set the box to inverse video

	if(has_colors()) then
		retval = start_color()
		retval = init_pair( 1, COLOR_WHITE, COLOR_RED )
		retval = wattrset( small, COLOR_PAIR(1) )
		retval = wattron( small, A_BOLD )
	else
		retval = wattrset(small, A_REVERSE)
	endif

	retval = wclear(small)

*	add text to window *

	retval = mvwaddstr(small, 1, 1, ' Bounce  ')
	retval = mvwaddstr(small, 2, 1, '         ')
	retval = mvwaddstr(small, 3, 1, '   Box   ')

*	box it, with "double line" graphics if at386, other terminal types
*	will inherit the single line definitions.

	retval = wborder( small, ACS_DVLINE, ACS_DVLINE, ACS_DHLINE,
     .		ACS_DHLINE, ACS_DULCORNER, ACS_DURCORNER,
     .		ACS_DLLCORNER, ACS_DLRCORNER)


	call bounce()

	retval = clear()
	retval = refresh()
	retval = endwin()
	stop

	end

************************************************************************

* bounce - make the small window bounce, vaguely like pong

	subroutine bounce()

	implicit none

	include '../../curses.fh'

	integer seed
	save seed

	integer shadow
	common /bckground/ shadow

	integer small
	common /smallcom/ small

	integer y, x
	common /position/ y,x

	integer yi, xi, ysign, xsign
	integer ch
	integer randomize, retval
	real random, randval

	data seed / 13567 /

	yi = 1
	xi = 1
	randomize = 0
	randval = .75

*	set "nodelay" so that "sgetch" won't wait for the user if a key
*	hasn't already been pressed.

	retval = nodelay(stdscr,.TRUE.)
	do while ( sgetch() .eq. ERR )

*		try continue moving in same direction *

		if( OK .eq. mvwin(small, y+yi, x+xi) ) then
			retval = mvwin(shadow, y+yi, x+xi)
			y = y + yi
			x = x + xi

*		hit the screen border, time to change direction.  randomly
*		increase or decrease the speed of the movebox each time
*		the screen border is encountered.

		else
			if( ERR .eq. mvwin(small, y+yi, x) ) then
				yi = -yi
				ysign = yi/iabs(yi)
				if( randomize .gt. 0 ) then
				  if( random(seed) .gt. .5 ) then
					yi = 1
				  else
				  	yi = yi + ysign
				  endif
				  yi = min(iabs(yi),2)
				  yi = yi*ysign
				endif
			
			else
				retval = mvwin(shadow, y+yi, x)
				xi = -xi
				xsign = xi/iabs(xi)
				if( randomize .gt. 0 ) then
				  if( random(seed) .gt. .5 ) then
					randval = random(seed)
					xi = 1
				  else
				  	xi = xi + xsign
				  endif
				  xi = min(iabs(xi),2)
				  xi = (xi * COLS/LINES*.8) * xsign
				endif
			endif
		endif

*		make the small window popup again (goes to screen)

		retval = touchwin(small)
		retval = wrefresh(small)

		retval = napms(int(randval*300))

*		and remove it (only goes to virtual screen)

		retval = touchwin(shadow)
		retval = wnoutrefresh(shadow)

		if( randomize .eq. 0 ) then
		  randomize = 1
		else
		  randomize = 0
		endif

	end do

*	if here, then the user finally pressed a key 

*	reset "nodelay" to false so that "sgetch" will wait for the user

	retval = nodelay(stdscr,.FALSE.)

*	flush the keystrokes that terminated the bouncing so that they won't
*	be processed by the calling routine.

	retval = flushinp()

	return
	end

*****************************************************************

*  a generic overflow psuedo random number generator
*	(requires a seed number to be passed from/to the calling program)

	real*4 function random( seed )
	
	implicit none
	
	integer*4 seed, iy

	iy = seed * 65539
	if ( iy .lt. 0 ) iy = ( iy + 2147483647 ) + 1

	random = iy * 0.4656613e-09

	if(random .lt. 0.0001) then
	  random = 0.000001
	else if( random .ge. 1.0) then
	  random = 0.999999
	endif

	seed = iy

	return
	end
