*  Copyright 1992 Wade R. Schauer

* wrs (8/9/92)  CURS-CALC
*  A character based screen calculator using F-Curses.
*	- started with limited goals in mind, but evolved into a decent
*		scientific calculator emulation.

*		(double precision and double-precision complex calculations)

*	some of the features of F-curses that it uses

*		- window and sub-windows
*		- the ability to create pop-up windows
*		- the ability to scan for keyboard input one character
*		  at a time and interpret special keys

*		- the ability to read strings from a screen window
*		- the ability to write strings to a screen window
*		- the ability to modify output attributes depending
*		   on the capabilities of a terminal (color vs. monochrome)

*		- a number of utility functions/subroutines that build on
*		  the basic abilities of F-curses, including a pop-up
*		  menu.

*	(this was done quickly, so don't laugh if it is not the
*	most efficient code you've ever seen)

	implicit none

	include '../../curses.fh'

* the function window

	integer functwin
	common /extras/ functwin

* the calculator window, its display fields, and its button windows

	include 'calclate.fh'

	integer  keyattr, hotattr
	common /attrcom/ keyattr, hotattr

	include 'calcstat.fh'

	include 'register.fh'

* where to go when arrow keys are pressed

	integer  up(45), down(45), left(45), right(45), keywin(45)

	common /keypoint/ keywin


* allow memory for 10 keyboard macros of 200 keystrokes
*	(always Make MAXMAC a multiple of 4)

	integer MAXMAC
	parameter (MAXMAC=4*50)
	integer keymac(10,MAXMAC), totstrokes(10), macitem, runitem
	integer keycount
	character*16 macname(10)
	logical macmode(10), macstep

	character*5  answer(3)


* menu items for popup menu of additional calculator functions

	integer MAXFUNCTS
	parameter (MAXFUNCTS=17)
	character*20 functmen(MAXFUNCTS)

	integer MAXCONV
	parameter(MAXCONV=19)
	character*25 convmen1(MAXCONV), convmen2(MAXCONV)

	integer MAXSTATS
	parameter(MAXSTATS=17)
	character*25 statmen(MAXSTATS)

	integer MAXCONST
	parameter(MAXCONST=15)
	character*30 constmen(MAXCONST)

* miscellaneous

 	integer i, retval, hotkey, oldkey, lightpad, displayval, y, x, imx, k
 	integer ch, wjustify, rectangle, help, popupmenu, KEY_SB, lastitem
	integer setpair, cpair, fpair, hpair, menupair, hlitpair
	integer lastchr, ll, irec, ie, seed, tmpkey, itemnum, lastconv
	integer convdir, laststat, lastconst, j, scrpair, revpair, functopt
	integer ichosen

	integer FKEY1,FKEY2,FKEY3,FKEY4,FKEY5,FKEY6,FKEY7,FKEY8,FKEY9,FKEY10
	integer fnpair, fhpair, fnattr, fhattr

	real*4  random
	real*8  xstat, x2stat, ystat, y2stat, xystat, nstat
	character  tmpstr*20, filenm*80, tmpmsg*80, menumsg*78

	character  HELPMSG*47, HELPFILE*12

	complex*16 cmpval, cmpval2
	real*8     splitcmp(2)
	equivalence ( cmpval, splitcmp(1) )

	data HELPMSG / 'Press <F1> or <?> for help, <CTRL>-<x> to exit'/
	data HELPFILE / 'curscalc.hlp'/

*  strings for a numeric functions menu

	functmen( 1) = '1/x'
	functmen( 2) = 'x^2'
	functmen( 3) = 'sqroot(x)'
	functmen( 4) = 'sin(x)'
	functmen( 5) = 'cos(x)'
	functmen( 6) = 'tan(x)'
	functmen( 7) = 'arcsin(x)'
	functmen( 8) = 'arccos(x)'
	functmen( 9) = 'arctan(x)'
	functmen(10) = 'e^x'
	functmen(11) = 'ln(x)'
	functmen(12) = 'log10(x)'
	functmen(13) = 'x!'
	functmen(14) = 'hypsin(x)'
	functmen(15) = 'hypcos(x)'
	functmen(16) = 'hyptan(x)'
	functmen(17) = 'fillstack(x)' 

*  strings for a statistics function menu

	statmen( 1) = 'ACCUM+ (^A)'
	statmen( 2) = 'ACCUM-'
	statmen( 3) = 'Mean (Arithmetic)'
	statmen( 4) = 'Standard Deviation'
	statmen( 5) = 'Linear Regression'
	statmen( 6) = 'Forecast y(x)'
	statmen( 7) = 'Correl. Coef. (r)'
	statmen( 8) = 'n'
	statmen( 9) = 'Sum of x'
	statmen(10) = 'Sum of x^2'
	statmen(11) = 'Sum of y'
	statmen(12) = 'Sum of y^2'
	statmen(13) = 'Sum of xy'
	statmen(14) = 'Permutations  P(y,x)'
	statmen(15) = 'Combinations  C(y,x)'
	statmen(16) = 'random #'
	statmen(17) = '<CLEAR Statistics>'

*  strings for a unit conversion menu

	convmen1( 1) = 'rect(x,y) --> polar(r,a)'
	convmen1( 2) = 'Degrees --> Radians'
	convmen1( 3) = 'Fahrenheit --> Celcius'
	convmen1( 4) = 'mile --> foot'
	convmen1( 5) = 'foot --> meter'
	convmen1( 6) = 'inch --> centimeter'
	convmen1( 7) = 'foot^3 --> gallon'
	convmen1( 8) = 'gallon --> liter'
	convmen1( 9) = 'mile/hr --> meter/s'
	convmen1(10) = 'lbm --> kilogram'
	convmen1(11) = 'lbf --> Newton'
	convmen1(12) = 'inHg --> atmosphere'
	convmen1(13) = 'atmosphere --> lbf/in^2'
	convmen1(14) = 'lbf/in^2 --> N/m^2'
	convmen1(15) = 'Btu --> ft-lbf'
	convmen1(16) = 'ft-lbf --> Joule'
	convmen1(17) = 'Joule --> calorie'
	convmen1(18) = 'horsepower --> Watt'
	convmen1(19) = 'Watt --> ft-lb/s'

	convmen2( 1) = 'rect(x,y) <-- polar(r,a)'
	convmen2( 2) = 'Degrees <-- Radians'
	convmen2( 3) = 'Fahrenheit <-- Celcius'
	convmen2( 4) = 'mile <-- foot'
	convmen2( 5) = 'foot <-- meter'
	convmen2( 6) = 'inch <-- centimeter'
	convmen2( 7) = 'foot^3 <-- gallon'
	convmen2( 8) = 'gallon <-- liter'
	convmen2( 9) = 'mile/hr <-- meter/s'
	convmen2(10) = 'lbm <-- kilogram'
	convmen2(11) = 'lbf <-- Newton'
	convmen2(12) = 'inHg <-- atmosphere'
	convmen2(13) = 'atmosphere <-- lbf/in^2'
	convmen2(14) = 'lbf/in^2 <-- N/m^2'
	convmen2(15) = 'Btu <-- ft-lbf'
	convmen2(16) = 'ft-lbf <-- Joule'
	convmen2(17) = 'Joule <-- calorie'
	convmen2(18) = 'horsepower <-- Watt'
	convmen2(19) = 'Watt <-- ft-lb/s'

* strings for a menu of physical constants

	constmen( 1) = 'Avogadro''s (molecules/mole)'
	constmen( 2) = 'Universal gas (J/mole-K)'
	constmen( 3) = 'Gas Volume @ stp. (m^3/mole)'
	constmen( 4) = 'Plank''s (Joule)'
	constmen( 5) = 'Gravitational (N-m^2/kg^2)'
	constmen( 6) = 'Standard grav. accel. (m/s^2)'
	constmen( 7) = 'Elementary charge (C)'
	constmen( 8) = 'Electrostat. force (N-m^2/C^2)'
	constmen( 9) = 'Electron rest mass (kg)'
	constmen(10) = 'Atomic mass (kg)'
	constmen(11) = 'Speed of light in Vac. (m/s)'
	constmen(12) = 'Boltzmann''s (J/molecule-K)'
	constmen(13) = 'Stefan-Boltzmann (W/m^2-K^4)'
	constmen(14) = 'Approx. radius of Earth (m)'
	constmen(15) = 'Faraday''s (C/mole)'

	menupair = 0
	hlitpair = 0

*   initialize curses

	retval = initscr()

*  Set Desired Screen Modes
*  don't echo keystrokes, get keystrokes immediately, trap kill & interrupt,
*  interpret special keys (function keys), hide cursor if possible

	retval = noecho()
	retval = raw()
	retval = keypad(stdscr,.TRUE.)

*	don't translate newline to carriage-return and linefeed

	retval = nonl()

*  only get a character after the screen has been completely written since
*  the last key action (Disable typeahead checking).  Popup windows will not
*  work correctly if not disabled.

	retval = typeahead(-1)

	if(has_colors()) then
		retval = start_color()
		scrpair = setpair( COLOR_WHITE, COLOR_BLUE )
		retval = wbkgd( stdscr, scrpair )
	endif

*  display the initial help message

	call message(HELPMSG)

*  display the program name ( CURS - CALC )

	if(has_colors()) then
		revpair = setpair( COLOR_BLACK, COLOR_WHITE )
		retval  = attron( revpair )
	else
		retval = standout()
	endif
	retval = mvaddstr( 5, 4,  '   ' )
	retval = mvaddstr( 6, 4,  ' C ' )
	retval = mvaddstr( 7, 4,  ' U ' )
	retval = mvaddstr( 8, 4,  ' R ' )
	retval = mvaddstr( 9, 4,  ' S ' )
	retval = mvaddstr( 10, 4, '   ' )
	retval = mvaddstr( 11, 4, ' | ' )
	retval = mvaddstr( 12, 4, '   ' )
	retval = mvaddstr( 13, 4, ' C ' )
	retval = mvaddstr( 14, 4, ' A ' )
	retval = mvaddstr( 15, 4, ' L ' )
	retval = mvaddstr( 16, 4, ' C ' )
	retval = mvaddstr( 17, 4, '   ' )
	if(has_colors()) then
		retval  = attroff( revpair )
	else
		retval = standend()
	endif

	retval = attron(A_BOLD)
	retval = mvaddstr( 20, 1, 'by')
	retval = mvaddstr( 21, 1, 'SCC')
	retval = mvaddstr( 22, 1, 'Software')
	retval = attroff(A_BOLD)

* create the calculator window

	calcwin = newwin(21, 32, 2, 10)
	if( calcwin .eq. NULL ) then
		retval = endwin()
		stop
	endif

* if the terminal has color capabilities, Use it

* if colors are not supported, set the calculator to inverse video

	if(has_colors()) then
		cpair = setpair( COLOR_BLACK, COLOR_YELLOW )
		retval = wbkgd( calcwin, cpair )
	else
		retval = wbkgd( calcwin, A_REVERSE )
	endif

* box it, with "double line" graphics if at386, other terminal types
* will inherit the single line definitions.

	retval = wborder( calcwin, ACS_DVLINE, ACS_DVLINE, ACS_DHLINE,
     .		ACS_DHLINE, ACS_DULCORNER, ACS_DURCORNER,
     .		ACS_DLLCORNER, ACS_DLRCORNER)

* create an internal box around the numeric keypad keys

	retval = rectangle(calcwin,6,9,16,31,0,0,0,0,0,0,0,0)
	retval = wmove( calcwin, 6, 31 )
	retval = waddch( calcwin, ACS_DRTEE )
	retval = wmove( calcwin, 16, 31 )
	retval = waddch( calcwin, ACS_DRTEE )
	retval = wnoutrefresh( calcwin )

* now create the display window ( actually two parts, one for the
*  number and one for any exponent values)
*	- make it a derived subwindow on the calculator

	mainfield = derwin( calcwin, 1, 19, 2, 3 )
	expfield = derwin( calcwin, 1, 7, 2, 22 )	

* depending on the color capabilities, try to differentiate different
* parts of the calculator.  The display fields will by white-on-black in
* any case.  All the calculator (keys) will have a particular attribute
* except for the key that is pressed or navigated to, which will have
* some attribute to highlight it.

	if( has_colors() ) then
	  fpair = setpair( COLOR_BLACK, COLOR_WHITE )
	  retval = wbkgd( mainfield, fpair )
	  retval = wbkgd( expfield, fpair )

*	  most keys will be black on cyan

	  keyattr = setpair( COLOR_BLACK, COLOR_CYAN )

	  hpair = setpair( COLOR_GREEN, COLOR_BLACK )
	  hotattr = b_or( hpair, A_BOLD )

	else

	  retval = wbkgd( mainfield, A_NORMAL )
	  retval = wbkgd( expfield, A_NORMAL )

*	  most keys will be normal

	  keyattr = A_NORMAL

*	  the key pressed or navigated to will be BOLD

	  hotattr = A_BOLD
	endif

*	now define windows for each of the calculator "buttons" (some map
*	to keyboard keys, some need to be navigated to)

*	all clear key
	AC = 1
	keywin(AC) = derwin( calcwin, 1, 3, 5, 4 )	
	retval = wbkgd( keywin(AC), hotattr )
	retval = waddstr( keywin(AC), ' AC' )
	retval = wnoutrefresh( keywin(AC) )
*	Backspace Key
	BS = 2
	keywin(BS) = derwin( calcwin, 1, 3, 5, 11 )	
	retval = wbkgd( keywin(BS), keyattr )
	retval = waddstr( keywin(BS), '<==' )
	retval = wnoutrefresh( keywin(BS) )
*	Modulus Key
	MD = 3
	keywin(MD) = derwin( calcwin, 1, 3, 5, 16 )	
	retval = wbkgd( keywin(MD), keyattr )
	retval = waddstr( keywin(MD), 'MOD' )
	retval = wnoutrefresh( keywin(MD) )
*	Truncate real number in X register to an integer
	TNI = 4
	keywin(TNI) = derwin( calcwin, 1, 3, 5, 21 )	
	retval = wbkgd( keywin(TNI), keyattr )
	retval = waddstr( keywin(TNI), 'INT' )
	retval = wnoutrefresh( keywin(TNI) )
*	Fix the precision of the display
	FIX = 5
	keywin(FIX) = derwin( calcwin, 1, 3, 5, 26 )	
	retval = wbkgd( keywin(FIX), keyattr )
	retval = waddstr( keywin(FIX), 'FIX' )
	retval = wnoutrefresh( keywin(FIX) )
*	clear key
	C = 6
	keywin(C) = derwin( calcwin, 1, 3, 7, 4 )	
	retval = wbkgd( keywin(C), keyattr )
	retval = waddstr( keywin(C), ' C ' )
	retval = wnoutrefresh( keywin(C) )
*	Memory in key
	STO = 7
	keywin(STO) = derwin( calcwin, 1, 3, 9, 4 )	
	retval = wbkgd( keywin(STO), keyattr )
	retval = waddstr( keywin(STO), 'STO' )
	retval = wnoutrefresh( keywin(STO) )
*	Memory recall key
	RCL = 8
	keywin(RCL) = derwin( calcwin, 1, 3, 11, 4 )	
	retval = wbkgd( keywin(RCL), keyattr )
	retval = waddstr( keywin(RCL), 'RCL' )
	retval = wnoutrefresh( keywin(RCL) )
*	PI key
	PIVAL = 9
	keywin(PIVAL) = derwin( calcwin, 1, 3, 13, 4 )	
	retval = wbkgd( keywin(PIVAL), keyattr )
	retval = waddstr( keywin(PIVAL), ' PI' )
	retval = wnoutrefresh( keywin(PIVAL) )
*	y^X key
	YRX = 10
	keywin(YRX) = derwin( calcwin, 1, 3, 15, 4 )	
	retval = wbkgd( keywin(YRX), keyattr )
	retval = waddstr( keywin(YRX), 'y^X' )
	retval = wnoutrefresh( keywin(YRX) )
*	Exchange the X and Y registers
	XXY = 11
	keywin(XXY) = derwin( calcwin, 1, 3, 17, 4 )	
	retval = wbkgd( keywin(XXY), keyattr )
	retval = waddstr( keywin(XXY), 'x:y' )
	retval = wnoutrefresh( keywin(XXY) )
*	GO Key - starts macro playback
	GO = 12
	keywin(GO) = derwin( calcwin, 1, 3, 19, 4 )	
	retval = wbkgd( keywin(GO), keyattr )
	retval = waddstr( keywin(GO), ' GO' )
	retval = wnoutrefresh( keywin(GO) )
*	Comma key -- toggle commas
	CMA = 13
	keywin(CMA) = derwin( calcwin, 1, 3, 19, 11 )	
	retval = wbkgd( keywin(CMA), keyattr )
	retval = waddstr( keywin(CMA), ' , ' )
	retval = wnoutrefresh( keywin(CMA) )
*	10-power (exponent) key
	EE = 14
	keywin(EE) = derwin( calcwin, 1, 3, 17, 11 )	
	retval = wbkgd( keywin(EE), keyattr )
	retval = waddstr( keywin(EE), ' EE' )
	retval = wnoutrefresh( keywin(EE) )
*	Change sign key
	CHS = 15
	keywin(CHS) = derwin( calcwin, 1, 3, 17, 16 )	
	retval = wbkgd( keywin(CHS), keyattr )
	retval = waddstr( keywin(CHS), '+\-' )
	retval = wnoutrefresh( keywin(CHS) )
*	Left Parenthesis
	LP = 16
	keywin(LP) = derwin( calcwin, 1, 3, 17, 21 )	
	retval = wbkgd( keywin(LP), keyattr )
	retval = waddstr( keywin(LP), ' ( ' )
	retval = wnoutrefresh( keywin(LP) )
*	Right Parenthesis
	RP = 17
	keywin(RP) = derwin( calcwin, 1, 3, 17, 26 )	
	retval = wbkgd( keywin(RP), keyattr )
	retval = waddstr( keywin(RP), ' ) ' )
	retval = wnoutrefresh( keywin(RP) )
*	Last X Key
	LSX = 18
	keywin(LSX) = derwin( calcwin, 1, 3, 19, 16 )	
	retval = wbkgd( keywin(LSX), keyattr )
	retval = waddstr( keywin(LSX), 'LSX' )
	retval = wnoutrefresh( keywin(LSX) )
*	Real exchange with Imaginary Register
	RXI = 19
	keywin(RXI) = derwin( calcwin, 1, 3, 19, 21 )	
	retval = wbkgd( keywin(RXI), keyattr )
	retval = waddstr( keywin(RXI), 'r|i' )
	retval = wnoutrefresh( keywin(RXI) )
*	semi-colon (;) indicates than entry in the imaginary register should
*			begin
	SEMI = 20
	keywin(SEMI) = derwin( calcwin, 1, 3, 19, 26 )	
	retval = wbkgd( keywin(SEMI), keyattr )
	retval = waddstr( keywin(SEMI), ' ; ' )
	retval = wnoutrefresh( keywin(SEMI) )

*	The Following keys should be on most numeric keypads, layout
*	is that of AT101 keyboard numeric keypad

*	Divide (/) key
	DIV = 21
	keywin(DIV) = derwin( calcwin, 1, 3, 7, 16 )	
	retval = wbkgd( keywin(DIV), keyattr )
	retval = waddstr( keywin(DIV), ' / ' )
	retval = wnoutrefresh( keywin(DIV) )
*	Multiply, as in "Be fruitful and" (*) key
	MULT = 22
	keywin(MULT) = derwin( calcwin, 1, 3, 7, 21 )	
	retval = wbkgd( keywin(MULT), keyattr )
	retval = waddstr( keywin(MULT), ' * ' )
	retval = wnoutrefresh( keywin(MULT) )
*	Minus (-) key
	MINUS = 23
	keywin(MINUS) = derwin( calcwin, 1, 3, 7, 26 )	
	retval = wbkgd( keywin(MINUS), keyattr )
	retval = waddstr( keywin(MINUS), ' - ' )
	retval = wnoutrefresh( keywin(MINUS) )
*	Seven (7) key
	SEVEN = 24
	keywin(SEVEN) = derwin( calcwin, 1, 3, 9, 11 )	
	retval = wbkgd( keywin(SEVEN), keyattr )
	retval = waddstr( keywin(SEVEN), ' 7 ' )
	retval = wnoutrefresh( keywin(SEVEN) )
*	Eight (8) key
	EIGHT = 25
	keywin(EIGHT) = derwin( calcwin, 1, 3, 9, 16 )	
	retval = wbkgd( keywin(EIGHT), keyattr )
	retval = waddstr( keywin(EIGHT), ' 8 ' )
	retval = wnoutrefresh( keywin(EIGHT) )
*	Nine (9) key
	NINE = 26
	keywin(NINE) = derwin( calcwin, 1, 3, 9, 21 )	
	retval = wbkgd( keywin(NINE), keyattr )
	retval = waddstr( keywin(NINE), ' 9 ' )
	retval = wnoutrefresh( keywin(NINE) )
*	Plus (+) key
	PLUS = 27
	keywin(PLUS) = derwin( calcwin, 3, 3, 9, 26 )	
	retval = wbkgd( keywin(PLUS), keyattr )
	retval = mvwaddstr( keywin(PLUS), 1, 0, ' + ' )
	retval = wnoutrefresh( keywin(PLUS) )
	PLUS2 = 28
	keywin(PLUS2) = derwin( calcwin, 3, 3, 9, 26 )	
	retval = wbkgd( keywin(PLUS2), keyattr )
	retval = mvwaddstr( keywin(PLUS2), 1, 0, ' + ' )
	retval = wnoutrefresh( keywin(PLUS2) )
*	Four (4) key
	FOUR = 29
	keywin(FOUR) = derwin( calcwin, 1, 3, 11, 11 )	
	retval = wbkgd( keywin(FOUR), keyattr )
	retval = waddstr( keywin(FOUR), ' 4 ' )
	retval = wnoutrefresh( keywin(FOUR) )
*	Five (5) key
	FIVE = 30
	keywin(FIVE) = derwin( calcwin, 1, 3, 11, 16 )	
	retval = wbkgd( keywin(FIVE), keyattr )
	retval = waddstr( keywin(FIVE), ' 5 ' )
	retval = wnoutrefresh( keywin(FIVE) )
*	Six (6) key
	SIX = 31
	keywin(SIX) = derwin( calcwin, 1, 3, 11, 21 )	
	retval = wbkgd( keywin(SIX), keyattr )
	retval = waddstr( keywin(SIX), ' 6 ' )
	retval = wnoutrefresh( keywin(SIX) )
*	One (1) key
	ONE = 32
	keywin(ONE) = derwin( calcwin, 1, 3, 13, 11 )	
	retval = wbkgd( keywin(ONE), keyattr )
	retval = waddstr( keywin(ONE), ' 1 ' )
	retval = wnoutrefresh( keywin(ONE) )
*	Two (2) key
	TWO = 33
	keywin(TWO) = derwin( calcwin, 1, 3, 13, 16 )	
	retval = wbkgd( keywin(TWO), keyattr )
	retval = waddstr( keywin(TWO), ' 2 ' )
	retval = wnoutrefresh( keywin(TWO) )
*	Three (3) key
	THREE = 34
	keywin(THREE) = derwin( calcwin, 1, 3, 13, 21 )	
	retval = wbkgd( keywin(THREE), keyattr )
	retval = waddstr( keywin(THREE), ' 3 ' )
	retval = wnoutrefresh( keywin(THREE) )
*	Zero (0) key
	ZERO = 35
	keywin(ZERO) = derwin( calcwin, 1, 8, 15, 11 )	
	retval = wbkgd( keywin(ZERO), keyattr )
	retval = waddstr( keywin(ZERO), '   0    ' )
	retval = wnoutrefresh( keywin(ZERO) )
*	Zero2 (0) key
	ZERO2 = 36
	keywin(ZERO2) = derwin( calcwin, 1, 8, 15, 11 )	
	retval = wbkgd( keywin(ZERO2), keyattr )
	retval = waddstr( keywin(ZERO2), '   0    ' )
	retval = wnoutrefresh( keywin(ZERO2) )
*	Decimal Point (.) key
	DP = 37
	keywin(DP) = derwin( calcwin, 1, 3, 15, 21 )	
	retval = wbkgd( keywin(DP), keyattr )
	retval = waddstr( keywin(DP), ' . ' )
	retval = wnoutrefresh( keywin(DP) )
*	Enter or Equal Key
	EQUAL = 38
	keywin(EQUAL) = derwin( calcwin, 3, 3, 13, 26 )	
	retval = wbkgd( keywin(EQUAL), keyattr )
	retval = mvwaddch( keywin(EQUAL), 1, 1, ICHAR('=') )
	retval = wnoutrefresh( keywin(EQUAL) )
	EQUAL2 = 39
	keywin(EQUAL2) = derwin( calcwin, 3, 3, 13, 26 )	
	retval = wbkgd( keywin(EQUAL2), keyattr )
	retval = mvwaddch( keywin(EQUAL2), 1, 1, ICHAR('=') )
	retval = wnoutrefresh( keywin(EQUAL2) )

* define where to go from one key to the next

	up(AC) = GO
	down(AC) = C
	left(AC) = FIX
	right(AC) = BS

	up(BS) = CMA
	down(BS) = SEVEN
	left(BS) = AC
	right(BS) = MD

	up(MD) = LSX
	down(MD) = DIV
	left(MD) = BS
	right(MD) = TNI

	up(TNI) = RXI
	down(TNI) = MULT
	left(TNI) = MD
	right(TNI) = FIX

	up(FIX) = SEMI
	down(FIX) = MINUS
	left(FIX) = TNI
	right(FIX) = AC
	
	up(C) = AC
	down(C) = STO
	left(C) = MINUS
	right(C) = DIV
	
	up(STO) = C
	down(STO) = RCL
	left(STO) = PLUS
	right(STO) = SEVEN
	
	up(RCL) = STO
	down(RCL) = PIVAL
	left(RCL) = PLUS2
	right(RCL) = FOUR
	
	up(PIVAL) = RCL
	down(PIVAL) = YRX
	left(PIVAL) = EQUAL
	right(PIVAL) = ONE
	
	up(YRX) = PIVAL
	down(YRX) = XXY
	left(YRX) = EQUAL2
	right(YRX) = ZERO
	
	up(XXY) = YRX
	down(XXY) = GO
	left(XXY) = RP
	right(XXY) = EE
	
	up(GO) = XXY
	down(GO) = AC
	left(GO) = SEMI
	right(GO) = CMA
	
	up(EE) = ZERO
	down(EE) = CMA
	left(EE) = XXY
	right(EE) = CHS
	
	up(CHS) = ZERO2
	down(CHS) = LSX
	left(CHS) = EE
	right(CHS) = LP
	
	up(LP) = DP
	down(LP) = RXI
	left(LP) = CHS
	right(LP) = RP
	
	up(RP) = EQUAL2
	down(RP) = SEMI
	left(RP) = LP
	right(RP) = XXY

	up(LSX)    = CHS
	down(LSX)  = MD
	left(LSX)  = CMA
	right(LSX) = RXI

	up(CMA)    = EE
	down(CMA)  = BS
	left(CMA)  = GO
	right(CMA) = LSX

	up(RXI)    = LP
	down(RXI)  = TNI
	left(RXI)  = LSX
	right(RXI) = SEMI

	up(SEMI)    = RP
	down(SEMI)  = FIX
	left(SEMI)  = RXI
	right(SEMI) = GO
	
	up(DIV) = MD
	down(DIV) = EIGHT
	left(DIV) = C
	right(DIV) = MULT
	
	up(MULT) = TNI
	down(MULT) = NINE
	left(MULT) = DIV
	right(MULT) = MINUS
	
	up(MINUS) = FIX
	down(MINUS) = PLUS
	left(MINUS) = MULT
	right(MINUS) = C
	
	up(SEVEN) = BS
	down(SEVEN) = FOUR
	left(SEVEN) = STO
	right(SEVEN) = EIGHT
	
	up(EIGHT) = DIV
	down(EIGHT) = FIVE
	left(EIGHT) = SEVEN
	right(EIGHT) = NINE

	up(NINE) = MULT
	down(NINE) = SIX
	left(NINE) = EIGHT
	right(NINE) = PLUS
	
	up(PLUS) = MINUS
	down(PLUS) = EQUAL
	left(PLUS) = NINE
	right(PLUS) = STO

	up(PLUS2) = MINUS
	down(PLUS2) = EQUAL
	left(PLUS2) = SIX
	right(PLUS2) = RCL

	up(FOUR) = SEVEN
	down(FOUR) = ONE
	left(FOUR) = RCL
	right(FOUR) = FIVE
	
	up(FIVE) = EIGHT
	down(FIVE) = TWO
	left(FIVE) = FOUR
	right(FIVE) = SIX
	
	up(SIX) = NINE
	down(SIX) = THREE
	left(SIX) = FIVE
	right(SIX) = PLUS2
	
	up(ONE) = FOUR
	down(ONE) = ZERO
	left(ONE) = PIVAL
	right(ONE) = TWO
	
	up(TWO) = FIVE
	down(TWO) = ZERO2
	left(TWO) = ONE
	right(TWO) = THREE
	
	up(THREE) = SIX
	down(THREE) = DP
	left(THREE) = TWO
	right(THREE) = EQUAL
	
	up(EQUAL) = PLUS2
	down(EQUAL) = RP
	left(EQUAL) = THREE
	right(EQUAL) = PIVAL
	
	up(EQUAL2) = PLUS2
	down(EQUAL2) = RP
	left(EQUAL2) = DP
	right(EQUAL2) = YRX
	
	up(ZERO) = ONE
	down(ZERO) = EE
	left(ZERO) = YRX
	right(ZERO) = DP
	
	up(ZERO2) = TWO
	down(ZERO2) = CHS
	left(ZERO2) = YRX
	right(ZERO2) = DP
	
	up(DP) = THREE
	down(DP) = LP
	left(DP) = ZERO2
	right(DP) = EQUAL2

*  have 8 to 10 windows at the bottom of the screen mapped to the function
*  keys

	if( has_colors() ) then
	  fnpair = setpair( COLOR_WHITE, COLOR_RED )
	  fnattr = fnpair
	  fhpair = setpair( COLOR_RED, COLOR_WHITE )
	  fhattr = fhpair
	else
	  fnattr = A_REVERSE
	  fhattr = A_BOLD
	endif

	FKEY1 = newwin( 1,6, LINES-1, 2 )
	retval = wbkgd( FKEY1, fnattr )
	retval = waddstr( FKEY1, ' HELP ' )
	retval = wnoutrefresh( FKEY1 )

	FKEY2 = newwin( 1,6, LINES-1, 9 )
	retval = wbkgd( FKEY2, fnattr )
	retval = waddstr( FKEY2, ' MATH ' )
	retval = wnoutrefresh( FKEY2 )

	FKEY3 = newwin( 1,6, LINES-1, 16 )
	retval = wbkgd( FKEY3, fnattr )
	retval = waddstr( FKEY3, 'STATS.' )
	retval = wnoutrefresh( FKEY3 )

	FKEY4 = newwin( 1,6, LINES-1, 23 )
	retval = wbkgd( FKEY4, fnattr )
	retval = waddstr( FKEY4, 'CNVERT' )
	retval = wnoutrefresh( FKEY4 )

	FKEY5 = newwin( 1,6, LINES-1, 32 )
	retval = wbkgd( FKEY5, fnattr )
	retval = waddstr( FKEY5, 'CONST.' )
	retval = wnoutrefresh( FKEY5 )

	FKEY6 = newwin( 1,6, LINES-1, 39 )
	retval = wbkgd( FKEY6, fnattr )
	retval = waddstr( FKEY6, 'NR/SCI' )
	retval = wnoutrefresh( FKEY6 )

	FKEY7 = newwin( 1,6, LINES-1, 46 )
	retval = wbkgd( FKEY7, fnattr )
	retval = waddstr( FKEY7, ' DRG  ' )
	retval = wnoutrefresh( FKEY7 )

	FKEY8 = newwin( 1,6, LINES-1, 53 )
	retval = wbkgd( FKEY8, fnattr )
	retval = waddstr( FKEY8, 'AL/RPN' )
	retval = wnoutrefresh( FKEY8 )

	FKEY9 = newwin( 1,6, LINES-1, 62 )
	retval = wbkgd( FKEY9, fnattr )
	retval = waddstr( FKEY9, 'CMPLEX' )
	retval = wnoutrefresh( FKEY9 )

	FKEY10 = newwin( 1,6, LINES-1, 69 )
	retval = wbkgd( FKEY10, fnattr )
	retval = waddstr( FKEY10, 'RECORD' )
	retval = wnoutrefresh( FKEY10 )


*  make the All-clear key the initial pressed key.

	hotkey = AC
	oldkey = ZERO

	retval = move(0,0)
	retval = wnoutrefresh( stdscr )
	retval = doupdate()
********************************
* initialize the state of the calculator

	numinit = .true.
	numpress = .false.
	eepress = .false.
	scinot = .false.
	radflag = 1
	action = .false.
	rpnmode = .false.
	compmode = .false.
	compentry = .false.
	commas = .true.
	memopt = 0
	mempoint = 0
	functopt = 0
	rclopt = 0
	errstate = 0
	seed = 1353

* assign some numeric values to variables used in tests for various
* arithmetic actions.

	PLUSACT  = 1
	MINUSACT = 2
	MULTACT  = 3
	DIVACT   = 4
	EQUALACT = 5
	CLOSEACT = 6
	POWRACT  = 7
	MODACT   = 8

* clear all registers used for parenthesis operations

	reglev = 1
	do i = 1,MAXREGS
		do j = 1,2
		   regval(i,j) = 0.
		   regvalmd(i,j) = 0.
		   regvalpw(i,j) = 0.
		end do
		regact(i) = 0
		regactmd(i) = 0
		regactpw(i) = 0
	end do

	xstat  = 0.
	x2stat = 0.
	ystat  = 0.
	y2stat = 0.
	xystat = 0.
	nstat  = 0.

* initial value of the display is zero, memory registers zero

	screenval = 0.
	screenval_i = 0.
	lastx   = 0.
	lastx_i = 0.
	do i = 1,20
		do j = 1,2
		   memstore(i,j) = 0.
		end do
	end do

	lastpop = 0.
	lastpop_i = 0.

	lastitem = 1
	lastconv = 1
	laststat = 1
	lastconst = 1
	convdir = 1
	precision = -1

	recording = 0
	playback  = 0
	macitem   = 1
	runitem   = 1
	do i = 1,10
		totstrokes(i) = 0
		macname(i) = ' '
		do j = 1,MAXMAC
			keymac(i,j) = 0
		end do
	end do
	answer(1) = 'No'
	answer(2) = 'Yes'
	answer(3) = 'Clear'


*	read calculator state from a hidden file in the user's home
*	directory ($HOME).  These values must be stored in identical order
*	when the program is exited.

	call getenv( 'HOME', filenm )
	ie = lastchr( filenm )
	if(ie .le. 0) then
	  filenm = 'c:\cclcdat'
	else
	  if( filenm(ie:ie) .eq. '/' ) ie = ie - 1
	  filenm = filenm(:ie)//'/.cld'
	endif

	open( 32, file=filenm,status='old',form='unformatted',
&		access='direct', recl=16, err=50 )
	irec = 1

	read( 32, rec=irec, end=45, err=45 ) screenval,screenval_i
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) scinot
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) radflag
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) rpnmode
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) precision
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) reglev
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) seed
	irec = irec + 1
	do i = 1, 20
		read( 32, rec=irec, end=45, err=45 ) (memstore(i,j),j=1,2)
		irec = irec + 1
	end do
	do i = 1, MAXREGS
		read( 32, rec=irec, end=45, err=45 ) (regval(i,j),j=1,2)
		irec = irec + 1
		read( 32, rec=irec, end=45, err=45 ) (regvalmd(i,j),j=1,2)
		irec = irec + 1
		read( 32, rec=irec, end=45, err=45 ) (regvalpw(i,j),j=1,2)
		irec = irec + 1
		read( 32, rec=irec, end=45, err=45 ) regact(i)
		irec = irec + 1
		read( 32, rec=irec, end=45, err=45 ) regactmd(i)
		irec = irec + 1
		read( 32, rec=irec, end=45, err=45 ) regactpw(i)
		irec = irec + 1
	end do
	read( 32, rec=irec, end=45, err=45 ) commas
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) xstat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) x2stat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) ystat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) y2stat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) xystat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) nstat
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) compmode
	irec = irec + 1
	read( 32, rec=irec, end=45, err=45 ) lastx, lastx_i
	irec = irec + 1
	do i = 1, 10
	   read( 32, rec=irec, end=45, err=45 ) macname(i)
	   irec = irec + 1
	   read( 32, rec=irec, end=45, err=45 ) totstrokes(i), macmode(i)
	   irec = irec + 1
	   do j = 1,MAXMAC/4
	      read( 32, rec=irec, end=45, err=45 )
&			(keymac(i,j+(k*MAXMAC/4)), k=0,3)
	      irec = irec + 1
	   end do
	end do

   45	close(32)

   50	if( rpnmode ) then
	  action = .true.
	  retval = mvwaddstr(keywin(C),0,0,'CLX')
	  retval = mvwaddstr(keywin(LP),0,0,' UP')
	  retval = mvwaddstr(keywin(RP),0,0,' DN')
	  retval = mvwaddstr(keywin(EQUAL),0,1,'E')
	  retval = mvwaddstr(keywin(EQUAL),1,1,'n')
	  retval = mvwaddstr(keywin(EQUAL),2,1,'t')
	  retval = wnoutrefresh( keywin(C) )
	  retval = wnoutrefresh( keywin(LP) )
	  retval = wnoutrefresh( keywin(RP) )
	  retval = wnoutrefresh( keywin(EQUAL) )
	endif

	retval = wnoutrefresh( mainfield )
	retval = wnoutrefresh( expfield )

	mainfield_i = derwin( calcwin, 1, 19, 3, 3 )
	expfield_i = derwin( calcwin, 1, 7, 3, 22 )	
	if( compmode ) then
		if( has_colors() ) then
		  retval = wbkgd( mainfield_i, fpair )
		  retval = wbkgd( expfield_i, fpair )
		else
		  retval = wbkgd( mainfield_i, A_NORMAL )
		  retval = wbkgd( expfield_i, A_NORMAL )
		endif
		retval = wnoutrefresh( mainfield_i )
		retval = wnoutrefresh( expfield_i )
	endif

* display the initial value in the x register

     	retval = displayval( screenval, screenval_i )

	retval = move(0,0)
	retval = refresh()

********************************
*    define the spacebar key

	KEY_SB = ichar(' ')

*  main command loop -- get user key strokes

	do while (.true.)

		if( numinit ) then
			compentry = .false.
		endif

		if( memopt .gt. 0 ) then
		  memopt = memopt + 1
		  if( memopt .gt. 3 ) then
		    memopt = 0
		  endif
		endif
		if( rclopt .gt. 0 ) then
		  rclopt = rclopt + 1
		  if( rclopt .gt. 3 ) then
		    rclopt = 0
		  endif
		endif

*		display how many levels of parenthesis have been invoked.

		if( rpnmode ) then
		  retval = mvwaddstr(calcwin,1,18,'RPN     ')
		else
		  retval = mvwaddstr(calcwin,1,18,'        ')
		  if( reglev .gt. 1 ) then
			retval = mvwaddstr(calcwin, 1, 18, '(' )
			write(tmpstr,'(i2)') reglev - 1
			retval = mvwaddstr(calcwin, 1, 20, tmpstr(:2) )
		  else
			retval = mvwaddstr(calcwin, 1, 18, '    ' )
		  endif
		endif

		if( scinot ) then
			retval = mvwaddstr(calcwin, 1, 3, 'Sci' )
		else
			if( precision .lt. 0 ) then
				retval = mvwaddstr(calcwin, 1, 3, 'Nrm' )
			else
				retval = mvwaddstr(calcwin, 1, 3, 'Fix' )
			endif
		endif
		if( radflag .eq. 1 ) then
			retval = mvwaddstr(calcwin, 1, 8, 'Deg' )
		elseif( radflag .eq. 2 ) then
			retval = mvwaddstr(calcwin, 1, 8, 'Rad' )
		else
			retval = mvwaddstr(calcwin, 1, 8, 'Gra' )
		endif

		if( compmode ) then
			retval = mvwaddstr(calcwin, 1, 13, 'CMP' )
			retval = mvwaddstr(calcwin, 2, 2, 'r' )
			retval = mvwaddstr(calcwin, 3, 2, 'i' )
			if( compentry ) then
			  retval = mvwaddstr(calcwin, 2, 29, ' ' )
			  retval = mvwaddstr(calcwin, 3, 29, '<' )
			else
			  retval = mvwaddstr(calcwin, 2, 29, '<' )
			  retval = mvwaddstr(calcwin, 3, 29, ' ' )
			endif
		else
			retval = mvwaddstr(calcwin, 1, 13, '   ' )
			retval = mvwaddstr(calcwin, 2, 2, ' ' )
			retval = mvwaddstr(calcwin, 3, 2, ' ' )
			retval = mvwaddstr(calcwin, 2, 29, ' ' )
			retval = mvwaddstr(calcwin, 3, 29, ' ' )
		endif

		if( recording .gt. 0 ) then
		  tmpmsg = 'RECORDING -- ' // macname(recording) //
&		  	'  Press <F10> Again to End Recording.'
		elseif( playback .gt. 0 ) then
		  if( macstep ) then
		    write(tmpmsg,'(A,I3,A)') 'STEP PLAYBACK -- ',
&		      keycount,'  >> Press <Spacebar> to step through macro.'
		  else
		    tmpmsg = 'RUNNING -- '//macname(playback)
		  endif
		else
			  tmpmsg = HELPMSG
		endif
		call message( tmpmsg(:lastchr(tmpmsg)) )

*		many of the functions called in the command loop make
*		modifications to the calculator or the display fields without
*		doing a refresh, cover that here.

		retval = wnoutrefresh( calcwin )
		retval = wnoutrefresh( mainfield )
		retval = wnoutrefresh( expfield )
		retval = wnoutrefresh( mainfield_i )
		retval = wnoutrefresh( expfield_i )
		retval = move(0,0)
		retval = wnoutrefresh( stdscr )

*		get command from user (from stdscr) and process it

  100		retval = doupdate()

		if( playback .gt. 0 ) then
			if( macstep ) then
			  if( sgetch() .ne. KEY_SB ) then
			    playback = 0
			    rpnmode  = holdmode
			    retval   = beep()
			    go to 1234
			  endif
			endif
			ch = keymac(playback, keycount)
			if( ch .ge. 10000 ) then
			  if( ch/10000 .eq. KEY_F(4) ) then
			    convdir = mod( ch, 100 )
			    ichosen = mod( ch-convdir, 10000 )/100
			  else
			    ichosen = mod( ch, 10000 ) / 100
			  endif
			  ch = ch/10000
			endif
		else
	  		ch = sgetch()
	  	endif

		if( functopt .eq. 1 ) then
		  functopt = 0
		  if( ch .ge. 48 .and. ch .le. 57 ) then
		    if( ch .eq. 48 ) then
		    	ch = KEY_F(10)
		    else
		    	ch = KEY_F( ch-48 )
		    endif
		    if( recording .gt. 0 ) then
		    	keycount = keycount - 1
		    endif
		  else
		  	retval = beep()
		  	go to 1234
		  endif
		endif

  500		continue


* function keys can be emulated with a <ctrl>-f followed by the number
*	0 - 9
		if ( ch .eq. CTRL('F') ) then

			functopt = 1

* arrow keys move the highlighted calculator key
		elseif( ch .eq. KEY_RIGHT ) then
			retval = lightpad( hotkey, oldkey, right(hotkey) )
			go to 100
		elseif( ch .eq. KEY_LEFT ) then
			retval = lightpad( hotkey, oldkey, left(hotkey) )
			go to 100
		elseif( ch .eq. KEY_UP ) then
			retval = lightpad( hotkey, oldkey, up(hotkey) )
			go to 100
		elseif( ch .eq. KEY_DOWN ) then
			retval = lightpad( hotkey, oldkey, down(hotkey) )
			go to 100

* all clear
		elseif( (ch .eq. ichar('a') .or. ch .eq. ichar('A')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. AC) ) then

			retval = lightpad( hotkey, oldkey, AC )
			ch = ichar('a')

			if( rpnmode ) then
			  retval = displayval( dble(0.), dble(0.) )
			else
			  screenval = 0.
			  screenval_i = 0.
			  retval = wjustify( mainfield, '0.', 2, .FALSE., .TRUE.)
			  retval = wjustify( expfield, '      ', 2, .FALSE., .TRUE.)
			  if( compmode ) then
			   retval = wjustify( mainfield_i, '0.', 2, .FALSE., .TRUE.)
			   retval = wjustify( expfield_i, '      ', 2, .FALSE., .TRUE.)
			  endif
			endif
			numinit = .true.
			action = .false.
			reglev = 1
			regact(1) = 0
			do i = 1, MAXREGS
			   do j = 1,2
				regval(i,j) = 0.
			   end do
			end do
			regactmd(1) = 0
			regactpw(1) = 0
			rclopt = 0
			memopt = 0

* clear x
		elseif( (ch .eq. ichar('c') .or. ch .eq. ichar('C')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. C) ) then

			retval = lightpad( hotkey, oldkey, C )
			ch = ichar('c')
			if((.not. action) .or. rpnmode) then
			  if(rpnmode) then
			    retval = displayval( dble(0.), dble(0.) )
			  else
			    screenval   = 0.
			    screenval_i = 0.
			    retval = wjustify( mainfield, '0.', 2, .FALSE., .TRUE.)
			    retval = wjustify( expfield, '      ', 2, .FALSE., .TRUE.)
			    if( compmode ) then
			      retval = wjustify( mainfield_i, '0.', 2, .FALSE., .TRUE.)
			      retval = wjustify( expfield_i, '      ', 2,
&							.FALSE., .TRUE.)
			    endif
			  endif
			  rclopt = 0
			  memopt = 0
			  numinit = .true.
			  if(rpnmode) then
			    action = .false.
			  endif
			endif

* begin "Store" operation
		elseif( (ch .eq. ichar('s') .or. ch .eq. ichar('S')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. STO) ) then

			retval = lightpad( hotkey, oldkey, STO )
			ch = ichar('s')
			numinit = .true.
			memopt = 1
			mempoint = 0
			memact = 0
			rclopt = 0

* begin recall operation
		elseif( (ch .eq. ichar('r') .or. ch .eq. ichar('R')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. RCL) ) then

			retval = lightpad( hotkey, oldkey, RCL )
			ch = ichar('r')
			rclopt = 1
			memopt = 0
			mempoint = 0
			memact = 0
			numinit = .true.
			if( .not. rpnmode ) then
			  action = .false.
			endif

* display the value of PI
		elseif( (ch .eq. ichar('p') .or. ch .eq. ichar('P')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. PIVAL) ) then

			retval = lightpad( hotkey, oldkey, PIVAL )
			ch = ichar('p')
			call liftstack
			if(rpnmode) then
				action = .true.
			else
				action = .false.
			endif
			numinit = .true.
			tmpval = asin(1.0D0)*2.0D0
			retval = displayval( tmpval, dble(0.) )

* begin a y^x (power) operation
		elseif( (ch .eq. ichar('y') .or. ch .eq. ichar('Y')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. YRX) ) then

			retval = lightpad( hotkey, oldkey, YRX )
			ch = ichar('y')
			call arithmetic(POWRACT)

* exchange the "x" and "y" memory registers (x being the screen value)
		elseif( (ch .eq. ichar('x') .or. ch .eq. ichar('X')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. XXY) ) then

			retval = lightpad( hotkey, oldkey, XXY )
			ch = ichar('x')
			tmpval   = regval(1,1)
			tmpval_i = regval(1,2)
			regval(1,1) = screenval
			regval(1,2) = screenval_i
			retval = displayval( tmpval, tmpval_i )
			numinit = .true.
			if( rpnmode ) then
				action = .true.
			else
				action = .false.
			endif

*  let the user specify the decimal precision of the display
		elseif( (ch .eq. ichar('f') .or. ch .eq. ichar('F')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. FIX) ) then


			retval = lightpad( hotkey, oldkey, FIX )
			ch = ichar('f')
			retval = move(0,0)
			retval = clrtoeol()
			retval = addstr('Enter decimal precision'//
&			' (just press <Enter> for floating precision): ')
			retval = echo()
			retval = wgetnstr( stdscr, tmpstr, 7 )
			retval = noecho()
			ll = lastchr( tmpstr )
			if( ll .le. 0 ) then
			    precision = -1
			else
			  read( tmpstr, '(I7)' , err=333 ) precision
			  if(precision .gt. 11) precision = -1
			endif
			go to 335
  333			precision = -1
  335			call message( tmpmsg(:lastchr(tmpmsg)) )
			retval = displayval( screenval, screenval_i )
			numinit = .true.

* convert the x value to an integer
		elseif( (ch .eq. ichar('i') .or. ch .eq. ichar('I')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. TNI) ) then

			retval = lightpad( hotkey, oldkey, TNI )
			ch = ichar('i')
			lastx   = screenval
			lastx_i = screenval_i
			screenval = int( screenval )
			retval = displayval( screenval, screenval_i )
			numinit = .true.
			if( rpnmode ) then
				action = .true.
			else
				action = .false.
			endif

* "e" tells the calculator that the user is starting to enter the 
*     exponent of a number
		elseif( (ch .eq. ichar('e') .or. ch .eq. ichar('E')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. EE) ) then

			retval = lightpad( hotkey, oldkey, EE )
			ch = ichar('e')
			if( (numpress .and. (.not. eepress)) .or. numinit .or.
&			    (compentry .and. (.not. eepress)) ) then
			  eepress = .true.
			  if( compentry ) then
			    retval = wjustify( expfield_i, 'ee+00', 2, .FALSE., .TRUE.)
			    if( .not. numpress ) then
			      retval = wjustify( mainfield_i, '1.', 2, .FALSE., .TRUE.)
			      numpress = .true.
			      dpoint = .false.
			    endif
			  else
			    retval = wjustify( expfield, 'ee+00', 2, .FALSE., .TRUE.)
			  endif
			  if( numinit ) then
			    call liftstack
			    retval = wjustify( mainfield, '1.', 2, .FALSE., .TRUE.)
			    call getval( screenval )
			    numinit = .false.
			    if( rpnmode ) then
			      action = .true.
			    else
			      action = .false.
			    endif
			    dpoint = .false.
			    numpress = .true.
			  endif
			endif

* change the sign of a number
		elseif( (ch .eq. ichar('\') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. CHS) ) then

			retval = lightpad( hotkey, oldkey, CHS )
			ch = ichar('\')

			if( numinit ) then
				retval = displayval( -screenval, screenval_i )
			else
				call number('\')
			endif

* begin parenthesis in algebraic mode or roll the stack up if in
* RPN mode.
		elseif( (ch .eq. ichar('(') ) .or. (rpnmode .and.
&			(ch .eq. ichar('u') .or. ch .eq. ichar('U')))
&			.or. (ch .eq. KEY_SB .and. hotkey .eq. LP) ) then

			retval = lightpad( hotkey, oldkey, LP )
			ch = ichar('(')
			if( rpnmode ) then
				numinit = .true.
				action = .true.
				tmpval = screenval
				tmpval_i = screenval_i
				screenval = regval(MAXREGS,1)
				screenval_i = regval(MAXREGS,2)
				do i = MAXREGS, 2, -1
				   regval(i,1) = regval(i-1,1)
				   regval(i,2) = regval(i-1,2)
				end do
				regval(1,1) = tmpval
				regval(1,2) = tmpval_i
				retval = displayval( screenval, screenval_i )
			else
				if( reglev .eq. MAXREGS ) then
				  retval = beep()
				else
				  numinit = .true.
				  action = .false.
				  reglev = reglev + 1
				  do i = reglev, 2, -1
				     do j = 1,2
				        regval(i,j) = regval(i-1,j)
				        regvalmd(i,j) = regvalmd(i-1,j)
				        regvalpw(i,j) = regvalpw(i-1,j)
				     end do
				     regact(i) = regact(i-1)
				     regactmd(i) = regactmd(i-1)
				     regactpw(i) = regactpw(i-1)
				  end do
				  regact(1) = 0
				  regval(1,1) = 0.
				  regval(1,2) = 0.
				  regactmd(1) = 0
				  regactpw(1) = 0
				  rclopt = 0
				  memopt = 0
				  tmpval   = 0.
				  tmpval_i = 0.
				  retval = displayval( tmpval, tmpval_i )
				endif
			endif

* closing parenthesis if in algebraic or role stack down
		elseif( (ch .eq. ichar(')') ) .or. (rpnmode .and.
&			(ch .eq. ichar('d') .or. ch .eq. ichar('D')))
&			.or. (ch .eq. KEY_SB .and. hotkey .eq. RP) ) then

			retval = lightpad( hotkey, oldkey, RP )
			ch = ichar(')')
			call arithmetic(CLOSEACT)

* divide operation
		elseif( (ch .eq. ichar('/') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. DIV) ) then

			retval = lightpad( hotkey, oldkey, DIV )
			ch = ichar('/')
			call arithmetic(DIVACT)

* modulus operation
		elseif( (ch .eq. ichar('m') .or. ch .eq. ichar('M') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. MD) ) then

			retval = lightpad( hotkey, oldkey, MD )
			ch = ichar('m')
			if( compmode ) then
			  retval = beep()
			else
			  call arithmetic(MODACT)
			endif

* multiplication operation
		elseif( (ch .eq. ichar('*') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. MULT) ) then

			retval = lightpad( hotkey, oldkey, MULT )
			ch = ichar('*')
			call arithmetic(MULTACT)

* subtraction operation
		elseif( (ch .eq. ichar('-') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. MINUS) ) then

			retval = lightpad( hotkey, oldkey, MINUS )
			ch = ichar('-')
			call arithmetic(MINUSACT)

* the number 7
		elseif( (ch .eq. ichar('7') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. SEVEN) ) then

			retval = lightpad( hotkey, oldkey, SEVEN )
			ch = ichar('7')
			call number('7')

* the number 8
		elseif( (ch .eq. ichar('8') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. EIGHT) ) then

			retval = lightpad( hotkey, oldkey, EIGHT )
			ch = ichar('8')
			call number('8')

* the number 9
		elseif( (ch .eq. ichar('9') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. NINE) ) then

			retval = lightpad( hotkey, oldkey, NINE )
			ch = ichar('9')
			call number('9')

* addition operation
		elseif( (ch .eq. ichar('+') ) .or.
&			(ch .eq. KEY_SB .and. (hotkey .eq. PLUS .or.
&						hotkey .eq. PLUS2) ) ) then

			retval = lightpad( hotkey, oldkey, PLUS )
			ch = ichar('+')
			call arithmetic(PLUSACT)

* the number 4
		elseif( (ch .eq. ichar('4') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. FOUR) ) then

			retval = lightpad( hotkey, oldkey, FOUR )
			ch = ichar('4')
			call number('4')

* the number 5
		elseif( (ch .eq. ichar('5') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. FIVE) ) then

			retval = lightpad( hotkey, oldkey, FIVE )
			ch = ichar('5')
			call number('5')

* the number 6
		elseif( (ch .eq. ichar('6') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. SIX) ) then

			retval = lightpad( hotkey, oldkey, SIX )
			ch = ichar('6')
			call number('6')

* the number 1
		elseif( (ch .eq. ichar('1') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. ONE) ) then

			retval = lightpad( hotkey, oldkey, ONE )
			ch = ichar('1')
			call number('1')

* the number 2
		elseif( (ch .eq. ichar('2') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. TWO) ) then

			retval = lightpad( hotkey, oldkey, TWO )
			ch = ichar('2')
			call number('2')

* the number 3
		elseif( (ch .eq. ichar('3') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. THREE) ) then

			retval = lightpad( hotkey, oldkey, THREE )
			ch = ichar('3')
			call number('3')

* Equal or Enter ( "=" in algebraic mode, "enter" in RPN mode )
		elseif( (ch .eq. ichar('=') .or. ch .eq. KEY_NEWLN .or.
&			 ch .eq. KEY_CR .or. ch .eq. NULL ) .or.
&			(ch .eq. KEY_SB .and. (hotkey .eq. EQUAL .or.
&						hotkey .eq. EQUAL2) ) ) then

			retval = lightpad( hotkey, oldkey, EQUAL )
			ch = ichar('=')
			call arithmetic(EQUALACT)

* the number 0
		elseif( (ch .eq. ichar('0') ) .or.
&			(ch .eq. KEY_SB .and. (hotkey .eq. ZERO .or.
&			 hotkey .eq. ZERO2)) ) then

			retval = lightpad( hotkey, oldkey, ZERO )
			ch = ichar('0')
			call number('0')

* decimal point
		elseif( (ch .eq. ichar('.') ) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. DP) ) then

			retval = lightpad( hotkey, oldkey, DP )
			ch = ichar('.')
			call number('.')

* backspace key for correcting numeric entry errors
		elseif( (ch .eq. KEY_BACKSPACE) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. BS) ) then

			retval = lightpad( hotkey, oldkey, BS )
			ch = KEY_BACKSPACE
			call number('-')

* last x
		elseif( (ch .eq. ichar('l') .or. ch .eq. ichar('L')) .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. LSX) ) then

			retval = lightpad( hotkey, oldkey, LSX )
			ch = ichar('l')
			call liftstack
			retval = displayval( lastx, lastx_i )
			numinit = .true.
			if( rpnmode ) then
				action = .true.
			else
				action = .false.
			endif

* Begin running a Macro
		elseif( ch .eq. ichar('g') .or. ch .eq. ichar('G') .or.
&			ch .eq. CTRL('G') .or.
&			(ch .eq. KEY_SB .and. hotkey .eq. GO) ) then

			retval = lightpad( hotkey, oldkey, GO )

			if( recording .gt. 0 ) then
				retval = beep()
				go to 100
			endif
			if ( has_colors() ) then
			  menupair = setpair(COLOR_RED,COLOR_WHITE)
			  hlitpair = setpair(COLOR_WHITE,COLOR_RED)
			endif

			do i = 1,10
				if(totstrokes(i) .eq. 0) then
					macname(i) = '...'
				endif
			end do

			retval = popupmenu( macname, 10, 4, 49,
&		   	menupair, hlitpair, runitem, 'Macro Run',
&	 		'Move to desired Macro and press <Return>.'//
&			'  <Backspace> cancels', -1, .false. )

			if( retval .eq. ERR ) then

			  call message('Error setting up Macro menu')

			elseif( retval .gt. 0 ) then

				if( totstrokes(retval) .le. 0 ) then
					retval = beep()
					go to 1234
				endif
				playback = retval
				if(ch .eq. CTRL('G')) then
					macstep = .true.
					retval  = beep()
				else
					macstep = .false.
				endif
				holdmode = rpnmode
				rpnmode  = macmode(retval)
				keycount = 0

			endif


* comma key to toggle commas in the display
		elseif( ch .eq. ichar(',') .or.
&			 (ch .eq. KEY_SB .and. hotkey .eq. CMA) ) then

			retval = lightpad( hotkey, oldkey, CMA )
			ch = ichar(',')
			if( commas ) then
			  commas = .false.
			else
			  commas = .true.
			endif
			retval = displayval( screenval, screenval_i )
			numinit = .true.
			if( .not. rpnmode ) then
				action = .false.
			endif
			if( recording .gt. 0 ) then
				go to 100
			endif

*-------------------------------------
* the help screen

		elseif( ch .eq. KEY_F(1) .or. ch .eq. ichar('?') ) then

			retval = wbkgd( FKEY1, fhattr )
			retval = wrefresh( FKEY1 )

			retval = help(HELPFILE)
			if( retval .eq. ERR ) then
			  call message('Error setting up help window')
			endif

			retval = napms( 200 )
			retval = wbkgd( FKEY1, fnattr )
			retval = wnoutrefresh( FKEY1 )

*-------------------------------------
*  when F2 is pressed, pop up a menu of additional math functions

		elseif( ch .eq. KEY_F(2) ) then

			retval = wbkgd( FKEY2, fhattr )
			retval = wrefresh( FKEY2 )

			if( playback .le. 0 .or. macstep ) then
			  if ( has_colors() ) then
			    menupair = setpair(COLOR_WHITE,COLOR_BLACK)
			    hlitpair = setpair(COLOR_BLACK,COLOR_WHITE)
			  endif

			  if( rpnmode ) then
			  	itemnum = MAXFUNCTS
			  else
			  	itemnum = MAXFUNCTS - 1
			  endif

			  if( playback .gt. 0 ) then
			    lastitem = ichosen
			    menumsg  = tmpmsg
			  else
			    menumsg = 
&			     'Move to desired function and press <Spacebar>.'//
&			     '  <Backspace> cancels menu.'
			  endif
			  retval = popupmenu( functmen, itemnum, 2, 46,
&		   	  menupair, hlitpair,
&			  lastitem, 'Math Functions',menumsg, -1, .true. )
			  if( playback .le. 0 ) then
			    ichosen = retval
			  endif

			  if ( has_colors() ) then
			  	call freepair( menupair )
			  	call freepair( hlitpair )
			  endif
			endif

			tmpkey = wbkgd( FKEY2, fnattr )
			tmpkey = wrefresh( FKEY2 )

			if( ichosen .eq. ERR ) then

			  call message('Error setting up function menu')

			elseif( ichosen .lt. -1000 ) then

			  retval = -ichosen/1000
			  if(retval .ge. KEY_F(2) .and.
&			  		retval .le. KEY_F(5) ) then
				ch = retval
				go to 500
			  else
			  	retval = beep()
			  endif

			elseif( ichosen .gt. 0 ) then

				call mathfunc(ichosen)

			endif

*-------------------------------------
*  when F3 is pressed, pop up a menu of statistical operations/functions

		elseif( ch .eq. KEY_F(3) ) then

			retval = wbkgd( FKEY3, fhattr )
			retval = wrefresh( FKEY3 )

			if( playback .le. 0 .or. macstep ) then
			  if ( has_colors() ) then
			    menupair = setpair(COLOR_BLACK,COLOR_GREEN)
			    hlitpair = setpair(COLOR_BLACK,COLOR_WHITE)
			  endif

			  if( playback .gt. 0 ) then
			    laststat = ichosen
			    menumsg  = tmpmsg
			  else
			    menumsg  =
&	 		     'Move to desired item and press <Spacebar>.'//
&			     '  <Backspace> cancels menu.'
			  endif
			  retval = popupmenu( statmen, MAXSTATS, 2, 46,
&		   	  menupair, hlitpair,
&			  laststat, 'Statistics', menumsg, -1, .true. )
			  if( playback .le. 0 ) then
			    ichosen = retval
			  endif

			  if ( has_colors() ) then
			  	call freepair( menupair )
			  	call freepair( hlitpair )
			  endif
			endif

			tmpkey = wbkgd( FKEY3, fnattr )
			tmpkey = wrefresh( FKEY3 )

			if( ichosen .eq. ERR ) then

			  call message('Error setting up statistics menu')

			elseif( ichosen .lt. -1000 ) then

			  retval = -ichosen/1000
			  if(retval .ge. KEY_F(2) .and.
&			  		retval .le. KEY_F(5) ) then
				ch = retval
				go to 500
			  else
			  	retval = beep()
			  endif

			elseif( ichosen .gt. 0 ) then

				call statfunc(ichosen, xstat, x2stat,
&					ystat, y2stat, xystat, nstat, seed)

			endif

*----------------------------------

*  CTRL-A:  shortcut for accumulating statistics (skips menu)

		elseif( ch .eq. CTRL('A') ) then

				lastx   = screenval
				lastx_i = screenval_i
				xstat  = xstat  + screenval
				x2stat = x2stat + screenval*screenval
				ystat  = ystat  + regval(1,1)
				y2stat = y2stat + regval(1,1)**2.0
				xystat = xystat + screenval*regval(1,1)

				nstat  = nstat + 1.

				retval = displayval( nstat, dble(0.) )
				action = .false.
				numinit = .true.

*----------------------------------

*  when F4 is pressed, pop up a menu of unit conversions

		elseif( ch .eq. KEY_F(4) ) then

			retval = wbkgd( FKEY4, fhattr )
			retval = wrefresh( FKEY4 )

			if( playback .le. 0 .or. macstep ) then
			  if ( has_colors() ) then
			    menupair = setpair(COLOR_WHITE,COLOR_RED)
			    hlitpair = setpair(COLOR_BLACK,COLOR_WHITE)
			  endif

*			  actually alternate between two menus that allow the
*			  user to switch the direction of the conversions.

			  if( playback .gt. 0 ) then
			    lastconv = ichosen
			    menumsg  = tmpmsg
			  else
			    menumsg  =
&	 		      'Arrows move, <Spacebar> selects.'//
&			      '  <Ctrl>-<R> reverses.  <Backspace> cancels.'
			  endif
  450			  if( convdir .eq. 1 ) then
			    retval = popupmenu( convmen1, MAXCONV, 2, 46,
&		   	      menupair, hlitpair,
&			      lastconv, 'Conversions',menumsg,
&			      CTRL('R'), .true.)
			  else
			    retval = popupmenu( convmen2, MAXCONV, 2, 46,
&		   	      menupair, hlitpair,
&			      lastconv, 'Conversions',menumsg,
&	 		      CTRL('R'),.true.)
			  endif
			  if( playback .le. 0 ) then
			    ichosen = retval
			  endif

			  if( ichosen .eq. -5 ) then
			    convdir = 3 - convdir
			    go to 450
			  endif

			  if ( has_colors() ) then
			  	call freepair( menupair )
			  	call freepair( hlitpair )
			  endif
			endif

			tmpkey = wbkgd( FKEY4, fnattr )
			tmpkey = wrefresh( FKEY4 )

			if( ichosen .eq. ERR ) then

			  call message('Error setting up conversions menu')

			elseif( ichosen .lt. -1000 ) then

			  retval = -ichosen/1000
			  if(retval .ge. KEY_F(2) .and.
&			  		retval .le. KEY_F(5) ) then
				ch = retval
				go to 500
			  else
			  	retval = beep()
			  endif

			elseif( ichosen .gt. 0 ) then

				call unitconv(ichosen, convdir)

			endif

*-------------------------------------
*  when F5 is pressed, pop up a menu of physical constants

		elseif( ch .eq. KEY_F(5) ) then

			retval = wbkgd( FKEY5, fhattr )
			retval = wrefresh( FKEY5 )

			if( playback .le. 0 .or. macstep ) then
			  if ( has_colors() ) then
			    menupair = setpair(COLOR_BLACK,COLOR_CYAN)
			    hlitpair = setpair(COLOR_BLACK,COLOR_WHITE)
			  endif

			  if( playback .gt. 0 ) then
			    lastconst = ichosen
			    menumsg   = tmpmsg
			  else
			    menumsg =
&	 		     'Move to desired item and press <Spacebar>.'//
&			     '  <Backspace> cancels menu.'
			  endif
			  retval = popupmenu( constmen, MAXCONST, 2, 45,
&		   	  menupair, hlitpair,
&			  lastconst, 'Constants',menumsg, -1, .true. )
			  if( playback .le. 0 ) then
			    ichosen = retval
			  endif

			  if ( has_colors() ) then
			  	call freepair( menupair )
			  	call freepair( hlitpair )
			  endif
			endif

			tmpkey = wbkgd( FKEY5, fnattr )
			tmpkey = wrefresh( FKEY5 )

			if( ichosen .eq. ERR ) then

			  call message('Error setting up constants menu')

			elseif( ichosen .lt. -1000 ) then

			  retval = -ichosen/1000
			  if(retval .ge. KEY_F(2) .and.
&			  		retval .le. KEY_F(5) ) then
				ch = retval
				go to 500
			  else
			  	retval = beep()
			  endif

			elseif( ichosen .gt. 0 ) then

			  call liftstack

			  if( ichosen .eq. 1 ) then
*  Avogadro's Number (molecules/mol)
				tmpval = 6.022045d23
			  elseif( ichosen .eq. 2 ) then
*  Universal gas constant (J/mol-K)
				tmpval = 8.31441d0
			  elseif( ichosen .eq. 3 ) then
*  Molar volume of ideal gas at s.t.p. (m^3/mol)
				tmpval = .02241383d0
			  elseif( ichosen .eq. 4 ) then
*  Plank's Constant (Joule)
				tmpval = 6.626176d-34
			  elseif( ichosen .eq. 5 ) then
*  Gravitational constant (N-m^2/kg^2)
				tmpval = 6.672d-11
			  elseif( ichosen .eq. 6 ) then
*  Standard gravitational acceleration ( m/s^2 )
				tmpval = 9.80665d0
			  elseif( ichosen .eq. 7 ) then
*  Elementary Charge (C)
				tmpval = 1.6021892d-19
			  elseif( ichosen .eq. 8 ) then
*  Electrostatic force (N-m^2/C^2)
				tmpval = 9.0d9
			  elseif( ichosen .eq. 9 ) then
*  Electron rest mass (kg)
				tmpval = 9.109534d-31
			  elseif( ichosen .eq. 10 ) then
*  Atomic mass (kg)
				tmpval = 1.6605655d-27
			  elseif( ichosen .eq. 11 ) then
*  Speed of light in a vacuum (m/s)
				tmpval = 2.99792458d8
			  elseif( ichosen .eq. 12 ) then
*  Boltzmann's constant (J/molecule-K)
				tmpval = 1.380662d-23
			  elseif( ichosen .eq. 13 ) then
*  Stefan-Boltzmann constant (W/m^2-K^4)
				tmpval = 5.67032d-8
			  elseif( ichosen .eq. 14 ) then
*  Approximate radius of the Earth ( meters )
				tmpval = 6.38d6
			  elseif( ichosen .eq. 15 ) then
*  Faraday's ( Coulumbs/mole )
				tmpval = 96484.6d0
			  endif

			  retval = displayval( tmpval, dble(0.) )
			  if( rpnmode ) then
			  	action = .true.
			  else
			  	action = .false.
			  endif
			  numinit = .true.

			endif

*----------------------------------

*  Scientific/Normal toggle
		elseif( ch .eq. KEY_F(6) ) then

			retval = wbkgd( FKEY6, fhattr )
			retval = wrefresh( FKEY6 )

			if( numinit ) then
			  if( scinot ) then
				  scinot = .false.
				  retval = displayval( screenval, screenval_i )
			  else
			  	scinot = .true.
			  	retval = displayval( screenval, screenval_i )
			  endif
			endif

			retval = napms( 200 )
			retval = wbkgd( FKEY6, fnattr )
			retval = wrefresh( FKEY6 )

*----------------------------------

* Radian/Degree toggle
		elseif( ch .eq. KEY_F(7) ) then

			retval = wbkgd( FKEY7, fhattr )
			retval = wrefresh( FKEY7 )

			if( radflag .eq. 1 ) then
				radflag = 2
			elseif( radflag .eq. 2 ) then
				radflag = 3
			else
				radflag = 1
			endif

			retval = napms( 200 )
			retval = wbkgd( FKEY7, fnattr )
			retval = wrefresh( FKEY7 )

*----------------------------------

* toggle RPN mode
		elseif ( ch .eq. KEY_F(8) ) then

			if( recording .gt. 0 ) then
				retval = beep()
				go to 100
			endif

			retval = wbkgd( FKEY8, fhattr )
			retval = wrefresh( FKEY8 )

			numinit = .true.
			action = .false.
			retval = wbkgd( keywin(C), keyattr )
			retval = wbkgd( keywin(LP), keyattr )
			retval = wbkgd( keywin(RP), keyattr )
			retval = wbkgd( keywin(EQUAL), keyattr )
			if( .not. rpnmode ) then
			  rpnmode = .true.
			  action = .true.
			  do i = 1,MAXREGS
				regval(i,1) = 0.
				regval(i,2) = 0.
			  end do
			  retval = mvwaddstr(keywin(C),0,0,'CLX')
			  retval = mvwaddstr(keywin(LP),0,0,' UP')
			  retval = mvwaddstr(keywin(RP),0,0,' DN')
			  retval = mvwaddstr(keywin(EQUAL),0,1,'E')
			  retval = mvwaddstr(keywin(EQUAL),1,1,'n')
			  retval = mvwaddstr(keywin(EQUAL),2,1,'t')
			else
			  rpnmode = .false.
			  reglev = 1
			  regact(1) = 0
			  regval(1,1) = 0.
			  regval(1,2) = 0.
			  retval = mvwaddstr(keywin(C),0,0,' C ')
			  retval = mvwaddstr(keywin(LP),0,0,' ( ')
			  retval = mvwaddstr(keywin(RP),0,0,' ) ')
			  retval = mvwaddstr(keywin(EQUAL),0,1,' ')
			  retval = mvwaddstr(keywin(EQUAL),1,1,'=')
			  retval = mvwaddstr(keywin(EQUAL),2,1,' ')
			endif
			retval = wnoutrefresh( keywin(C) )
			retval = wnoutrefresh( keywin(LP) )
			retval = wnoutrefresh( keywin(RP) )
			retval = wnoutrefresh( keywin(EQUAL) )
			retval = refresh()
			tmpkey = hotkey
			retval = lightpad( hotkey, oldkey, tmpkey )

			retval = napms( 200 )
			retval = wbkgd( FKEY8, fnattr )
			retval = wrefresh( FKEY8 )

*----------------------------------

* toggle COMPLEX Mode
* switch from real number entry field to complex entry field
* swap real/imaginary values
		elseif ( ch .eq. KEY_F(9) .or. ch .eq. ichar(';') .or.
&			 ch .eq. ichar('|') .or.
&			 (ch .eq. KEY_SB .and. hotkey .eq. SEMI) .or.
&			 (ch .eq. KEY_SB .and. hotkey .eq. RXI) ) then

			if( ch .eq. KEY_F(9) ) then
				retval = wbkgd( FKEY9, fhattr )
				retval = wrefresh( FKEY9 )
				numinit = .true.
			endif

	if( .not. compmode ) then
		compmode = .true.
		if( has_colors() ) then
		  retval = wbkgd( mainfield_i, fpair )
		  retval = wbkgd( expfield_i, fpair )
		else
		  retval = wbkgd( mainfield_i, A_NORMAL )
		  retval = wbkgd( expfield_i, A_NORMAL )
		endif
		retval = displayval( screenval, dble(0.) )
	elseif( ch .eq. KEY_F(9) ) then
		if( recording .gt. 0 ) then
			retval = beep()
			retval = wbkgd( FKEY9, fnattr )
			retval = wrefresh( FKEY9 )
			go to 100
		endif
		compmode    = .false.
		screenval_i = 0.
		do i = 1, MAXREGS
		   regval(i,2) = 0.
		end do
		if( has_colors() ) then
		  retval = wbkgd( mainfield_i, cpair )
		  retval = wbkgd( expfield_i, cpair )
		else
		  retval = wbkgd( mainfield_i, A_REVERSE )
		  retval = wbkgd( expfield_i, A_REVERSE )
		endif
		retval = wjustify( mainfield_i, '     ', 2, .FALSE., .TRUE.)
		retval = wjustify( expfield_i, '      ', 2, .FALSE., .TRUE.)
	endif

			if( ch .eq. KEY_F(9) ) then

			  retval = napms( 200 )
			  retval = wbkgd( FKEY9, fnattr )
			  retval = wrefresh( FKEY9 )

*			switch from real display register to the imaginary
*			display register for numeric input.

			elseif( ch .eq. ichar(';') .or.
&				(ch .eq. KEY_SB .and. hotkey .eq. SEMI) ) then

			  retval = lightpad( hotkey, oldkey, SEMI )
			  ch = ichar(';')
			  if( (.not. compentry) ) then
			  	call number(';')
			  else
			  	retval = beep()
			  endif

*			exchange the real and imaginary display register values

			else

			  retval = lightpad( hotkey, oldkey, RXI )
			  ch = ichar('|')
			  tmpval      = screenval
			  screenval   = screenval_i
			  screenval_i = tmpval
			  retval = displayval( screenval, screenval_i )
			  numinit = .true.
			  if( .not. rpnmode ) then
			  	action = .false.
			  endif

			endif

*-------------------------------------
*  when F10 is pressed, start/stop recording keyboard macro

		elseif( ch .eq. KEY_F(10) ) then

		  retval = wbkgd( FKEY10, fhattr )
		  retval = wrefresh( FKEY10 )
		  if( recording .gt. 0 ) then
		  	totstrokes(recording) = keycount
		  	recording = 0
			retval = napms( 200 )
		  else

			if ( has_colors() ) then
			  menupair = setpair(COLOR_BLACK,COLOR_MAGENTA)
			  hlitpair = setpair(COLOR_BLACK,COLOR_WHITE)
			endif

			do i = 1,10
				if(totstrokes(i) .eq. 0) then
					macname(i) = '...'
				endif
			end do

			macitem = 1
			retval = popupmenu( macname, 10, 4, 49,
&		   	menupair, hlitpair,
&			macitem, 'Macro Record',
&	 		'Select Macro storage register.'//
&			'  <Backspace> cancels', -1, .false. )

			if( retval .eq. ERR ) then

			  call message('Error setting up function menu')

			elseif( retval .gt. 0 ) then

			  recording = retval
			  macmode(retval) = rpnmode
			  keycount = 0
			  if( totstrokes(recording) .gt. 0 ) then
				macitem = 1
				retval = popupmenu( answer, 3, 7, 49,
&			   	menupair, hlitpair,
&				macitem, ' Overwrite ? ',
&	 			'Yes, No or Clear Macro?  <Backspace> cancels',
&				 -1, .false.)
				if( retval .eq. 3 ) then
					totstrokes(recording) = 0
					recording = 0
					go to 600
				elseif( retval .ne. 2) then
					recording = 0
					go to 600
				endif
				totstrokes(recording) = 0
			  endif

  590			  retval = move(0,0)
			  retval = clrtoeol()
			  retval = wrefresh( stdscr )
			  retval = addstr('Enter Macro Name: ')
			  retval = echo()
			  retval = wgetnstr( stdscr, tmpstr, 18 )
			  retval = noecho()
			  if(tmpstr(1:1) .eq. ' ' .or. tmpstr(1:1) .eq.
&			     char(NULL) ) then
				retval = beep()
				go to 590
			  endif
			  macname(recording) = tmpstr

			endif
  600			if ( has_colors() ) then
				call freepair( menupair )
				call freepair( hlitpair )
			endif

		  endif

		  tmpkey = wbkgd( FKEY10, fnattr )
		  tmpkey = wrefresh( FKEY10 )

*-------------------------------------

* screen refresh
		elseif ( ch .eq. CTRL('L') ) then

*			pressing CTRL-L redraws the screen in the event that
*			it has been corrupted by UNIX system messages (mail
*			arrival notices, etc.)

			retval = touchwin( curscr )
			retval = wrefresh( curscr )

* exit
		elseif ( ch .eq. CTRL('X') ) then

*			clear the screen when exiting

			retval = wbkgd( stdscr, A_NORMAL )
			retval = clear()

			retval = move( int(LINES/2)-3, int(COLS/2)-11 )
			retval = addstr('Curs-Calc version 1.2')
			retval = move( int(LINES/2)-1, int(COLS/2)-15 )
			retval = addstr('Programmed by Wade R. Schauer')
			retval = move( int(LINES/2)-0, int(COLS/2)-15 )
			retval = addstr('Sierra Consulting Corp.')
			retval = move( int(LINES/2)+3, int(COLS/2)-15 )
			retval = addstr('>>>>>>> 916-782-5421 <<<<<<<<')

			retval = refresh()
			retval = endwin()

*			now save the state of the calculator to a file

			open( 32, file=filenm,status='unknown',
&				form='unformatted',access='direct',
&				recl=16, err=750 )
			irec = 1
			write( 32, rec=irec, err=750 ) screenval,screenval_i
			irec = irec + 1
			write( 32, rec=irec, err=750 ) scinot
			irec = irec + 1
			write( 32, rec=irec, err=750 ) radflag
			irec = irec + 1
			write( 32, rec=irec, err=750 ) rpnmode
			irec = irec + 1
			write( 32, rec=irec, err=750 ) precision
			irec = irec + 1
			write( 32, rec=irec, err=750 ) reglev
			irec = irec + 1
			write( 32, rec=irec, err=750 ) seed
			irec = irec + 1
			do i = 1, 20
			   write( 32, rec=irec, err=750 ) (memstore(i,j),j=1,2)
			   irec = irec + 1
			end do
			do i = 1, MAXREGS
			   write( 32, rec=irec, err=750 ) (regval(i,j),j=1,2)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) (regvalmd(i,j),j=1,2)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) (regvalpw(i,j),j=1,2)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) regact(i)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) regactmd(i)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) regactpw(i)
			   irec = irec + 1
			end do
			write( 32, rec=irec, err=750 ) commas
			irec = irec + 1
			write( 32, rec=irec, err=750 ) xstat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) x2stat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) ystat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) y2stat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) xystat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) nstat
			irec = irec + 1
			write( 32, rec=irec, err=750 ) compmode
			irec = irec + 1
			write( 32, rec=irec, err=750 ) lastx, lastx_i
			irec = irec + 1
			do i = 1, 10
			   write( 32, rec=irec, err=750 ) macname(i)
			   irec = irec + 1
			   write( 32, rec=irec, err=750 ) totstrokes(i),
&			   					macmode(i)
			   irec = irec + 1
			   do j = 1,MAXMAC/4
			      write( 32, rec=irec, err=750 )
&			      		(keymac(i,j+(k*MAXMAC/4)), k=0,3)
			      irec = irec + 1
			   end do
			end do
			close(32)

  750			stop
		else
*			I don't know what to do with that key, so I'll beep
			retval = beep()
		endif

 1234		continue

		if( recording .gt. 0 ) then
			keycount = keycount + 1
			if( keycount .gt. MAXMAC ) then
			  totstrokes(recording) = MAXMAC
			  retval = beep()
			  recording = 0
			else
			  if( ch .ge. KEY_F(1) .and. ch .le. KEY_F(10) ) then
			    if(ch .ge. KEY_F(2) .and. ch .le. KEY_F(5) .and.
&			    	ichosen .gt. 0) then
			      keymac(recording,keycount) = ch*10000 +
&			      				ichosen*100
			      if( ch .eq. KEY_F(4) ) then
			        keymac(recording,keycount) = ch*10000 +
&			      				ichosen*100 + convdir
			      endif
			    else
			      keycount = keycount - 1
			    endif
			  else
			  	keymac(recording,keycount) = ch
			  endif
			endif
		elseif( playback .gt. 0 ) then
			keycount = keycount + 1
			if( keycount .gt. totstrokes(playback) ) then
			  playback = 0
			  rpnmode  = holdmode
			  if(macstep) then
			  	retval = beep()
			  endif
			endif
		endif

	end do

* program exiting was handled in the above loop

	end
