
************************************************************************

* the following two functions, help() and fillwin(), are general
*  purpose routines.  help displays a help window; fillwin fills
*  a window from a file, interpreting control characters as attributes.

*  Usage:
*	integer function help(filename)
*	  character*(*) filename
*	returns OK if successful, ERR otherwise.
*	In this version, the filename is only used the first time
*	through; subsequent calls just use the same data.
*	An error return is possible only on the first call, and
*	indicates a problem with the help file or a memory shortage.
*
*	The help window disappears when any key is pressed.

	integer function help(file)

	implicit none

	include '../../curses.fh'

	character*(*) file

	integer helpwin
	save helpwin

*	shadow holds the contents of the screen that will be hidden
*	when another window is displayed

	integer shadow

*	"screenimage" utility function
	integer screenimage

*	help file "f"

	integer f, helppair

	save f, shadow, helppair

	character*4 tmpstr, tmpfile*80

	integer fillwin, retval, setpair

	data helpwin / -1 /
	data f / 23 /

	if( helpwin .lt. 0 .or. helpwin .eq. NULL ) then

*		create help window *

		helpwin = newwin(0,0,0,0)

		if( helpwin .eq. NULL ) then
		  help = ERR
		  return
		endif
		call clearok( helpwin, .FALSE. )

*  make screen a nice white on BLUE

		if(has_colors() .and. COLOR_PAIRS .gt. 0) then
		  helppair = setpair( COLOR_WHITE, COLOR_BLUE )
		  if( helppair .gt. 0 ) then
		    retval = wbkgd( helpwin, helppair )
		  endif
		else
		    retval = wbkgd( helpwin, A_NORMAL )
		endif

		retval = leaveok(helpwin, .TRUE.)
		retval = keypad(helpwin, .TRUE.)

*		open up help file *

		open(f, file=file, status='old', err=50)
		go to 100
  50		tmpfile = '/usr/local/lib/'//file
		open(f, file=tmpfile, status='old', err=500)

*		read help file into window *

  100		retval = fillwin(f, helpwin)

		close(f)

		shadow = NULL

	endif

*	before displaying the help window, save a copy of "curscr" so it
*	can be retreived when help is exited.  Use the "screenimage" utility.

	shadow = screenimage( shadow, stdscr )
	if( shadow .eq. NULL ) then
		help = ERR
		return
	endif

*	now display the help window on the physical screen.

	retval = touchwin( helpwin )
	retval = wrefresh( helpwin )

*	when any key is pressed, restore the "pre-help" contents of the
*	screen.

	do while ( .true. )

		retval = wgetch(helpwin)

		if ( retval .eq. CTRL('L') ) then
			retval = touchwin( curscr )
			retval = wrefresh( curscr )
		else
			retval = touchwin( shadow )
			retval = wrefresh( shadow )
			retval = flushinp()
			help = OK
			return
		endif
	end do

  500	help = ERR
  	return
  	end
