**************************************************************************
*  wrs (8/16/92)
*  --  RECTANGLE -- create a rectangular border of characters

*	similar to wborder, but the user specifies the boundaries of the
*	rectangular region within a window rather than using the window
*	boundaries by default.

	integer*4 function rectangle( win, ty, tx, by, bx,
&		ls, rs, ts, bs, tl, tr, bl, br )

	implicit none
	
	include '../../curses.fh'

*	positions are relative to the top corner of the window "win"

*	ty is the y coordinate of the top left corner of the rectangle (0 min)
*	tx is the x coordinate of the top left corner of the rectangle (0 min)
*	by is the y coordinate of the bottom left corner of the rectangle
*	by is the x coordinate of the bottom left corner of the rectangle

	integer*4  win, ty, tx, by, bx

*	left, right, top, bottom sides and top-left, top-right, bottom-left
*	and bottom right corner characters.  PASSED AS INTEGERS, SO REMEMBER
*	TO CONVERT STANDARD CHARACTER USING "ICHAR" WHEN CALLING THIS
*	FUNCTION.  Single and double line character names defined for the
*	alternate character set are already defined as integers.

	integer*4  ls, rs, ts, bs, tl, tr, bl, br

	integer*4  bytmp, bxtmp
	integer*4  lstmp, rstmp, tstmp, bstmp, tltmp, trtmp,
&	           bltmp, brtmp, retval, ymax, xmax

*	make sure user did not specify bottom coordinates greater than the
*	window border.  Fortran programmers may tend to do this since
*	the window arrays start with 0 instead of 1.
	
	retval = getmaxyx( win, ymax, xmax )

	bytmp = by
	if( by .gt. ymax-1 ) bytmp = ymax-1
	bxtmp = bx
	if( bx .gt. xmax-1 ) bxtmp = xmax-1

*	use the double line character set by default if the user passes
*	zeros for the border characters.  Of course, these will default
*	to single line characters unless the double line set for the
*	terminal running the program has been defined in "initscr.f"

	lstmp = ls
	if(lstmp .eq. 0) then
		lstmp = ACS_DVLINE
	endif
	rstmp = rs
	if(rstmp .eq. 0) then
		rstmp = ACS_DVLINE
	endif	
	tstmp = ts
	if(tstmp .eq. 0) then
		tstmp = ACS_DHLINE
	endif
	bstmp = bs
	if(bstmp .eq. 0) then
		bstmp = ACS_DHLINE
	endif
	tltmp = tl
	if(tltmp .eq. 0) then
		tltmp = ACS_DULCORNER
	endif
	trtmp = tr
	if(trtmp .eq. 0) then
		trtmp = ACS_DURCORNER
	endif
	bltmp = bl
	if(bltmp .eq. 0) then
		bltmp = ACS_DLLCORNER
	endif
	brtmp = br
	if(brtmp .eq. 0) then
		brtmp = ACS_DLRCORNER
	endif

	rectangle = ERR

	if( wmove( win, ty, tx ) .eq. ERR ) then
		return
	else
		if( waddch( win, tltmp ) .eq. ERR ) then
			return
		else
			if( whline( win, tstmp, bx-tx-1 ) .eq. ERR ) then
				return
			endif
		endif
	endif

	if( wmove( win, ty+1, tx ) .eq. ERR ) then
		return
	else
		if( wvline( win, lstmp, by-ty-1 ) .eq. ERR ) then
			return
		endif
	endif

	if( wmove( win, by, tx ) .eq. ERR ) then
		return
	else
		if( waddch( win, bltmp ) .eq. ERR ) then
			return
		else
			if( whline( win, bstmp, bx-tx-1 ) .eq. ERR ) then
				return
			endif
		endif
	endif

	if( wmove( win, ty, bx ) .eq. ERR ) then
		return
	else
		if( waddch( win, trtmp ) .eq. ERR ) then
			return
		endif
	endif

	if( wmove( win, ty+1, bx ) .eq. ERR ) then
		return
	else
		if( wvline( win, rstmp, by-ty-1 ) .eq. ERR ) then
			return
		endif
	endif

	if( wmove( win, by, bx ) .eq. ERR ) then
		return
	else
		if( waddch( win, brtmp ) .eq. ERR ) then
			return
		endif
	endif

	rectangle = OK

	return

	end
