* wrs(10/93) - setpair, freepair, resetpair

*   setpair used in conjunction with "freepair" allows simple
*   management of color pairs as variables rather than having to
*   worry about what specific numbers have been used throughout a F-curses
*   program that uses colors.
*
*     syntax is...
*	 attrib = setpair(COLOR_BLUE,COLOR_RED)
*
*        if "attrib" is greater than zero, then a pair was allocated
*	 successfully and the routine "init_pair" initialized a color pair with
*	 the colors BLUE on RED.  The value returned is the set attribute
*	 COLOR_PAIR(pair).
*
*	 you would then use the attribute like any other
*
*		call attron( attrib )
*
*		once done with "pair", you can make it available by
*		calling...   freepair( attrib )
*
*		or you can change it by using "resetpair"
*
*			attrib = resetpair( attrib, COLOR_BLACK, COLOR_WHITE )
*		or	call resetpair( attrib, COLOR_BLACK, COLOR_WHITE )
*
*	 WARNING:  IF YOU ARE GOING TO USE THESE UTILITY FUNCTIONS, THEN YOU
*	 NEED TO BE SURE TO USE THEM IN EVERY ROUTINE THAT USES COLORS,
*	 BECAUSE THEY DON'T CHECK TO SEE IF YOU'VE MANUALLY USED THE init_pair
*	 FUNCTION.
*
*-----------------------------------------------------------------------------

	integer function setpair(fg,bg)

	implicit none
	
	include '../../curses.fh'

*	foreground and background colors comprising the pair being set
	integer fg, bg

* most pairs there will ever be (256 including 0)
	
	logical pairstat( 255 )
	common / pairuse / pairstat

	integer i, maxpair

	data pairstat / 255*.false. /


	setpair = -1

	if( .not. has_colors() ) return

	if( COLOR_PAIRS .le. 0 ) return	

*  only allocate from 1 to COLOR_PAIRS-1 pairs  (color pair zero is
*    pre-assigned white on black, so if you have 64 color pairs, you can
*    use 1 - 63 for different pairs )

	maxpair = min( 255, COLOR_PAIRS-1 )

	do i = 1, maxpair
	   if( .not. pairstat(i) ) then

		if( init_pair( i, fg, bg ) .eq. OK ) then
			setpair = COLOR_PAIR(i)
			pairstat(i) = .true.
			return
		endif

	   endif
	end do

	return
	end

*********************************************************************
* free a color pair that was assigned using "setpair"
*
	integer function freepair( attrib )

	implicit none

	include '../../curses.fh'

	integer attrib

	logical pairstat( 255 )
	common / pairuse / pairstat

	integer pair

	freepair = -1	

*	attrib is an attribute of the form COLOR_PAIR(pair).  Determine "pair"
*	so that we can free it from the list of pairs being used.

	pair = PAIR_NUMBER(attrib)

	if( pair .gt. 0 .and. pair .le. 255 .and.
&				pairstat(pair) .eq. .TRUE. ) then
		pairstat(pair) = .FALSE.
		freepair = OK
	endif

	return
	end

*********************************************************************
* reset a color pair that was assigned using "setpair" or "resetpair"

* note that when you change the definition of a color pair using init_pair,
* F-curses will immediately update the screen based on the new definition.

	integer function resetpair( attrib, fg, bg )

	implicit none

	include '../../curses.fh'

	integer attrib, fg, bg

	logical pairstat( 255 )
	common / pairuse / pairstat

	integer pair

	resetpair = -1	

*	attrib is an attribute of the form COLOR_PAIR(pair).  Determine "pair"
*	so that we can free it from the list of pairs being used.

	pair = PAIR_NUMBER(attrib)

	if( pair .gt. 0 .and. pair .le. 255 .and.
&				pairstat(pair) .eq. .TRUE. ) then

		if( init_pair( pair, fg, bg ) .eq. OK ) then
			resetpair = COLOR_PAIR(pair)
			attrib = resetpair
			return
		endif

	endif

	return
	end

