* wrs (4/30/93)
*
* colorset --  pre-defines a bunch of color pairs as attributes so that
*  they are quickly accessible without the process of using init_pair
*  every time the user wants to use a color.
*
*   assumes that color terminals will have 8 colors and can accomodate
*   64 color pairs,
*   - should fail somewhat gracefully if they have fewer pairs.
*
*   REMEMBER that these can then be combined (ORed) with the A_BOLD attribute
*   to yield the bright foreground colors, for a total of 16 foreground and
*   8 background colors (128 pairs)
*   
*
*   this routine must only be called after initscr() has been called
*
*     (it only needs to be called once in a program, but every routine
*      that uses the pairs set in colorset must include the header file
*      "colorset.fh")
*
*****************************************************************************

	function colorset()

	implicit none
	
	include '../../curses.fh'
	include 'colorset.fh'

	integer*4 pairtoattr

	colorset = ERR

*	check the basics, se if a terminal has color capabilities and make
*	sure the routine start_color has been called.

	if( .not. has_colors() ) then
		return
	endif

	if( COLORS .eq. 0 ) then
		if( start_color() .eq. ERR) then
			return
		endif
	endif


*	there should be at least 8 color pairs available on all color terminals
*	- according to curses documentation, color pair zero is reserved for
*	use by curses (it is supposed to be the default background),
*	unfortunately that only leaves us with 63 color pairs.

*	sacrafice black on black

	colorattr(1)	= A_INVIS

	RED_BLACK	= pairtoattr( 1, COLOR_RED, COLOR_BLACK )
	GREEN_BLACK	= pairtoattr( 2, COLOR_GREEN, COLOR_BLACK )
	YELLOW_BLACK	= pairtoattr( 3, COLOR_YELLOW, COLOR_BLACK )
	BLUE_BLACK	= pairtoattr( 4, COLOR_BLUE, COLOR_BLACK )
	MAGENTA_BLACK	= pairtoattr( 5, COLOR_MAGENTA, COLOR_BLACK )
	CYAN_BLACK	= pairtoattr( 6, COLOR_CYAN, COLOR_BLACK )
	WHITE_BLACK	= pairtoattr( 7, COLOR_WHITE, COLOR_BLACK )

*	now cycle through the other combinations

	BLACK_RED	= pairtoattr( 8, COLOR_BLACK, COLOR_RED )
	RED_RED		= pairtoattr( 9, COLOR_RED, COLOR_RED )
	GREEN_RED	= pairtoattr( 10, COLOR_GREEN, COLOR_RED )
	YELLOW_RED	= pairtoattr( 11, COLOR_YELLOW, COLOR_RED )
	BLUE_RED	= pairtoattr( 12, COLOR_BLUE, COLOR_RED )
	MAGENTA_RED	= pairtoattr( 13, COLOR_MAGENTA, COLOR_RED )
	CYAN_RED	= pairtoattr( 14, COLOR_CYAN, COLOR_RED )
	WHITE_RED	= pairtoattr( 15, COLOR_WHITE, COLOR_RED )

	BLACK_GREEN	= pairtoattr( 16, COLOR_BLACK, COLOR_GREEN )
	RED_GREEN	= pairtoattr( 17, COLOR_RED, COLOR_GREEN )
	GREEN_GREEN	= pairtoattr( 18, COLOR_GREEN, COLOR_GREEN )
	YELLOW_GREEN	= pairtoattr( 19, COLOR_YELLOW, COLOR_GREEN )
	BLUE_GREEN	= pairtoattr( 20, COLOR_BLUE, COLOR_GREEN )
	MAGENTA_GREEN	= pairtoattr( 21, COLOR_MAGENTA, COLOR_GREEN )
	CYAN_GREEN	= pairtoattr( 22, COLOR_CYAN, COLOR_GREEN )
	WHITE_GREEN	= pairtoattr( 23, COLOR_WHITE, COLOR_GREEN )

	BLACK_YELLOW	= pairtoattr( 24, COLOR_BLACK, COLOR_YELLOW )
	RED_YELLOW	= pairtoattr( 25, COLOR_RED, COLOR_YELLOW )
	GREEN_YELLOW	= pairtoattr( 26, COLOR_GREEN, COLOR_YELLOW )
	YELLOW_YELLOW	= pairtoattr( 27, COLOR_YELLOW, COLOR_YELLOW )
	BLUE_YELLOW	= pairtoattr( 28, COLOR_BLUE, COLOR_YELLOW )
	MAGENTA_YELLOW	= pairtoattr( 29, COLOR_MAGENTA, COLOR_YELLOW )
	CYAN_YELLOW	= pairtoattr( 30, COLOR_CYAN, COLOR_YELLOW )
	WHITE_YELLOW	= pairtoattr( 31, COLOR_WHITE, COLOR_YELLOW )

	BLACK_BLUE	= pairtoattr( 32, COLOR_BLACK, COLOR_BLUE )
	RED_BLUE	= pairtoattr( 33, COLOR_RED, COLOR_BLUE )
	GREEN_BLUE	= pairtoattr( 34, COLOR_GREEN, COLOR_BLUE )
	YELLOW_BLUE	= pairtoattr( 35, COLOR_YELLOW, COLOR_BLUE )
	BLUE_BLUE	= pairtoattr( 36, COLOR_BLUE, COLOR_BLUE )
	MAGENTA_BLUE	= pairtoattr( 37, COLOR_MAGENTA, COLOR_BLUE )
	CYAN_BLUE	= pairtoattr( 38, COLOR_CYAN, COLOR_BLUE )
	WHITE_BLUE	= pairtoattr( 39, COLOR_WHITE, COLOR_BLUE )

	BLACK_MAGENTA	= pairtoattr( 40, COLOR_BLACK, COLOR_MAGENTA )
	RED_MAGENTA	= pairtoattr( 41, COLOR_RED, COLOR_MAGENTA )
	GREEN_MAGENTA	= pairtoattr( 42, COLOR_GREEN, COLOR_MAGENTA )
	YELLOW_MAGENTA	= pairtoattr( 43, COLOR_YELLOW, COLOR_MAGENTA )
	BLUE_MAGENTA	= pairtoattr( 44, COLOR_BLUE, COLOR_MAGENTA )
	MAGENTA_MAGENTA	= pairtoattr( 45, COLOR_MAGENTA, COLOR_MAGENTA )
	CYAN_MAGENTA	= pairtoattr( 46, COLOR_CYAN, COLOR_MAGENTA )
	WHITE_MAGENTA	= pairtoattr( 47, COLOR_WHITE, COLOR_MAGENTA )

	BLACK_CYAN	= pairtoattr( 48, COLOR_BLACK, COLOR_CYAN )
	RED_CYAN	= pairtoattr( 49, COLOR_RED, COLOR_CYAN )
	GREEN_CYAN	= pairtoattr( 50, COLOR_GREEN, COLOR_CYAN )
	YELLOW_CYAN	= pairtoattr( 51, COLOR_YELLOW, COLOR_CYAN )
	BLUE_CYAN	= pairtoattr( 52, COLOR_BLUE, COLOR_CYAN )
	MAGENTA_CYAN	= pairtoattr( 53, COLOR_MAGENTA, COLOR_CYAN )
	CYAN_CYAN	= pairtoattr( 54, COLOR_CYAN, COLOR_CYAN )
	WHITE_CYAN	= pairtoattr( 55, COLOR_WHITE, COLOR_CYAN )

	BLACK_WHITE	= pairtoattr( 56, COLOR_BLACK, COLOR_WHITE )
	RED_WHITE	= pairtoattr( 57, COLOR_RED, COLOR_WHITE )
	GREEN_WHITE	= pairtoattr( 58, COLOR_GREEN, COLOR_WHITE )
	YELLOW_WHITE	= pairtoattr( 59, COLOR_YELLOW, COLOR_WHITE )
	BLUE_WHITE	= pairtoattr( 60, COLOR_BLUE, COLOR_WHITE )
	MAGENTA_WHITE	= pairtoattr( 61, COLOR_MAGENTA, COLOR_WHITE )
	CYAN_WHITE	= pairtoattr( 62, COLOR_CYAN, COLOR_WHITE )
	WHITE_WHITE	= pairtoattr( 63, COLOR_WHITE, COLOR_WHITE )


	colorset = OK
	return
	end


************************************************************************

	integer*4 function pairtoattr( pairnum, fg, bg )

	implicit none
	
	include '../../curses.fh'	
	include 'colorset.fh'

	integer*4 pairnum, fg, bg

	if ( init_pair(pairnum, fg, bg ) .eq. OK ) then
		pairtoattr =	COLOR_PAIR(pairnum)
	else
		pairtoattr =	A_NORMAL
	endif

	colorattr(pairnum+1) = pairtoattr

	return
	end
