* wrs (8/94)
*  demo showing some of the features of F-curses
*
	implicit none

	include '../../curses.fh'
	include 'colorset.fh'

	integer retval, attr1, attr2, attr3, win1, win2, win3, win4, win5,
&		char1, char2, x, y, i, shadow, pop1, srow, scol, tmpitem

	character * 50 menulist(5)

*	a couple utility funcions
	integer poptext
	integer popupmenu
	integer popform
	integer screenimage

*       field values to pass to the popform routine

	character*30 fldlabel(20)
	integer*4    fldly(20), fldlx(20)
	integer*4    fldypos(20), fldxpos(20), fldlen(20), fldwin(20)
	character    fldtype(20)*1, fldinit(20)*16, fldret(20)*16

*****************************************************

* startup screen uses standard fortran read/writes...

	write(*,'(60(/))')
	write(*,'(A,//)') '       Attention FORTRAN programmers!'
	
	print *,' Perhaps you''ve been using FORTRAN for years or have just learned'
	print *,' it recently after inheriting the maintenance of a large FORTRAN'
	print *,' program that has been around for some time.'
	print *,' '
**	print *,' If you are like the programmers at SCC who use the language'
**	print *,' frequently, you probably find one of its most significant drawbacks'
	print *,' Whichever the case, you''ve likely discovered that a major drawback'
	print *,' is the limited screen writing capability.  The kind of scrolling'
	print *,' text that you see here is about as good as it gets...'
	print *,' '
	print *,' '
	print *,' '
	print *,' Press <Enter> to continue.'
	print *,' '
	print *,' '
	print *,' '
	print *,' '
	read(*,*)

**	write(*,'(60(/))')
**	print *,' While it''s true that certain FORTRAN compilers come with proprietary'
**	print *,' screen writing libraries, very few have been developed for both UNIX'
**	print *,' systems and even fewer are compiler independent.  We at SCC Software'
**	print *,' felt there was a need to develop a general screen writing library'
**	print *,' for FORTRAN compilers on UNIX systems that would work on a variety'
**	print *,' of character-based terminals.'
**	print *,' '
**	print *,' The result of that effort is F-curses, a library package that '
**	print *,' accesses the power of the C library "curses" which is available'
**	print *,' on most UNIX systems.'
**	print *,' '
**	print *,' '
**	print *,' Press <Enter> to continue.'
**	read(*,*)
	
***********************************************************************
*  this is where it gets good, from here on out we're in curses mode

	call initscr()

*  Set Desired Screen Modes
*  don't echo keystrokes, get keystrokes immediately,
*  interpret special keys (function keys)

	call noecho()
	call cbreak()
	call keypad(stdscr,.TRUE.)	

* allow scrolling of the default window
	retval = scrollok( stdscr, .TRUE. )
	

*  only get a character after the screen has been completely written since
*  the last key action (Disable typeahead checking).

	retval = typeahead(-1)

* if the terminal being used has color capabilities, start them up

	if(has_colors()) then
		call start_color()

*		colorset is a F-curses utility routine provided as source that
*		pre-initializes a number of color pair combinations

		call colorset()
	endif

* clear the screen, won't happen until after first refresh call

	call clear()

* let's draw a box that starts small and gets bigger with a nice introduction

	call introbox()

* give a brief message about F-curses
*  use the poptext function (poptext.f) to display the message contained
*  in the file helpmes.1

	call clear()
	call refresh()

	if( has_colors() ) then
	  pop1 = poptext(NULL, 'intromes.1',
&		0,0, 0,9, 20, 71,
&		BLUE_WHITE, B_OR(YELLOW_RED,A_BOLD), 2,
&		'An Introduction to F-curses', 2, .FALSE.,
&		RED_WHITE, GREEN_WHITE, WHITE_BLUE )
	else
	  pop1 = poptext(NULL, 'intromes.1',
&		0,0, 0,9, 20, 71,
&		A_NORMAL, A_BOLD, 2,
&		'An Introduction to F-curses', 2, .FALSE.,
&		-1, -1, -1 )
	endif

	call delwin(pop1)

	call mvaddstr( 22, 10, '<Press any key to continue>')
	retval = sgetch()

	call clear()
	call touchwin(stdscr)


* Begin describing F-curses highlights with multiple scrolling windows.

* first window (black on white surrounded by a white on blue border

	attr1 = A_STANDOUT
	attr2 = A_NORMAL
	if(has_colors()) then

		attr1 = WHITE_BLUE
		attr2 = BLACK_WHITE
	endif

	call attron(attr1)
	call rectangle( stdscr, 2, 4, 9, 45, 0,0,0,0,0,0,0,0)
	call wnoutrefresh(stdscr)
	call attrset(0)

	win1 = newwin( 6, 40, 3, 5 )
	retval = wbkgd( win1, attr2 )

* allow scrolling in the window

	retval = scrollok( win1, .TRUE. )
	call wrefresh(win1)

	retval = wmove( win1, 0, 0 )
	call wprintw( win1, 'As you will see, the F-curses library' )
	call wprintw( win1, 'gives you some very attractive screen' )
	call wprintw( win1, 'writing capabilities.' )
	call wrefresh(win1)

*	pause for 3 seconds

	call napms(3000)

*  a second window listing features

	attr1 = A_NORMAL
	attr2 = A_REVERSE
	if(has_colors()) then
		attr1 = BLACK_RED
		attr2 = BLACK_GREEN

*		- if color
*		before displaying the second window, create an offset shadow

		win2 = newwin( LINES-5, 30, 4, 49)
		retval = wbkgd( win2, B_OR(attr1,ACS_BOARD) )
		call overwrite(win2,stdscr)
		call wnoutrefresh(stdscr)
		call delwin(win2)
	endif

	win2 = newwin( LINES-5, 30, 3, 48 )
	retval = wbkgd( win2, attr2 )
	retval = scrollok( win2, .TRUE. )
	call wrefresh(win2)

	call mvwprintw(win2, 1,0, 'Like cursor position control.' )
	call wrefresh(win2)

	call napms(500)

	call attrset(0)
	call mvaddstr( 17, 20, 'I''m here.' )
	call refresh()
	call napms(500)
	call mvaddstr( 14,3, 'NO, OVER HERE.')
	call refresh()
	call napms(500)
	call mvaddstr( 21,10, 'Who''s on first?')
	call refresh()
	call napms(500)
	call mvaddstr( 19,7, 'Well, Isn''t that special.')
	call refresh()
	call napms(1000)
	call mvaddstr( LINES-1, 2, '<Press any key to continue>')
	call refresh()
	call flushinp()
	retval = sgetch()

	call eraseblock( stdscr, 14, 0, LINES-1, 45 )
	call wnoutrefresh(stdscr)

	call wprintw(win2, ' ')
	call wprintw(win2, 'Text attributes.' )
	call wrefresh(win2)

	call napms(500)

	call attron(A_BOLD)
	call mvaddstr( 17, 20, 'I''m here.' )
	call attroff(A_BOLD)
	call attron(A_STANDOUT)
	call mvaddstr( 14,3, 'NO, OVER HERE.')
	call attroff(A_STANDOUT)
	call attron(A_BLINK)
	call mvaddstr( 21,10, 'Who''s on first?')
	call attroff(A_BLINK)
	call attron(A_DIM)
	call mvaddstr( 19,7, 'Well, Isn''t that special.')
	call attroff(A_DIM)
	call mvaddstr( LINES-1, 2, '<Press any key to continue>')
	call refresh()
	call flushinp()
	retval = sgetch()

	call eraseblock( stdscr, 14, 0, LINES-1, 45 )
	call wnoutrefresh(stdscr)

	call wprintw(win2, ' ')
	call wprintw(win2, 'Line drawing and timing' )
	call wprintw(win2, 'control.')
	call wrefresh(win2)

	call napms(700)

* draw a tic-tac-toe box and play a game

	call attron(A_BOLD)
	call rectangle( stdscr, 14, 15, 20, 27, 0,0,0,0,0,0,0,0)
	call move( 16, 15 )
	call hline( ACS_DHLINE, 13 )
	call move( 18, 15 )
	call hline( ACS_DHLINE, 13 )
	call move( 14, 19 )
	call vline( ACS_DVLINE, 7 )
	call move( 14, 23 )
	call vline( ACS_DVLINE, 7 )
	call mvaddch( 16, 19, ACS_DPLUS )
	call mvaddch( 16, 23, ACS_DPLUS )
	call mvaddch( 18, 19, ACS_DPLUS )
	call mvaddch( 18, 23, ACS_DPLUS )
	call mvaddch( 14, 19, ACS_DTTEE )
	call mvaddch( 14, 23, ACS_DTTEE )
	call mvaddch( 20, 19, ACS_DBTEE )
	call mvaddch( 20, 23, ACS_DBTEE )
	call mvaddch( 16, 15, ACS_DLTEE )
	call mvaddch( 18, 15, ACS_DLTEE )
	call mvaddch( 16, 27, ACS_DRTEE )
	call mvaddch( 18, 27, ACS_DRTEE )
	call attroff( A_BOLD )

	call refresh()

	call napms(500)

	attr1 = A_NORMAL
	attr2 = A_NORMAL
	if(has_colors()) then
		attr1 = RED_BLACK
		attr2 = CYAN_BLACK
	endif

	char1 = B_OR( attr1, ICHAR('X'))
	char2 = B_OR( attr2, ICHAR('O'))

	call mvaddch(17,21, char1)
	call refresh()
	call napms(500)
	call mvaddch(15,25, char2)
	call refresh()
	call napms(500)
	call mvaddch(19,25, char1)
	call refresh()
	call napms(500)
	call mvaddch(15,17, char2)
	call refresh()
	call napms(500)
	call mvaddch(15,21, char1)
	call refresh()
	call napms(500)
	call mvaddch(19,21, char2)
	call refresh()
	call napms(500)
	call mvaddch(17,17, char1)
	call refresh()
	call napms(500)
	call mvaddch(17,25, char2)
	call refresh()
	call napms(500)
	call mvaddch(19,17, char1)
	call refresh()
	call napms(500)


	call mvaddstr( LINES-1, 2, '<Press any key to continue>')
	call refresh()

	call flushinp()
	retval = sgetch()

	call eraseblock( stdscr, 14, 0, LINES-1, 45 )
	call wnoutrefresh(stdscr)

*	demonstrate overlapping windows

	call wprintw(win2, ' ')
	call wprintw(win2, 'Unlimited overlapping windows.' )
	call wrefresh(win2)

	call napms(700)

	attr1 = A_NORMAL
	attr2 = A_REVERSE
	attr3 = A_NORMAL
	if(has_colors()) then
		attr1 = B_OR( WHITE_RED, A_BOLD )
		attr2 = BLACK_CYAN
		attr3 = BLACK_MAGENTA
	endif

	win3 = newwin(5,15, 14, 7)
	retval = wbkgd( win3, attr1, ICHAR(' '))
	retval = wborder( win3, 0,0,0,0,0,0,0,0)
	call mvwaddstr( win3, 2, 6, 'POP')
	call wrefresh(win3)
	call napms(500)

	win4 = newwin(5,15, 17, 12)
	retval = wbkgd( win4, attr2, ICHAR(' '))
	retval = wborder( win4, 0,0,0,0,0,0,0,0)
	call mvwaddstr( win4, 2, 6, 'POP')
	call wrefresh(win4)
	call napms(500)

	win5 = newwin(5,15, 15, 15)
	retval = wbkgd( win5, attr3, ICHAR(' '))
	retval = wborder( win5, 0,0,0,0,0,0,0,0)
	call mvwaddstr( win5, 2, 6, 'POP')
	call wrefresh(win5)
	call napms(500)


	call mvaddstr( LINES-1, 2, '<Press any key to continue>')
	call refresh()

	call flushinp()
	retval = sgetch()

	call eraseblock( stdscr, 14, 0, LINES-1, 45 )
	call delwin(win3)
	call delwin(win4)
	call delwin(win5)

*	after the three box windows have been deleted, their image still
*	remains on the screen until we force the other windows on the
*	screen to overwrite the current image

	call touchwin(stdscr)
	call wnoutrefresh(stdscr)
	call touchwin(win1)
	call wnoutrefresh(win1)
	call touchwin(win2)
	call wnoutrefresh(win2)
	call doupdate()


*	demonstrate scrolling

	call wprintw(win2, ' ')
	call wprintw(win2, 'Window Scrolling.' )
	call wrefresh(win2)

	call napms(700)

*   copy the current screen image onto the standard window, then scroll it,
*   but first save a copy of what the screen currently looks like using
*   a general purpose function "screenimage"

	shadow = screenimage( NULL, curscr )
	call overwrite( curscr, stdscr )

	if( has_colors() ) then
		call attron(BLACK_YELLOW)
	else
		call attron(A_REVERSE)
	endif

*	have a happy face scroll onto the screen

	do i = 1,25
		call scroll(stdscr)
		call move( LINES-1,35 )
		if( i .eq. 12 ) call addstr(' _   _ ')
		if( i .eq. 13 ) call addstr(' o   o ')
		if( i .eq. 14 ) call addstr('   V   ')
		if( i .eq. 15 ) call addstr('  \_/  ')
		if( i .eq. 16 ) call addstr('       ')
		call refresh()
*		call napms( 100 )
	end do
	call attrset( 0 )

	call mvaddstr( LINES-1, 2, '<Press any key to continue>')
	call refresh()

	call flushinp()
	retval = sgetch()

*	restore the screen to what it was before the scrolling.

	call touchwin(shadow)
	call wrefresh(shadow)

	call napms(200)

*	mention the color support if the screen has the capability

	if ( has_colors() ) then
		call wprintw(win2, ' ')
		call wprintw(win2, 'OH YEAH, AND THERE''S COLOR' )
		call wprintw(win2, 'SUPPORT SUPPORT AS WELL')
		call wrefresh(win2)
		call wattron(win1, MAGENTA_WHITE)
	endif

	call napms(1000)

*	now show some higher level capability examples

	do i = 1,200
		call waddch(win1,'-')
		call wrefresh(win1)
		call napms(5)
	end do
	
	call werase(win1)
	call werase(win2)
	call mvwaddstr( win1, 1, 1, 'Here are some of the things you can')
	call mvwaddstr( win1, 2, 1, 'do with these capabilities...')
	call wnoutrefresh(win1)
	call wnoutrefresh(win2)
	call doupdate()


*	run a dummy simulation program (mysim)

	call napms(2000)
	call wprintw(win2, ' ')
	call wprintw(win2, 'Report status information for' )
	call wprintw(win2, 'a numerical simulation.' )
	call wrefresh(win2)

	call delay_output(3000)

	shadow = screenimage( shadow, curscr )
	call flushinp()

	call mysim()

	call touchwin( shadow )
	call wrefresh(shadow)

*	demonstrate a pop-up help screen

	call napms(1000)
	call wprintw(win2, ' ')
	call wprintw(win2, 'Provide online help for the' )
	call wprintw(win2, 'user.' )
	call wrefresh(win2)
	call napms(2000)

	srow = 0
	scol = 0
	if( has_colors() ) then
	  pop1 = poptext(NULL, 'helpmes.1', 
&		srow,scol, 0,0,0,0,
&		BLACK_WHITE, B_OR(YELLOW_RED,A_BOLD), 2,
&		'Help Screens Made Easy', 2, .TRUE.,
&		RED_WHITE, GREEN_WHITE, WHITE_BLUE )
	else
	  pop1 = poptext(NULL, 'helpmes.1', 
&		srow,scol, 0,0,0,0, 
&		A_NORMAL, A_BOLD, 2,
&		'Help Screens Made Easy', 2, .TRUE.,
&		-1,-1,-1 )
	endif


*	demonstrate a pop-up menu

	call napms(1000)
	call wprintw(win2, ' ')
	call wprintw(win2, 'Give the user some choices.' )
	call wrefresh(win2)
	call napms(2000)

	menulist(1) = 'Proceed with Demo'
	menulist(2) = 'Return to the MYSIM Program'
	menulist(3) = 'Return to the Help Screen'

*	popupmenu is another utility routine (popmenu.f) provided with
*	F-curses.

	call flushinp()
	tmpitem = 1
	if( has_colors() ) then
		retval = popupmenu( menulist, 3, 15, 10,
&			RED_WHITE, WHITE_RED,
&			tmpitem, 'What Do You Want to Do Next?', -1, .false. )
	else
		retval = popupmenu( menulist, 3, 15, 10,
&			A_NORMAL, A_REVERSE,
&			tmpitem, 'What Do You Want to Do Next?', -1, .false. )
	endif

	if( retval .gt. 1 ) then
		if( retval .eq. 2 ) then
			shadow = screenimage( shadow, curscr )
			call flushinp()
			call mysim()
			call touchwin( shadow )
			call wrefresh(shadow)
		else
			if( has_colors() ) then
			  pop1 = poptext(pop1, 'helpmes.1', 
&				srow,scol, 0,0,0,0,
&				BLACK_WHITE, B_OR(YELLOW_RED,A_BOLD), 2,
&				'Help Screens Made Easy', 2, .TRUE.,
&				RED_WHITE, GREEN_WHITE, WHITE_BLUE )
			else
			  pop1 = poptext(pop1, 'helpmes.1', 
&				srow,scol, 0,0,0,0, 
&				A_NORMAL, A_BOLD, 2,
&				'Help Screens Made Easy', 2, .TRUE.,
&				-1,-1,-1 )
			endif
		endif
	endif

*	now that we will no longer display the help window, delete the memory
*	reserved for it.

	call delwin(pop1)


*	demonstrate a simple input data form

	call napms(1000)
	call wprintw(win2, ' ')
	call wprintw(win2, 'Facilitate Data Entry.' )
	call wrefresh(win2)
	call napms(2000)

*	pass the different field definitions for the form

	fldlabel(1) = 'PLANT NAME'
	fldly(1)    = 2
	fldlx(1)    = 1
	fldypos(1)  = 3
	fldxpos(1)  = 1
	fldlen(1)   = 14
	fldtype(1)  = 's'
	fldinit(1)  = 'Four Corners'

	fldlabel(2) = 'START END'
	fldly(2)    = 1
	fldlx(2)    = 20
	fldlen(2)   = 0

	fldlabel(3) = 'YEAR'
	fldly(3)    = 2
	fldlx(3)    = 20
	fldypos(3)  = 3
	fldxpos(3)  = 20
	fldlen(3)   = 4
	fldtype(3)  = 'i'
	fldinit(3)  = '1980'

	fldlabel(4) = 'YEAR'
	fldly(4)    = 2
	fldlx(4)    = 26
	fldypos(4)  = 3
	fldxpos(4)  = 26
	fldlen(4)   = 4
	fldtype(4)  = 'i'
	fldinit(4)  = '2020'

	fldlabel(5) = 'Forced Outage Rate (%/yr)'
	fldly(5)    = 5
	fldlx(5)    = 1
	fldypos(5)  = 5
	fldxpos(5)  = 27
	fldlen(5)   = 5
	fldtype(5)  = 'r'
	fldinit(5)  = '12.1'

	fldlabel(6) = 'Fuel Type'
	fldly(6)    = 6
	fldlx(6)    = 17
	fldypos(6)  = 6
	fldxpos(6)  = 27
	fldlen(6)   = 12
	fldtype(6)  = 's'
	fldinit(6)  = 'COAL'

	fldlabel(7) = 'Power Level Data'
	fldly(7)    = 8
	fldlx(7)    = 10
	fldlen(7)   = 0

	fldlabel(8) = 'Capacity (MW)'
	fldly(8)    = 10
	fldlx(8)    = 1
	fldlen(8)   = 0

	fldlabel(9) = ' LEVEL 1'
	fldly(9)    = 9
	fldlx(9)    = 15
	fldypos(9)  = 10
	fldxpos(9)  = 15
	fldlen(9)   = 8
	fldtype(9)  = 'r'
	fldinit(9)  = '175.'

	fldlabel(10) = ' LEVEL 2'
	fldly(10)    = 9
	fldlx(10)    = 24
	fldypos(10)  = 10
	fldxpos(10)  = 24
	fldlen(10)   = 8
	fldtype(10)  = 'r'
	fldinit(10)  = '350.'

	fldlabel(11) = ' LEVEL 3'
	fldly(11)    = 9
	fldlx(11)    = 33
	fldypos(11)  = 10
	fldxpos(11)  = 33
	fldlen(11)   = 8
	fldtype(11)  = 'r'
	fldinit(11)  = '525.'

	fldlabel(12) = ' LEVEL 4'
	fldly(12)    = 9
	fldlx(12)    = 42
	fldypos(12)  = 10
	fldxpos(12)  = 42
	fldlen(12)   = 8
	fldtype(12)  = 'r'
	fldinit(12)  = '700.'

	fldlabel(13) = 'Heat Rate '
	fldly(13)    = 11
	fldlx(13)    = 1
	fldlen(13)   = 0

	fldlabel(14) = ' (BTUs/KWh) '
	fldly(14)    = 12
	fldlx(14)    = 1
	fldlen(14)   = 0

	fldlabel(15) = ' '
	fldypos(15)  = 12
	fldxpos(15)  = 15
	fldlen(15)   = 8
	fldtype(15)  = 'r'
	fldinit(15)  = '13125.'

	fldlabel(16) = ' '
	fldypos(16)  = 12
	fldxpos(16)  = 24
	fldlen(16)   = 8
	fldtype(16)  = 'r'
	fldinit(16)  = '12500.'

	fldlabel(17) = ' '
	fldypos(17)  = 12
	fldxpos(17)  = 33
	fldlen(17)   = 8
	fldtype(17)  = 'r'
	fldinit(17)  = '12621.'

	fldlabel(18) = ' '
	fldypos(18)  = 12
	fldxpos(18)  = 42
	fldlen(18)   = 8
	fldtype(18)  = 'r'
	fldinit(18)  = '12714.'

	if( has_colors() ) then
		retval = popform('Power Plant Characteristics', 2,2,
&			B_OR(CYAN_YELLOW,A_BOLD), BLACK_YELLOW, WHITE_BLUE, 18, 1,
&			fldlabel, fldly, fldlx,
&			fldypos, fldxpos, fldlen, fldtype,
&			fldinit, fldret, fldwin)
	else
		retval = popform('Power Plant Characteristics', 2,2,
&			A_BOLD, A_NORMAL, A_REVERSE, 18, 1,
&			fldlabel, fldly, fldlx,
&			fldypos, fldxpos, fldlen, fldtype,
&			fldinit, fldret, fldwin)
	endif

*	the string "values" returned from popform are stored in the array
*	fldret.  You could now store them in permanent variables.

**	do i = 1,18
**		if( fldlen(i) .gt. 0 ) then
**			call mvaddstr( i,2, fldret(i) )
**		endif
**	end do

	call doupdate()

*	that's all to the demo for now, have some closing thoughts

	call napms(1000)
	call wprintw(win2, ' ')
	call wprintw(win2, 'And have a little fun, too...' )
	call wrefresh(win2)
	call napms(2000)

*	call the scatter routine the mixes up the screen

	call scatter()

	if( has_colors() ) then
	  pop1 = poptext(NULL, 'endmes.1',
&		0,0, 2,7, 21, 73,
&		BLUE_WHITE, B_OR(YELLOW_RED,A_BOLD), 2,
&		' ', 2, .FALSE.,
&		RED_WHITE, GREEN_WHITE, WHITE_BLUE )
	else
	  pop1 = poptext(NULL, 'endmes.1',
&		0,0, 2,7, 21, 73,
&		A_NORMAL, A_BOLD, 2,
&		' ', 2, .FALSE.,
&		-1, -1, -1 )
	endif

	call delwin(pop1)


* return terminal to normal mode and end F-curses

	call endwin()

	end

**************************************************************

* display a box with intro information that starts small and expands
* to its final size

	subroutine introbox()

	implicit none
	
	include '../../curses.fh'
	include 'colorset.fh'

	integer attr, retval, ny, nx, y, x, tx, bx, ty, by

* if colors are available, make box bright white on red

	attr = A_NORMAL
	if(has_colors()) then
		attr = WHITE_RED
	endif

	call attron(attr)

* draw a filled box, starting with a two by two and expanding to a
*  14 by 56

	do ny = 1,7

		ty = LINES/2 - ny
		by = LINES/2 + ny - 1

		do nx = 3, 0, -1
			tx = COLS/2 - (ny*4 - nx)
			bx = COLS/2 + (ny*4 - nx) - 1

*			the rectangle routine writes the border

			call rectangle( stdscr, ty, tx, by, bx, 0,0,0,0,0,0,0,0)

*			filling in the rectangle one character at a time is
*			each time is a little tedious, since many of the
*			characters were filled in by the smaller rectangles, but
*			this is only a slight penalty since the refresh() call
*			only changes those portions of the screen that have
*			changed.
	
			do y = ty+1, by-1
				do x = tx+1, bx-1
					retval = mvaddstr(y,x,' ')
				end do
			end do

			call refresh()
		end do

	end do

	retval = mvaddstr(LINES/2 - 4, COLS/2 - 6, 'Introducing' )
	call attron(A_BOLD)
	retval = mvaddstr(LINES/2 - 1, COLS/2 - 4, 'F-curses' )
	retval = mvaddstr(LINES/2 + 1, COLS/2 - 19, 'Character Graphics Library for FORTRAN')
	call refresh()

*       sleep for a second (1000 miliseconds)

	call napms(1000)

	call attroff(A_BOLD)
	call attron(A_BLINK)
	retval = mvaddstr(LINES/2 + 4, COLS/2-8, 'by Sierra Consulting Corp.' )

	call attrset( 0 )

* prompt for a response once the user has got their fill of the message

	call attron(A_STANDOUT)
	retval = mvaddstr(LINES/2+9, COLS/2-10, 'Press any key to continue...')
	call attroff(A_STANDOUT)
	call refresh()

	call flushinp()
	retval = sgetch()
	call clear()

	return
	end

***********************************************************************
* eraseblock
*  (like werase, but for a defined region of a window)

	integer*4 function eraseblock( win, ty, tx, by, bx )

	implicit none
	
	include '../../curses.fh'

*	positions are relative to the top corner of the window "win"

*	ty is the y coordinate of the top left corner of the erase region (0 min)
*	tx is the x coordinate of the top left corner of the erase region (0 min)
*	by is the y coordinate of the bottom left corner of the erase region
*	by is the x coordinate of the bottom left corner of the erase region

	integer*4  win, ty, tx, by, bx, x, y

	call wattrset( win, 0 )

	do x = tx, bx
		do y = ty, by
			call mvwaddch( win, y, x, ICHAR(' ') )
		end do
	end do

	return
	end
