*  MYSIM ---   Wade Schauer (9/93)

*  an imaginary simulation model (mysim) that shows how F-curses can be
*  used to create informative status screens for simulation models.

*  note: all of the calls to "napms" in this routine are to slow down
*        the rate at which screen information is updated (because this
*        simulation really isn't doing anything).   For a real simulation,
*        you wouldn't have the napms calls because they would slow it down
*        and just the process of doing real calulations would cause some
*        elapsed time between messages to the screen.


	subroutine mysim()
	
	implicit none
	
	include '../../curses.fh'
	include 'colorset.fh'

	integer simwin, retval, inputwin, yrwin, wkwin, hrwin, costwin,
&		statwin, warnwin, errwin, i, seed, iy, iwk

	common /simwins/ simwin, inputwin, yrwin, wkwin, hrwin, costwin,
&		statwin, warnwin, errwin

	real    random

	character  outstring*80

	data seed /34567/


	call clear()
	call touchwin( stdscr )
	call refresh()

	simwin = newwin( 7, 40, LINES/2-3, COLS/2-20 )

	if( has_colors() ) then
		call wbkgd( simwin, B_OR( WHITE_RED, A_BOLD ) )
	endif

	call box( simwin, 0,0 )
	call mvwaddstr( simwin, 3,4, 'Press Any Key to Begin Simulation')
	call wrefresh(simwin)
	retval = wgetch(simwin)
	call delwin(simwin)


	call attron(A_STANDOUT)
	call mvaddstr(1,COLS/2-4,' MYSIM ')

	call move( 22, COLS/2 - 15 )
	call addstr(' Press <CTRL>-x to Pause/Abort ')
	call refresh()

	call raw()
	call flushinp()
	call nodelay(stdscr, .TRUE.)

	simwin = newwin(20,78,2,1)
	
	if( has_colors() ) then
		call wbkgd( simwin, b_or( WHITE_BLUE, A_BOLD) )
	else
		call wbkgd( simwin, A_REVERSE )
	endif
	
	call wborder( simwin, ACS_DVLINE, ACS_DVLINE, ACS_DHLINE, ACS_DHLINE,
&		ACS_DULCORNER, ACS_DURCORNER, ACS_DLLCORNER, ACS_DLRCORNER )

	call mvwaddch( simwin, 10, 0, ACS_DLTEE )
	call whline( simwin, ACS_DHLINE, 76 )
	call mvwaddch( simwin, 10, 77, ACS_DRTEE )

	call wmove( simwin, 1,23)
	call wvline( simwin, ACS_DVLINE, 9 )
	call mvwaddch( simwin, 0,23, ACS_DTTEE )
	call mvwaddch( simwin, 10,23, ACS_DBTEE )

	call wrefresh( simwin )

	call mvwaddstr( simwin,1,2, 'INPUT/OUTPUT FILES:' )

	inputwin = derwin( simwin, 8,20, 2,2 )
	if( has_colors() ) then
		call wbkgd( inputwin, BLACK_WHITE )
	else
		call wbkgd( inputwin, A_NORMAL )
	endif
	call wrefresh( inputwin )

	call mvwaddstr( simwin,2,27, 'YEAR' )
	call mvwaddstr( simwin,4,27, 'WEEK' )
	call mvwaddstr( simwin,6,27, 'HOUR' )

	yrwin = derwin( simwin, 1,4, 2,32 )
	wkwin = derwin( simwin, 1,4, 4,32 )
	hrwin = derwin( simwin, 1,4, 6,32 )

	if( has_colors() ) then
		call wbkgd( yrwin, BLACK_WHITE )
		call wbkgd( wkwin, BLACK_WHITE )
		call wbkgd( hrwin, BLACK_WHITE )
	else
		call wbkgd( yrwin, A_NORMAL )
		call wbkgd( wkwin, A_NORMAL )
		call wbkgd( hrwin, A_NORMAL )
	endif

	call wrefresh( yrwin )
	call wrefresh( wkwin )
	call wrefresh( hrwin )

	call mvwaddstr( simwin,1,39, 'RUN STATUS:' )
	statwin = derwin( simwin, 6,35, 2,39 )
	if( has_colors() ) then
		call wbkgd( statwin, MAGENTA_WHITE )
	else
		call wbkgd( statwin, A_NORMAL )
	endif
	call wrefresh( statwin )

	call mvwaddstr( simwin,11,2, 'WARNINGS:' )
	warnwin = derwin( simwin, 3,74, 12,2 )
	if( has_colors() ) then
		call wbkgd( warnwin, RED_WHITE )
	else
		call wbkgd( warnwin, A_BOLD )
	endif
	call wrefresh( warnwin )

	call mvwaddstr( simwin,15,2, 'ERRORS:' )
	errwin = derwin( simwin, 3,74, 16,2 )
	if( has_colors() ) then
		call wbkgd( errwin, B_OR(RED_WHITE,A_BLINK) )
	else
		call wbkgd( errwin, B_OR(A_BOLD,A_BLINK) )
	endif
	call wrefresh( errwin )

	call wrefresh( simwin )

	call napms(1000)

	call winwrite( statwin, 'Beginning Simulation...', 1 )
	call napms(500)

	call abort(*9999)

	call wmove( inputwin, 0,0 )
	call wjustify( inputwin, '../basecase.DA', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '../basecase.LA', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '../basecase.MA', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '1sttest.AN', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '1sttest.WK', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '1sttest.ER', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '1sttest.DB', 1, .TRUE., .FALSE. )
	call wjustify( inputwin, '1sttest.MC', 1, .TRUE., .FALSE. )
	call wrefresh( inputwin )

	call abort(*9999)

	call napms(1000)
	call winwrite( statwin, 'Reading Input Data...', 1 )

	call mvwaddstr( simwin,9,39, 'Percent of Data Read:' )
	call wrefresh( simwin )
	costwin = derwin( simwin, 1,7, 9,60 )
	if( has_colors() ) then
		call wbkgd( costwin, B_OR( WHITE_RED, A_BOLD ) )
	else
		call wbkgd( costwin, A_BOLD )
	endif
	call wrefresh( costwin )

	do i = 1,100

		call abort(*9999)

		write( outstring, '(i3,'' %'')' ) i
		call winwrite( costwin, outstring(1:5), 2 )
		if( i .eq. 25 ) then
		  call winwrite( warnwin, 'Note: The format of data card 19 has changed.', 1)
		elseif( i .eq. 72 ) then
		  call winwrite( warnwin, 'WARNING: extremely high fuel price entered for fuel "OIL"', 1)
		  call winwrite( warnwin, '         (see the Error file)', 1)
		endif
		call napms(100)
	end do

	do iy = 1993, 2000

		write( outstring, '(i4)' ) iy
		call winwrite( yrwin, outstring(1:4), 2 )

		call winwrite( statwin, 'Escalating annual data...' , 1 )
                call napms(1000)

		call abort(*9999)

		do iwk = 1,52

		   write( outstring, '(i4)' ) iwk
                   call winwrite( wkwin, outstring(1:4), 2 )

		   call abort(*9999)

		   call winwrite( statwin, 'Updating weekly variables...' , 1 )
                   call napms(1000)

		   call abort(*9999)

		   call winwrite( statwin, 'Checking system reliability...' , 1 )
                   call napms(1000)

                   if( iwk .eq. 5 ) then
			call beep()
			write( outstring, '(''FATAL: Critical Reliability Shortage !'')' )
			call winwrite( errwin, outstring, 1 )
			call winwrite( errwin, '         (see the Error file)', 1)
			call move( 22, COLS/2 - 15 )
			call addstr('   Press any Key to Continue   ')
			call refresh()

			call nodelay(stdscr, .FALSE.)
			retval = sgetch()
			go to 9999
                   endif

		   call abort(*9999)

		   call winwrite( statwin, 'Beginning hourly system analysis...' , 1 )
                   call napms(500)

		   do i = 1, 168

		      call abort(*9999)

		      write( outstring, '(i4)' ) i
                      call winwrite( hrwin, outstring(1:4), 2 )

		      if( random(seed) .gt. 0.99 ) then
			write( outstring, '(''Warning: Generation shortage in hour '',i3,''.'')' ) i
			call winwrite( warnwin, outstring, 1 )
		      endif

		      call napms(10)

		   end do

		   call winwrite( statwin, 'Calculating weekly results...' , 1 )
                   call napms(1000)

		   if( iwk .eq. 1 ) then
		     write( outstring, '( ''  Costs for Week '',I2,''          ($1000)'')' ) iwk
		     call mvwaddstr( simwin,9,39, outstring(1:36) )
		     call touchwin( costwin )
		   else
		     write( outstring, '(I2)' ) iwk
		     call mvwaddstr( simwin,9,56, outstring(1:2) )
		   endif
		   call wrefresh( simwin )
		   write( outstring, '(f7.1)' ) 4000. + random(seed)*12345.
		   call winwrite( costwin, outstring, 2 )

		   call abort(*9999)

		end do
		call winwrite( statwin, 'Calculating annual results...' , 1 )
                call napms(1000)
	end do

 9999	continue
 
	call noraw()
	call cbreak()

*  you must close all subwindows before the parent (simwin)

*  the images from these windows will stay on the screen even after they are
*  deleted until other windows start overwriting the screen.

	call delwin( errwin )
	call delwin( warnwin )
	call delwin( statwin )
	call delwin( costwin )
	call delwin( hrwin )
	call delwin( wkwin )
	call delwin( yrwin )
	call delwin( inputwin )
	call delwin( simwin )

	call nodelay(stdscr, .FALSE.)

	return
	end


****************************************************************

	subroutine winwrite( win, str, opt )
	
	implicit none
	
	include '../../curses.fh'

	integer*4	win, opt
	character*(*)	str

	integer*4	retval, ymax, xmax

	call scrollok( win, .TRUE. )
	call scroll(win)

	retval = getmaxyx( win, ymax, xmax )
	call wmove( win, ymax-1, 0 )

	call wjustify( win, str, opt, .FALSE., .FALSE. )

	call wnoutrefresh( win )
	call move(0,0)
	call refresh()

	return
	end

****************************************************************

	subroutine abort( * )
	
	implicit none
	
	include '../../curses.fh'

	integer retval

	retval = sgetch()
	if( retval .eq. CTRL('X') ) then

		call beep()
		call move( 22, COLS/2 - 15 )
		call addstr('   Abort Simulation (Y/N) ?    ')
		call refresh()

		call nodelay(stdscr, .FALSE.)
		retval = sgetch()

		if( retval .eq. ichar('Y') .or. retval .eq. ichar('y') ) then
			return 1
		else
			call nodelay( stdscr, .TRUE.)
			call move( 22, COLS/2 - 15 )
			call addstr(' Press <CTRL>-x to Pause/Abort ')
			call refresh()
		endif
	endif

	return
	end
