* WRS -- 8/94
*	popform

	integer function popform( formtitle, minrow, mincol,
&		bordattr, background, fieldback, totf, firstfield,
&		fldlabel, fldly, fldlx,
&		ypos, xpos, fldlength, fldtype,
&		fldinit, fldret, fldwin)


* this routine displays input data fields within a form and does basic
* management of movement between form fields.  The user then enters the
* data required in the form.

* if popform returns ERR, then it failed

* if it returns 5, then the user pressed CTRL-A to abort the data entry
* if it returns OK, then the user pressed CTRL-X after entering the
* requested data.

* <ESC> restores a data field to its original value

* The calling program will need to ascertain if the
* data entered meets conditions other than correct type.  MORE WORK COULD
* BE DONE ON THIS ROUTINE TO HAVE IT DO MORE DATA CHECKING, HAVE IT PROVIDE
* SOME FIELD CHOISES, ETC.  BUT IT IS A GOOD START.

* ASSUMES THAT THE MAIN PROGRAM IS USING CURSES AND HAS SET
*     noecho()

*------------------------------------------------------------------------

* formtitle (character)
*	will be displayed at the top of the form in the upper border.  The
*	border is always a single line.

* minrow, mincol (integer)
*	the form will be size to accomodate the layout of the data fields
*	and their titles.  these coordinates position the upper left corner
*	of the form on the screen.  Minimum values 0,0.

* bordattr	(integer)
*	this contains the attribute that you want to use for the form
*	border (which includes the title).  It could be a color pair.

* background	(integer)
*	this contains the attribute that you want to use as the background for
*	the form.  It could be a color pair.

* fieldback	(integer)
*	this contains the attribute that you want to use for the input data
*	fields (this should contrast the form background if possible).
*	It could be a color pair.

* totf	(integer)
*	many of the values passes to this routine are arrays so that there
*	can be multiple fields passed to the form.  totf tells popform
*	hom many array elements are to be used.

* firstfield (integer)
*	which field will have the cursor and be open for editing when
*	the form is first opened.  This value will also contain the field
*	number of the last field used when the form is closed.

* fldlabel	(character array)
*	there are two parts to each field, its title and the area where the
*	area where the data will be entered.  The field title (fldlabel)
*	is used to label the data contained in the field.

* fldly, fldlx	(integer arrays)
*	beginning coordinates for the field label (fldlabel)

* ypos, xpos (integer arrays)
*	beginning coordinates of the field

* fldlength (integer array)
*	length of the data field.  the user can enter a string in the
*	field up to this length.

* fldtype (character array)
*	type of data entered in the field
*	's' - string
*	'r' - real (floating point)
*	'i' - integer

* fldinit (character array)
*	a string containing an initial value for the field.

* fldret (character array)
*	value read and returned to the calling program


	implicit none

	include '../../curses.fh'

	character*(*)	formtitle
	integer*4	minrow, mincol
	integer*4	bordattr, background, fieldback, totf, firstfield
	character*(*)	fldlabel(totf)
	integer*4	fldly(totf), fldlx(totf)
	integer*4	ypos(totf), xpos(totf)
	integer*4	fldlength(totf)
	character*(*)	fldtype(totf)
	character*(*)	fldinit(totf), fldret(totf)

*	the program also needs to be passed an array of integers to hold
*	the memory addresses for the fields that will be created

	integer*4	fldwin(totf)

*	shadow holds the contents of the screen that will be hidden
*	when another window is displayed

	integer shadow

*	"screenimage" utility function (scrimage.f)

	integer screenimage

*	"lastchr" function

	integer lastchr

*	"isreal" and "isint" functions

	logical isreal, isint

*---------------------------------

	character*80 tmpstr

	integer formwin, actfld

	integer fmaxy, fmaxx, ll, i, retval, endstr, fn, ch, y, positn

	logical insmode

*******************************************************************

	popform = ERR

	fn = firstfield

	formwin = NULL
	do i = 1,totf
		fldwin(i) = NULL
	end do

*	invoke raw mode so that the delete character won't kill the program
*		(this should be redundant -- most interactive curses programs
*		would have already called this)

	call raw()

*	determine the form size from the layout of the fields

	fmaxy = 0
	fmaxx = 0
	do i = 1, totf
		ll = lastchr( fldlabel(i) )
		if( ll .gt. 0 ) then
			fmaxy = max( fmaxy, fldly(i) )
			fmaxx = max( fmaxx, fldlx(i) + ll )
		endif

		if( fldlength(i) .gt. 0 ) then
			fmaxy = max( fmaxy, ypos(i) )
			fmaxx = max( fmaxx, xpos(i) + fldlength(i) )
		endif

	end do

	fmaxy = min( fmaxy + 3, LINES )
	fmaxx = min( max(fmaxx + 3, 36), COLS )


*	create the form

	formwin = newwin( fmaxy, fmaxx, minrow, mincol )
	if( formwin .eq. NULL ) then
		call popferr( ' could not create form window ' )
		go to 1000
	endif
	call keypad(formwin,.TRUE.)	

*	store a copy of the screen before the form is displayed

	shadow = screenimage( NULL, formwin )
	if( shadow .eq. NULL ) then
		call popferr( ' couldn''t save screen image ' )
		return
	endif

	call wbkgd( formwin, background )

	call wattron( formwin, bordattr )
	call box( formwin, 0,0 )
	retval = mvwaddstr( formwin, fmaxy-1,1, 'CTRL-X closes/saves, CTRL-A aborts ' )

*	display the form title

	endstr = lastchr(formtitle)
	if( endstr .gt. 0 ) then
		endstr = min( endstr, fmaxx-1 )
		call wmove( formwin, 0, (fmaxx/2-endstr/2-1) )
		call waddstr( formwin, ' ' )
		call waddstr( formwin, formtitle(:endstr) )
		call waddstr( formwin, ' ' )
	endif
	call wrefresh( formwin )

	call wattrset( formwin, 0 )

*	now place the fields and their titles, the fields will be derived
*	subwindows.  Put any initial field values in the fields.

	do i = 1, totf

		write(tmpstr,'(i3)') i
		ll = lastchr( fldlabel(i) )
		if( ll .gt. 0 ) then
		  if( mvwaddnstr( formwin, fldly(i)+1, fldlx(i)+1, fldlabel(i), ll )
&			.eq. ERR ) then
			call popferr( ' could not place label for field '// tmpstr(:3) )
			go to 1000
		  endif
		  call wnoutrefresh( formwin )
		endif

*		if a field length of zero is passed for a particular field,
*		then don't create the field (assume that we are just using
*		the field label for labeling purposes)

		if( fldlength(i) .gt. 0 ) then

*                 when the form is built, initialize the return strings to
*                 be equal to the initial field strings.
		  fldret(i) = fldinit(i)

		  fldwin(i) = derwin( formwin, 1,fldlength(i), ypos(i)+1, xpos(i)+1 )
		  if( fldwin(i) .eq. NULL ) then
			call popferr( ' could not open field window '// tmpstr(:3) )
			go to 1000
		  endif

		  call wbkgd( fldwin(i), fieldback )

		  call putfield( fldwin(i), fldinit(i), fldtype(i) )

		  call wmove( fldwin(i), 0,0 )

		  call wnoutrefresh( fldwin(i) )

		endif

	end do

*	have the form in insert mode by default, in which case the letter
*	'I' appears in the upper right corner.

	insmode = .true.
	call mvwaddch( formwin, 0, fmaxx-2, B_OR( ICHAR('I'),bordattr ) )

*	input loop that processes input characters in a field and movement
*	between fields

	firstfield = 0
	ch = NULL

	do while( .true. )

  500		continue

*		make sure we are not in a zero length field

		do while ( fldlength(fn) .le. 0 )

			if( ch .eq. KEY_HOME ) then
				fn = fn + 1
			elseif( ch .eq. KEY_END ) then
				fn = fn - 1
			elseif( firstfield .le. fn ) then
				fn = fn + 1
			else
				fn = fn - 1
			endif

			if( fn .le. 0 ) then
				fn = totf
				firstfield = fn + 1
			elseif( fn .gt. totf ) then
				fn = 1
				firstfield = 0
			endif

		end do

		actfld = fldwin(fn)
		call getyx( actfld, y, positn )
		call wmove( actfld, 0, positn )
		call wrefresh(formwin)
		call wrefresh(actfld)

  		ch = wgetch(formwin)
  		if( wgetbkgd(actfld) .ne. fieldback ) then
			call wbkgd( actfld, fieldback )
		endif

* abort without saving changes to the data
		if ( ch .eq. CTRL('A') ) then

			popform = 5
			firstfield = fn
			go to 1000

* there are several keys that cause us to leave a field, it is at that
* point that we check the data for correct type

		elseif ( ch .eq. KEY_NEWLN .or. ch .eq. KEY_CR .or.
&		     ch .eq. KEY_UP .or. ch .eq. KEY_DOWN .or.
&		     ch .eq. KEY_TAB .or. ch .eq. KEY_BTAB .or.
&		     ch .eq. KEY_HOME .or. ch .eq. KEY_END .or.
&		     ch .eq. CTRL('X') ) then

			firstfield = fn

* 			only need to check if not a string
			if( fldtype(fn) .ne. 's' ) then

*			  read the data out of the field window
			  retval = mvwinnstr( actfld, 0,0, tmpstr, fldlength(fn) )

			  if( fldtype(fn) .eq. 'r' ) then
			    if( .not. isreal( tmpstr ) ) then
			  	call beep()
				call wbkgd( actfld, b_or(fieldback,A_BLINK) )
				go to 500
			    else
				call putfield( actfld, tmpstr, 'r' )
			    endif
			  else
			    if( .not. isint( tmpstr ) ) then
			  	call beep()
				call wbkgd( actfld, b_or(fieldback,A_BLINK) )
				go to 500
			    else
				call putfield( actfld, tmpstr, 'i' )
			    endif
			  endif

			endif

*			things checked out, now let's leave this field

			call wmove( actfld, 0,0 )
			call wrefresh(actfld)

*                  these keys leave the field and move to the next field

			if ( ch .eq. KEY_NEWLN .or. ch .eq. KEY_CR .or.
&			     ch .eq. KEY_DOWN .or. ch .eq. KEY_TAB ) then

				fn = fn + 1
				if ( fn .gt. totf ) then
					fn = 1
					firstfield = 0
				endif

*                  these keys leave the field and move to the previous field
*                     ( up arrow and shifted TAB )

			elseif ( ch .eq. KEY_UP .or. ch .eq. KEY_BTAB ) then

				fn = fn - 1
				if( fn .lt. 1 ) then
					fn = totf
					firstfield = fn + 1
				endif

*		   home key takes you to the first field

			elseif ( ch .eq. KEY_HOME ) then

				fn = 1

*		   end key takes you to the last field

			elseif ( ch .eq. KEY_END ) then

				fn = totf


*                   CTRL-X tells indicates that the user wants to close
*                   the form and pass the new data back to the calling
*                   routine.

			elseif ( ch .eq. CTRL('X') ) then

*                               store the field data in the return variable array
				do i = 1, totf
					if( fldlength(i) .gt. 0 ) then
						call mvwinstr( fldwin(i), 0,0, fldret(i) )
					endif
				end do

				popform = OK
				go to 1000
			endif

* the rest have to do with stuff within a field

		else

*          restore the value of the current field to its original value

		  if ( ch .eq. KEY_ESC ) then

			call putfield( actfld, fldinit(fn), fldtype(fn) )
			call wmove( actfld, 0,0 )
			call beep()


*	  arrow left in the field

		  elseif ( ch .eq. KEY_LEFT .or. ch .eq. KEY_BACKSPACE ) then

			positn = positn - 1
			if( positn .lt. 0 ) then
				call beep()
			else
				call wmove( actfld, 0, positn )

*				delete character if backspace

				if( ch .eq. KEY_BACKSPACE ) then
					call wdelch( actfld )
				endif

			endif

*	  arrow right in the field

		  elseif ( ch .eq. KEY_RIGHT ) then

			positn = positn + 1
			if( positn .ge. fldlength(fn) ) then
				call beep()
			else
				call wmove( actfld, 0, positn )
			endif

*	  delete a character

		  elseif ( ch .eq. KEY_DELETE .or. ch .eq. KEY_DC ) then

			call wdelch( actfld )

*	  toggle Insert mode

		  elseif ( ch .eq. KEY_IC ) then			

			if( insmode ) then
				insmode = .false.
				call mvwaddch( formwin, 0, fmaxx-2, B_OR( ACS_HLINE, bordattr ) )
			else
				insmode = .true.
				call mvwaddch( formwin, 0, fmaxx-2, B_OR( ICHAR('I'), bordattr ) )
			endif

*	  OK, let's get all valid ASCII characters (non-control characters)
*	  and process them.

		  elseif ( ch .ge. ICHAR(' ') .and. ch .le. ICHAR('~') ) then

*			if first character in a numeric field or if pressing the
*			space bar when first entering a string field, the
*			field is cleared.

			if( fn .ne. firstfield .and.
&			 ( fldtype(fn) .ne. 's' .or. ch .eq. ICHAR(' ') ) ) then

				call wclrtoeol( actfld )
			endif

*			if in insert mode, add the character and then
*			advance the cursor, otherwise overwrite the
*			existing character and advance the cursor.

			if( insmode ) then
				call winsch( actfld, ch )
				call wmove( actfld, 0, positn+1 )
			else
				call waddch( actfld, ch )
			endif

		  else
		  	call beep()
		  	go to 500
		  endif

		  firstfield = fn

		endif

	end do

 1000	continue

	do i = 1, totf
		if( fldwin(i) .ne. NULL ) then
			call delwin( fldwin(i) )
		endif
	end do
	if( formwin .ne. NULL ) then
		call delwin( formwin )
	endif

*	refresh and delete the shadow window (use wnoutrefresh so that the
*	original screen isn't returned until the next call to doupdate() )

	call touchwin( shadow )
	call wnoutrefresh( shadow )
 	call delwin(shadow)

*	flush any input keystrokes and return

	call flushinp()
 	return

  	end

*-------------------------------------------------------------

*  dignostic error message

	subroutine popferr( errmes )
	
	implicit none
	
	include '../../curses.fh'
	
	character*(*) errmes
	integer*4 retval

*	print the message in the upper left corner of the screen and wait
*	for a keystroke (won't wait if "nodelay" has been set true on
*	stdscr).

	call mvaddstr(0,0, errmes)
	call refresh()

	retval=sgetch()

	return
	end

*-------------------------------------------------------------

*  put a string in a field window, if the type specified is integer or
*  real, then right justify it.

	subroutine putfield( win, str, stype )
	
	implicit none
	
	include '../../curses.fh'

	integer*4	win
	character*(*)	str
	character*(*)	stype

*	"wjustify" is another utility routine

	if( stype .eq. 'r' .or. stype .eq. 'i' ) then
*		real or integer, right justify
		call wjustify( win, str, 2, .FALSE., .FALSE.)
	else
		call wjustify( win, str, 1, .FALSE., .FALSE.)
	endif

	return
	end
