* WRS -- 9/93
*	poptext

	integer function poptext( pad, filename, 
&		pminrow, pmincol, sminrow, smincol, smaxrow, smaxcol,
&		background, bordback, bc,
&		helptitle, leftmargin, interact,
&		bld, und, rev )

* A more sophisticated routine that allows the programmer to specify the
* layout of a text window and allows the user to scroll or page through
* it.  The text is read in from a file.

* returns a pointer to the help window if successful, otherwise it returns
* null.

* pad	(integer)
*	a pointer to the help window.   Pass NULL to "poptext" when
*       you call it for a particular help window the first time.  It will then
*	store the contents of the file "filename" into a new window and return
*	a pointer to the new window.  Pass this pointer as "pad" the next time
*	you want to display the same help window.
*	NOTE: THESE HELP WINDOWS TAKE UP MEMORY -- each text character read
*	will take up two or four bytes -- SO DELETE THEM WITH THE
*	"delwin" routine if you don't plan to display them again or if you
*	must free up the memory.  You can always rebuild them again by calling
*	"poptext" with a NULL pointer.

* filename	(character)
*	the name of the file containing the text you want to go into a
*	help window.  The file will be scanned once to determine the
*	appropriate memory size for storing the text, and then read into
*	a window pad (can be larger than screen size).  THIS READ WILL ONLY
*	OCCUR WHEN THE VALUE FOR "pad" is NULL.  See the discussion of the
*	default directory location of this text file (helpdir).

* pminrow	(integer)
*	the help text will be stored in a pad that is probably larger than the
*	screen and most likely larger than the help window size that will
*	be displayed on the screen.  pminrow tells "poptext" which line of
*	the text should be the top line in the displayed window ( pass 0 to
*	get the first line ).  If the user uses arrow keys to move through
*	the text, then pminrow will change to the new coordinates displayed
*	(returned to the calling routine).

* pmincol	(integer)
*	like "pminrow" above, except for specifying the first column of the
*	text displayed ( 0 is the minimum )

* sminrow, smincol, smaxrow, smaxcol (integer)
*	these are used to specify the position of the upper and lower corners 
*	of the help window on the terminal screen (the text specified by
*	pminrow and pmincol will then be positioned in this window).  If all
*	zeros are entered, then the window will be screen size.
*	Minimum size is 5 lines and 20 columns.

* background	(integer)
*	this contains the attribute that you want to use as the background for
*	the help window text ( could be a color pair, but the calling routine
*	should be the one that determines if colors are O.K. and pass the
*	appropriate attributes )

* bordback	(integer)
*	this contains the attribute that you want to use for the border that
*	surrounds the help window.  if the value passed is -1, then use the
*	background attribute.

* bc	(integer)
*	a flag specifying the border character to use.
*		< 0 = no border will be drawn (more space for the window)
*		  0 = blank character
*		  1 = single line characters
*		  2 = double line characters

* helptitle (character)
*	a string that will be printed in the border at the top of the window
*	( pass a blank character if you don't want a title )

* leftmargin (integer*4)
*	the number of leading blank characters that will be padded in front of
*	the lines of text read from "helpfile" before the text is displayed.
*	THIS ONLY APPLIES WHEN THE WINDOW IS FIRST CREATED.

* interact (logical)
*	flag specifying whether or not the window will be interactive (.TRUE.),
*	which will allow the user to scroll/page through the help message
*	and will automatically restore the screen to its previous state when
*	"poptext" is finished.  If "interact" is .FALSE., then the help screen
*	will be displayed and control will be returned to the calling routine.

* bld, und, rev (integer*4)
*	the input text file can contain control characters that specify when
*	to display certain text attributes, by default, CTRL-B (^B) will turn
*	on bold, CTRL-U (^U) will turn on underline, CTRL-R (^R) will turn
*	on Reverse, and CTRL-F (^F) will turn on flash (blink).  CTRL-N (^N)
*	will set the attributes back to the default background.
*	If working with colors, then these default values may not be as eye
*	appealing as you would like (and Reverse or Underline may not show up
*	at all).
*	Therefore the variables "bld", "und", and "rev" are passed to the
*	routine so that the programmer can override the default values with
*	any attributes desired.  To use the a default value, pass -1 with
*	the appropriate variable.  Note these values won't change after a text
*	file has been read into a window the first time, whether or not you
*	specify new attributes.

	implicit none

	include '../../curses.fh'

	integer*4	pad
	character*(*)	filename
	integer*4	background
	integer*4	bordback
	integer*4	bc
	integer*4	pminrow, pmincol
	integer*4	sminrow, smincol, smaxrow, smaxcol
	character*(*)	helptitle
	integer*4	leftmargin
	logical*4	interact
	integer*4	bld, und, rev

*	shadow holds the contents of the screen that will be hidden
*	when another window is displayed

	integer shadow

*	"screenimage" utility function (scrimage.f)

	integer screenimage

*	"lastchr" function

	integer lastchr

*---------------------------------

	character tmpstr*256

	integer retval, sy, sx, ey, ex, horz, vert, ur, ul, br, bl, i,
&		pc, pr, opc, opr, bldval, undval, revval, tpos

	integer fn, longline, totlines, endstr, y, ch, borderwin, maxy, maxx,
&		count

	logical exist, updatepad

*******************************************************************

*	have a default directory where help files are found if they are not
*	found in the current directory.

	character helpdir*256, tmpname*256

	data helpdir / '/usr/local/lib/fcdemo/' /

*******************************************************************

	poptext = NULL

*	if pad is null, then read the helpfile and create the window pad

	if( pad .lt. 0 .or. pad .eq. NULL ) then

*		open the file

		tmpname = filename
		inquire( file=tmpname, exist=exist )
		if( .not. exist ) then

*			if it's not in the current directory, see if it is
*			located in the default help directory.

			tmpname = helpdir(:lastchr(helpdir))//filename
			inquire( file=tmpname, exist=exist )
			if( .not. exist ) then
			  tmpname = filename
			  call popterr( ' file ' // tmpname(:lastchr(tmpname))//
&			   ' does not exist ' )
			  return
			endif
		endif
		do i = 50,63
			inquire( i, opened=exist )
			if( .not. exist ) then
				fn = i
				open( fn, file=tmpname, status='unknown', err=100 )
				go to 200
			endif
  100			continue
		end do
		call popterr( ' no available file numbers ' )
 		return
  200		continue

*		read through the file once to determine the size of the
*		window pad

		longline = 0
		totlines = 0
		do while ( .true. )
			read( fn, '(A)', end=300 ) tmpstr
			totlines = totlines + 1

* 	we need to expand tabs to determine the true "length" of the line
*	when displayed ( assuming tabs get expanded to every 8th column
*	by addch ).

			endstr = lastchr(tmpstr)
			tpos = 0
			do i = 1, endstr
				tpos = tpos + 1
				if(ichar(tmpstr(i:i)) .eq. KEY_TAB) then
					tpos = tpos + (8 - MOD( tpos,8 ))
				endif
			end do

			longline = max( tpos, longline )
		end do

  300		totlines = totlines + 1
  		longline = longline + leftmargin
  		if( longline .gt. 256 ) longline = 256

  		if( totlines .eq. 0 .or. longline .eq. 0 ) then
  			call popterr( 'no text in help file' )
  			return
  		endif

*		create help window *

		poptext = newpad( max(totlines,LINES), max(longline,COLS) )

		if( poptext .eq. NULL ) then
			call popterr( ' couldn''t create pad ' )
			return
		endif

		call wbkgd( poptext, background )

		call keypad( poptext, .TRUE.)
		call nodelay( poptext, .FALSE.)
* assumes noecho() has been called previously

*		read the help file again, this time storing its contents
*		into the pad

		rewind fn

		if( bld .lt. 0 ) then
			bldval = A_BOLD
		else
			bldval = bld
		endif
		if( und .lt. 0 ) then
			undval = A_UNDERLINE
		else
			undval = und
		endif
		if( rev .lt. 0 ) then
			revval = A_REVERSE
		else
			revval = rev
		endif

		y = 0
		retval = wmove( poptext, y, max(leftmargin,0) )
		do while (.true.)

			read(fn,'(A)', end=400) tmpstr

			endstr = lastchr(tmpstr)

*			add attributes to the text if certain
*			control characters are imbedded in the input file.

			do i = 1, endstr
				ch = ichar(tmpstr(i:i))
	
				if ( ch .eq. CTRL('b') ) then
					retval = wattron(poptext, bldval)
				elseif ( ch .eq. CTRL('u') ) then
					retval = wattron(poptext, undval)
				elseif ( ch .eq. CTRL('r') ) then
					retval = wattron(poptext, revval)
				elseif ( ch .eq. CTRL('f') ) then
					retval = wattron(poptext, A_BLINK)
				elseif ( ch .eq. CTRL('n') ) then
					retval = wattrset(poptext, 0)
				else
					retval = waddch(poptext, ch)
				endif
			end do

			y = y + 1
			if( ERR .eq. wmove( poptext, y, max(leftmargin,0) ) ) then
				go to 400
			endif
		end do

  400		close(fn)

	else

		poptext = pad

		retval = B_AND( wgetbkgd( poptext ), A_ATTRIBUTES )
		if( retval .ne. background ) then
			call wbkgd( poptext, background )
		endif

	endif


*	before displaying the help window, save a copy of "curscr" so it
*	can be retreived when help is exited.  Use the "screenimage" utility.
*		(scrimage.f)

	if( interact ) then
		shadow = screenimage( NULL, stdscr )
		if( shadow .eq. NULL ) then
			call popterr( ' couldn''t save screen image ' )
			return
		endif
	endif

*	now display the help window on the physical screen.


*	account for border in window positioning and set window size according
*	to passed arguments.

	sy = sminrow
	sx = smincol
	ey = smaxrow
	ex = smaxcol

	if( ey .eq. 0 .or. ex .eq. 0 ) then
		sy = 0
		sx = 0
		ey = LINES-1
		ex = COLS-1
	endif

	if( (ey - sy) .lt. 4 ) then
		ey = sy + 4
	endif
	if( (ex - sx) .lt. 19 ) then
		ex = sx + 19
	endif

* create the border window, display it then delete it from memory

	if( bc .ge. 0 ) then

		borderwin = newwin( ey-sy+1, ex-sx+1, sy, sx )
		if( borderwin .ne. NULL ) then

			call clearok( borderwin, .FALSE. )

			if( bordback .lt. 0 ) then
				retval = wattron( borderwin, background )
			else
				retval = wattron( borderwin, bordback )
			endif

			if( bc .eq. 1 ) then
				vert = ACS_VLINE
				horz = ACS_HLINE
				ul = ACS_ULCORNER
				ur = ACS_URCORNER
				bl = ACS_LLCORNER
				br = ACS_LRCORNER
			elseif( bc .eq. 2 ) then
				vert = ACS_DVLINE
				horz = ACS_DHLINE
				ul = ACS_DULCORNER
				ur = ACS_DURCORNER
				bl = ACS_DLLCORNER
				br = ACS_DLRCORNER
			else
				vert = ichar(' ')
				horz = ichar(' ')
				ul = ichar(' ')
				ur = ichar(' ')
				bl = ichar(' ')
				br = ichar(' ')
			endif

			call wborder( borderwin, vert, vert, horz, horz,
&				ul, ur, bl, br )

			endstr = lastchr(helptitle)
			if( endstr .gt. 0 ) then
				endstr = min( endstr, ex-sx-1 )
				retval = wmove( borderwin, 0, (ex-sx)/2-endstr/2-1)
				call waddstr( borderwin, ' ' )
				call waddstr( borderwin, helptitle(:endstr) )
				call waddstr( borderwin, ' ' )
			endif

			call wnoutrefresh( borderwin )
			call delwin( borderwin )

			sy = sy + 1
			sx = sx + 1
			ey = ey - 1
			ex = ex - 1

		endif
	endif


*	now display the text within the border

	retval = wmove(poptext, pminrow+ey-sy, pmincol+ex-sx)
	call clearok( poptext, .FALSE. )
	if( prefresh(poptext, pminrow,pmincol, sy, sx, ey, ex) .eq. ERR ) then
		call beep()
		call delwin(poptext)
		poptext = NULL
		call popterr( ' couldn''t refresh pad ' )
		go to 1000
	endif

*	if "interact" is FALSE, then return control of the window to the
*	calling routine (leave it displayed).

	if( .not. interact ) return

*	when any key is pressed, restore the "pre-help" contents of the
*	screen.

	call getmaxyx( poptext, maxy, maxx )

	pr = pminrow
	pc = pmincol
	count = 0
	do while ( .true. )

		updatepad = .FALSE.
		retval = wgetch(poptext)

		opr = pr
		opc = pc

		if ( retval .eq. CTRL('L') ) then
			call touchwin( curscr )
			call wrefresh( curscr )
		elseif( retval .eq. KEY_BACKSPACE .or. retval .eq. KEY_ESC ) then
			call touchwin( shadow )
			call wrefresh( shadow )
			call flushinp()
			go to 1000
		elseif( retval .eq. KEY_UP ) then
			pr = max( 0, pr - 1 )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_DOWN ) then
			pr = min( maxy-(ey-sy+1), pr + 1 )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_LEFT ) then
* left and right arrow keys, move 8 columns
			pc = max( 0, pc - 8 )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_RIGHT ) then
			pc = min( maxx-(ex-sx+1), pc + 8 )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_PPAGE ) then
			pr = max( 0, pr - ( ey - sy + 1 ) )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_NPAGE ) then
			pr = min( maxy-(ey-sy+1), pr + ( ey - sy + 1 ) )
			updatepad = .TRUE.
		elseif( retval .eq. KEY_HOME ) then
			pr = 0
			pc = 0
			updatepad = .TRUE.
		elseif( retval .eq. KEY_END ) then
			pr = maxy-(ey-sy+1)
			updatepad = .TRUE.
		endif
		if( updatepad ) then
		  if( pc .eq. opc .and. pr .eq. opr) then
		  	call beep()
		  else
		  	retval = wmove(poptext, pr+ey-sy, pc+ex-sx)
		  	retval = prefresh(poptext, pr,pc, sy, sx, ey, ex)
		  endif

		endif

*	calling flushinp() here keeps a lot of keystrokes from building up
*	if a person is sitting on an arrow key
		count = count + 1
		if( count .gt. 2 ) then 
			call flushinp()
			count = 0
		endif
	end do

 1000	continue
 
*	return the coordinates of the text that was being displayed when
*	the user closed the window.

	pminrow = pr
	pmincol = pc

*	delete the shadow window

 	call delwin(shadow)

*	flush any input keystrokes and return

	call flushinp()
 	return

  	end

*-------------------------------------------------------------

*  dignostic error message

	subroutine popterr( errmes )
	
	implicit none
	
	include '../../curses.fh'
	
	character*(*) errmes
	integer*4 retval

*	print the message in the upper left corner of the screen and wait
*	for a keystroke (won't wait if "nodelay" has been set true on
*	stdscr).

	call mvaddstr(0,0, errmes)
	call refresh()

	retval=sgetch()

	return
	end
