* movebox - use cursor keys to move a box around screen *

* wrs (7/18/92) - converted to fortran from an example provided as part of a
*		C curses package, although with some significant methodologic
*		changes.

	implicit none

	include '../../curses.fh'

* move a small window over stdscr, alternatingly copy one over the other too
* the virtual screen, but only send output to the physical screen after the
* small window has been refreshed.  This gives the effect that the small window
* is moving in front of a background.

* "small" is the popup window (box)

	integer small
	common /smallcom/ small

* current position of the popup *
	integer y, x
	common /position/ y,x

 	integer i, retval
 	integer ch

	character  HELPMSG*59, HELPFILE*12
	integer    FIRST_X, FIRST_Y, W_LINES, W_COLS, help

	data HELPMSG / 
&		'<Enter> turns bounce on.  Hit F1 or <CTRL>-h for more help.'/
	data HELPFILE / 'movebox.hlp'/

	data FIRST_X / 5 /
	data FIRST_Y / 5 /
	data W_LINES / 5 /
	data W_COLS  / 11 /

*   initialize curses

	retval = initscr()

*  set certain input/output options **************************

*  don't echo keystrokes, get keystrokes immediately, 
*  interpret special keys, hide cursor if possible

	retval = noecho()
	retval = cbreak()
	retval = raw()
	retval = keypad(stdscr,.TRUE.)
	retval = leaveok(stdscr, .TRUE.)
	retval = nonl()
	retval = curs_set( 0 )

*  only get a character after the screen has been completely written since
*  the last key action (Disable typeahead checking).  THIS IS CRITICAL
*  FOR GETTING A SMOOTH APPEARANCE OF THE BOX WHEN IN MOTION.  Comment
*  out this call to see the difference, especially when holding down
*  one of the arrow keys.

	retval = typeahead(-1)

**************************************************************


*  display the initial help message

	call message(HELPMSG)

* fill up the background window with various characters

	do y = 1, LINES-1
	   do x = 0, COLS-1
	      retval = mvwaddch(stdscr, y, x, ICHAR(' ') + x )
	   end do
	end do

*	refresh the background to the virtual screen only

	retval = wnoutrefresh(stdscr)

*	create a small pop-up window to move around

	y = FIRST_Y
	x = FIRST_X
	small = newwin(W_LINES, W_COLS, y, x)

*	if the terminal has color capabilities, make it stand out with a
*	white on red color pair.

*	if colors are not supported, set the box to inverse video

	if(has_colors()) then
		retval = start_color()
		retval = init_pair( 1, COLOR_WHITE, COLOR_RED )
		retval = wattrset( small, COLOR_PAIR(1) )
		retval = wattron( small, A_BOLD )
	else
		retval = wattrset(small, A_REVERSE)
	endif

	retval = wclear(small)

*	add text to window *

	retval = mvwaddstr(small, 1, 1, ' Movebox ')
	retval = mvwaddstr(small, 2, 1, '         ')
	retval = mvwaddstr(small, 3, 1, '  Demo   ')

*	box it, with "double line" graphics if at386, other terminal types
*	will inherit the single line definitions.

	retval = wborder( small, ACS_DVLINE, ACS_DVLINE, ACS_DHLINE,
     .		ACS_DHLINE, ACS_DULCORNER, ACS_DURCORNER,
     .		ACS_DLLCORNER, ACS_DLRCORNER)

********************************
*  main command loop -- get user commands and move the small window 
*                       around the screen
*    - this is accomplished by alternately writing "stdscr" and the small window
*	as the "movebox" is moved around the screen.   The physical screen is
*	updated only after the movebox is copied over the background.
*       This makes it appear that the "movebox" is always in front of the
*	background window.

  100	continue

*		popup the small window

		retval = touchwin(small)
		retval = wrefresh(small)

*		get command from user (from stdscr)

		ch = sgetch()

*		clear away the small window by touching stdscr
*		but do not show the change on the physical screen
*			(wnoutrefresh instead of wrefresh)

		retval = touchwin(stdscr)
		retval = wnoutrefresh(stdscr)

*		depending on the key pressed, either move the box, start the
*		box bouncing, bring up the help window, or exit.

		if( ch .eq. KEY_RIGHT ) then
			if( ERR .eq. mvwin(small, y, x+1) ) then
				retval = beep()
			else
				x = x + 1
			endif
		elseif( ch .eq. KEY_LEFT ) then
			if( ERR .eq. mvwin(small, y, x-1) ) then
				retval = beep()
			else
				x = x - 1
			endif
		elseif( ch .eq. KEY_UP ) then
			if( ERR .eq. mvwin(small, y-1, x) ) then
				retval = beep()
			else
				y = y - 1
			endif
		elseif( ch .eq. KEY_DOWN ) then
			if( ERR .eq. mvwin(small, y+1, x) ) then
				retval = beep()
			else
				y = y + 1
			endif
		elseif( ch .eq. KEY_NEWLN .or. ch .eq. KEY_CR ) then
			call bounce()
			call message(HELPMSG)
		elseif( ch .eq. KEY_F(1) .or. ch .eq. CTRL('H') ) then
			if( ERR .eq. help(HELPFILE) ) then
			  call message('Error setting up help window')
			endif
		else

*			scroll 1 line when quitting to clear the prompt line

			retval = scrollok(stdscr,.TRUE.)
			retval = wscrl(stdscr,1)
			retval = wrefresh(stdscr)
			retval = endwin()
			stop
		endif
	go to 100

	end

************************************************************************

* bounce - make the small window bounce, vaguely like pong

	subroutine bounce()

	implicit none

	include '../../curses.fh'

	integer seed
	save seed

	integer small
	common /smallcom/ small

	integer y, x
	common /position/ y,x

	integer yi, xi, ysign, xsign
	integer ch
	integer randomize, retval
	real random

	character*28 bouncemes
	data bouncemes / 'Hit any key to stop bouncing' /

	data seed / 13567 /

	yi = 1
	xi = 1
	randomize = 0

	call message( bouncemes )

*	set "nodelay" so that "sgetch" won't wait for the user if a key
*	hasn't already been pressed, instead it will just say there was an
*	error when trying to get user input.

	retval = nodelay(stdscr,.TRUE.)
  100	if ( sgetch() .ne. ERR ) go to 1000

*		try continue moving in same direction *

		if( OK .eq. mvwin(small, y+yi, x+xi) ) then
			y = y + yi
			x = x + xi

*		if it hit the screen border, time to change direction.  randomly
*		increase or decrease the speed of the movebox each time
*		the screen border is encountered.

		else
			if( ERR .eq. mvwin(small, y+yi, x) ) then
				yi = -yi
				ysign = yi/iabs(yi)
				if( randomize .gt. 0 ) then
				  if( random(seed) .gt. .5 ) then
					yi = 1
				  else
				  	yi = yi + ysign
				  endif
				  yi = min(iabs(yi),5)
				  yi = yi*ysign
				endif
			
			else
				xi = -xi
				xsign = xi/iabs(xi)
				if( randomize .gt. 0 ) then
				  if( random(seed) .gt. .5 ) then
					xi = 1
				  else
				  	xi = xi + xsign
				  endif
				  xi = min(iabs(xi),5)
				  xi = (xi * COLS/LINES*.8) * xsign
				endif
			endif
		endif

*		make the small window popup again (goes to screen)

		retval = touchwin(small)
		retval = wrefresh(small)

*		and remove it (only goes to virtual screen)

		retval = touchwin(stdscr)
		retval = wnoutrefresh(stdscr)

		if( randomize .eq. 0 ) then
		  randomize = 1
		else
		  randomize = 0
		endif

	go to 100
 1000	continue

*	if here, then the user finally pressed a key 

*	reset "nodelay" to false so that "sgetch" will wait for the user

	retval = nodelay(stdscr,.FALSE.)

*	flush the keystrokes that terminated the bouncing so that they won't
*	be processed by the calling routine.

	retval = flushinp()

	return
	end

***********************************************************************

* print a message on line one of the background screen and copy that
* line to "stdscr" for final display

	integer function message(str)

	implicit none

	include '../../curses.fh'

	character*(*) str
	integer retval

*	print message on the first line of the screen

	retval = move(0,0)

*	clear to the end of the line

	retval = clrtoeol()

*	display the message in the terminals best highlighting mode

	retval = standout()
	retval = addstr( str )
	retval = standend()

	retval = wnoutrefresh(stdscr)

	message = OK

	return
	end

************************************************************************

* the following two functions, help() and fillwin(), are general
*  purpose routines.  help displays a help window; fillwin fills
*  a window from a file, interpreting control characters as attributes.

* these also must be copied to stdscr before displaying stdscr

*  Usage:
*	integer function help(filename)
*	  character*(*) filename
*	returns OK if successful, ERR otherwise.
*	In this version, the filename is only used the first time
*	through; subsequent calls just use the same data.
*	An error return is possible only on the first call, and
*	indicates a problem with the help file or a memory shortage.
*
*	The help window disappears when any key is pressed.
*
*	integer function fillwin(f, win)
*	 integer f; integer win;
*	returns 0 on EOF, 1 otherwise
*	Reads data until a formfeed, puts data into window.  Several
*	control characters are allowed; their occurrence in the file
*	causes fillwin to modify attributes.  The valid characters are:
*		^B - bold
*		^F - flash
*		^N - normal
*		^R - reverse
*		^U - underline


	integer function help(file)

	implicit none

	include '../../curses.fh'

	character*(*) file

	integer helpwin

*	shadow holds the contents of the screen that will be hidden
*	when another window is displayed

	integer shadow

*	"screenimage" utility function
	integer screenimage

*	help file "f"

	integer f

	save f, shadow, helpwin

	character*4 tmpstr

	integer fillwin, retval

	data helpwin / -1 /
	data f / 23 /

	if( helpwin .lt. 0 .or. helpwin .eq. NULL ) then

*		create help window *

		helpwin = newwin(0,0,0,0)

		if( helpwin .eq. NULL ) then
		  help = ERR
		  return
		endif

		retval = leaveok(helpwin, .TRUE.)
		retval = keypad(helpwin, .TRUE.)

*		open up help file *

		OPEN(f, file=file, status='old', err=500)	

*		read help file into window *

		retval = fillwin(f, helpwin)

		close(f)

		shadow = NULL

	endif

*	before displaying the help window, save a copy of "stdscr" so it
*	can be retreived when help is exited.  Use the "screenimage" utility.

	shadow = screenimage( shadow, stdscr )
	if( shadow .eq. NULL ) then
		help = ERR
		return
	endif

*	now display the help window on the physical screen.

	retval = touchwin( helpwin )
	retval = wrefresh( helpwin )

*	when any key is pressed, restore the "pre-help" contents of the
*	screen.

	retval = wgetch(helpwin)
	retval = touchwin(shadow)
	retval = wrefresh( shadow )
	retval = flushinp()

	help = OK

	return

  500	help = ERR
  	return
  	end

***************************************************************************

* fillwin - fill window with next paragraph from a file
*
*	returns 1 on success (or end of window), 0 if at end of file
*
*	if special control characters are found, they are
*	interpreted as attribute control:
*		^B bold
*		^F flashing
*		^N normal
*		^R reverse
*		^U underline

	integer function fillwin(f, win)

	implicit none

	include '../../curses.fh'

	integer f, win

	integer ch
	integer y, x

	character  tmpstr*256
	integer  endstr, lastchr,i, retval
 
*	indent a few columns

	retval = wmove(win, 0, 3)

*	read the contents of the file into the window until the last line
*	of the window or the file is reached.

  100	continue

		read(f,'(A)', end=1000) tmpstr

		endstr = lastchr(tmpstr)

*		break on formfeed, add attributes to the text if certain
*		control characters are imbedded in the input file.

		do i = 1, endstr
			ch = ichar(tmpstr(i:i))

			if( ch .eq. KEY_FF ) go to 500

			if ( ch .eq. CTRL('b') ) then
				retval = wattron(win, A_BOLD)
			elseif ( ch .eq. CTRL('f') ) then
				retval = wattron(win, A_BLINK)
			elseif ( ch .eq. CTRL('n') ) then
				retval = wattrset(win, 0)
			elseif ( ch .eq. CTRL('r') ) then
				retval = wattron(win, A_REVERSE)
			elseif ( ch .eq. CTRL('u') ) then
				retval = wattron(win, A_UNDERLINE)
			else
				if( ERR .eq. waddch(win, ch) ) then
					fillwin = 1
					return
				endif
			endif
		end do

		retval = getyx(win, y, x)
		if( ERR .eq. wmove(win, y+1, 3) ) then
			fillwin = 1
			return
		endif
	go to 100
  500	fillwin = 1
	return
 1000	fillwin = 0
 	return
 	end

**************************************************************************

* find the last non-blank character in a fortran character string

	integer function lastchr( str )

	implicit none

	character*(*) str

	integer tmplen, i
	
	tmplen = len(str)

	do i = tmplen, 1, -1
		if ( str(i:i) .ne. ' ' .and. str(i:i) .ne. char(0)) then
			lastchr = i
			return
		endif
	end do
	lastchr = 0
	return
	end

*****************************************************************

*  a generic overflow psuedo random number generator
*	(requires a seed number to be passed from/to the calling program)

	real*4 function random( seed )
	
	implicit none
	
	integer*4 seed, iy

	iy = seed * 65539
	if ( iy .lt. 0 ) iy = ( iy + 2147483647 ) + 1

	random = iy * 0.4656613e-09

	if(random .lt. 0.0001) then
	  random = 0.000001
	else if( random .ge. 1.0) then
	  random = 0.999999
	endif

	seed = iy

	return
	end

***********************************************************************
* wrs (8/13/92)
* -- SCREENIMAGE --
*  a routine to save any contents of a region the physical screen to a window
*	so that they can later be restored.

*  ideal for popping up menus

*  return NULL on error.
*  if successful, return a integer window pointer if win is NULL
*			return non-NULL otherwise

*  this routine may not work well if typeahead in enabled (Curses default)
*    because the screen may only get partially written before it is saved.
*    To disable typeahead the main program should make
*    the call  "call typeahead(-1)"
*****************************************************

	integer*4 function screenimage( shadow, win )

	implicit none

	include '../../curses.fh'

	integer*4  shadow, win, retval, begy, begx, ydim, xdim

***depending on the situation, one might want to do an update at this point**
*	retval = doupdate()
**********************

	screenimage = NULL

*	"shadow" is the copy that will be made of window "win"
*	if "shadow" is NULL, then the window hasn't ever been created
*	up to this point.  Duplicate the contents of "win" in "shadow"

	if( shadow .eq. NULL) then

*		get the orientation and dimensions of the window "win"

		call getbegyx( win, begy, begx )
		call getmaxyx( win, ydim, xdim )

		retval = newwin( ydim, xdim, begy, begx )
		if( retval .ne. NULL ) then
			if( overwrite( curscr, retval ) .eq. ERR ) then
				call delwin( retval )
				return
			endif
			screenimage = retval
		else
			return
		endif
	else

*	if "shadow" has already been used once before, then just overwrite it
*	with the current contents of the screen in that area.

*	"curscr" is a window pointer to the physical screen image.

		if( overwrite(curscr, shadow) .eq. ERR ) then
			call delwin( shadow )
			shadow = NULL
			return
		else
			screenimage = shadow
		endif
	endif

	return

	end
