*
*  this routine offers the user a choice of color pairs that they can
*  pick from in a table layout
*
*  it relies on the utility routine colorset.f and the include file
*  colorset.fh
*

	integer*4 function pickcolor()

	implicit none
	
	include '../../curses.fh'
	include 'colorset.fh'

	integer*4 tmpf, tmpb, pickwin, win128, y, x, i, j, pairnum, attr, ch
	integer*4 oj, oi, ox, oy

	character*1  backgd(8), forgd(16)
	
	data backgd / '1','2','3','4','5','6','7','8' /


*	returns A_NORMAL if something goes wrong or if the user
*	presses escape.

	pickcolor = A_NORMAL

	if( .not. has_colors() ) then
		return
	endif

*	the next tests check if you were lazy and called this routine
*	without calling "start_color" and "colorset"

	if( COLORS .eq. 0 ) then
		if( start_color() .eq. ERR) then
			return
		endif
	endif
	if( pair_content( COLOR_PAIRS-1, tmpf, tmpb) .eq. ERR ) then
		call colorset()
	endif

*	cbreak() and noecho() need to be called for this function to work
*	as intended.  Do them explicitly here with the caution that the
*	programmer might want to have different settings made after
*	returning to the calling routine.

	call cbreak()
	call noecho()


*	create a new window that takes up the entire screen, then display
*	a table that contains all the color pairs available to the user

*	they can then pick from the coordinates of the table

	pickwin = newwin(0,0,0,0)

	call mvwaddstr( pickwin, LINES/2-10, COLS/2-7 , 'Color Choices' )

	call wrefresh(pickwin)

	win128 = newwin( 18,51, LINES/2-9, COLS/2-26 )
	call keypad(win128,.TRUE.)
	call wbkgd( win128, MAGENTA_BLACK )
	call wborder(win128, 0,0,0,0,0,0,0,0)

	do j = 1,8
		y = (j-1)*2 + 1
		do i = 1, 16
			x = 2 + (i-1)*3

			pairnum = (j - 1) * 8 + i
			if( i .gt. 8 ) then
				pairnum = pairnum - 8
			endif

*			"colorattr" is set by the routine colorset

			attr = colorattr(pairnum)
			if( i .gt. 8 ) then
				attr = B_OR( A_BOLD, attr)
			endif

			if(pairnum .ne. 1) then
				call wattrset(win128,attr)
				call mvwaddstr(win128,y,x,'XX')
			endif

		end do

	end do

	call wnoutrefresh(win128)

	call wattron(pickwin, A_REVERSE)

	call mvwaddstr(pickwin,LINES/2+10, 6, 'Use arrow keys to change '//
&		'color choice, then press <Enter> to select.')

	call mvwaddstr(pickwin,LINES/2+11, 6, '<Escape> Cancels.        '//
&		'                                           ')

	call wrefresh(pickwin)

	call wattrset( win128, B_OR(A_BLINK,A_BOLD) )
	j = 1
	i = 2
	oj = 1
	oi = 2
	do while( .TRUE. )
		oy = (oj-1)*2 + 2
		ox = 2 + (oi-1)*3
		y = (j-1)*2 + 2
		x = 2 + (i-1)*3
		call mvwaddstr(win128,oy,ox,'  ')
		call mvwaddstr(win128,y,x,'^^')
		call wnoutrefresh(win128)
		call wmove(pickwin,LINES-1,COLS-1)
		call wrefresh(pickwin)

		oi = i
		oj = j

		ch = wgetch(win128)

 500		if( ch .eq. KEY_LEFT ) then
			i = i - 1
			if(i .lt. 1) i = 16
		elseif( ch .eq. KEY_RIGHT ) then	
			i = i + 1
			if(i .gt. 16) i = 1
		elseif( ch .eq. KEY_UP ) then
			j = j - 1
			if(j .lt. 1) j = 8
		elseif( ch .eq. KEY_DOWN ) then	
			j = j + 1
			if(j .gt. 8) j = 1
		elseif( ch .eq. KEY_CR .or. ch .eq. KEY_NEWLN ) then
		
			pairnum = (j - 1) * 8 + i
			if( i .gt. 8 ) then
				pairnum = pairnum - 8
			endif
			attr = colorattr(pairnum)
			if( i .gt. 8 ) then
				attr = B_OR( A_BOLD, attr)
			endif

			pickcolor = attr
			go to 1000

		elseif( ch .eq. KEY_ESC .or. ch .eq. KEY_BACKSPACE ) then
			go to 1000
		else
			call beep()
		endif

		if( j .ne. oj .or. i .ne. oi ) then
			if( j .eq. 1 ) then
			  if( i .eq. 1 .or. i .eq. 9 ) go to 500
			endif
		endif

	end do

 1000	call delwin(pickwin)
	call delwin(win128)

	return

	end
