*  tests many of the F-curses functions to verify that they work correctly
*     (taken from the testcurs.c routine that comes with PDcurses)

	implicit none

	include '../../curses.fh'

	logical inittest

	integer*4 win
	integer*4 width, height

	if ( .not. inittest(win, height, width) ) stop

	call padtest()

	call introtest(win)

	call scrolltest (win, height, width)

	call inputTest (win)

	call outputTest (win)

	call delwin (win)

	call endwin()

	end

***********************************************************************

	subroutine continue(win)

	implicit none
	
	include '../../curses.fh'

	integer win

	call wmove(win, 10, 1)
	call wclrtoeol(win)
	call mvwaddstr(win, 10, 1, ' Press any key to continue')
	call wrefresh(win)
	call raw()
	call wgetch(win)

	return
	end

*********************************************************

	logical function inittest (win, height, width)

	implicit none

	include '../../curses.fh'

	integer win
	integer height
	integer width

	inittest = .TRUE.

	call initscr()
	if ( has_colors() ) call start_color()

	width  = 60
	height = 13
*		 Create a drawing window 
	win = newwin(height, width, (LINES-height)/2, (COLS-width)/2)

	if(win .eq. NULL) then
		call endwin()
		inittest = .FALSE.
		return
	endif

	call wmove( win, height/2-5, width/2 )
	call wvline( win, ACS_VLINE, 10 )
	call wmove( win, height/2, width/2-10 )
	call whline( win, ACS_HLINE, 20 )
	call continue(win)

	return
	end

*****************************************************************

	subroutine introtest(win)

	implicit none

	include '../../curses.fh'

	integer win

	call beep()
	if (has_colors()) then
		call init_pair(1,COLOR_WHITE,COLOR_BLUE)
		call wbkgd(win, COLOR_PAIR(1))
	else
		call wbkgd(win, A_REVERSE)
	endif

	call werase(win)

	call box(win, ACS_VLINE, ACS_HLINE )
	call wrefresh(win)
	call cbreak()
	call mvwaddstr(win, 1, 1, 'You should have rectangle in the middle of the screen')
	call mvwaddstr(win, 2, 1, 'You should have heard a beep')
	call continue(win)

	return
	end

**************************************************************

	subroutine scrolltest (win, height, width)

	implicit none

	include '../../curses.fh'

	integer win
	integer height, width

	integer i
	integer OldX, OldY
	character Message*34
	
	Message = 'The window will now scroll slowly'

* disable typeahead checking

	call typeahead(-1)

	call mvwaddstr (win, height - 2, 1, Message)
	call wrefresh (win)
	call scrollok(win, .TRUE.)

	do i = 1, height
		call delay_output (150)
		call scroll(win)
		call wrefresh (win)
	end do

	call getmaxyx (win, OldY, OldX)
	call mvwaddstr (win, 6, 1, 'The top of the window will scroll')
	call wmove (win, 1, 1)
	call wsetscrreg (win, 0, 4)

	call box(win, 0, 0)
	call wrefresh (win)
	do i = 1, 5
		call delay_output (500)
		call scroll(win)
		call wrefresh (win)
	end do

	call mvwaddstr (win, 3, 1, 'The bottom of the window will scroll')
	call wmove (win, 8, 1)
	OldY = OldY - 1
	call wsetscrreg (win, 5, OldY)
	call box(win, 0, 0)
	call wrefresh (win)
	do i = 5, OldY
		call delay_output (300)
		call wscrl(win,-1)
		call wrefresh (win)
	end do
	call wsetscrreg (win, 0, OldY)

	return
	end

******************************************************************

	subroutine inputTest (win)

	implicit none

	include '../../curses.fh'

	integer*4 win

	integer w, h, bx, by, sw, sh, i, c

	character buffer*80
	integer subW

	call wclear (win)

	call getmaxyx(win, h, w)
	call getbegyx(win, by, bx)

	sw = w / 3
	sh = h / 3

	subW = subwin(win, sh, sw, by + h - sh - 2, bx + w - sw - 2)
	if( subW .eq. NULL) return

	if (has_colors()) then
		call init_pair(2,COLOR_WHITE,COLOR_RED)
		call wbkgd(subW, B_OR(COLOR_PAIR(2), A_BOLD) )
	else
		call wbkgd(subW, A_BOLD)
	endif

	call box(subW, ACS_VLINE, ACS_HLINE)
	call wrefresh(win)

	call nocbreak()
	call mvwaddstr(win, 2, 1, 'Press some keys for 5 seconds')
	call mvwaddstr(win, 1, 1, 'Pressing Interrupt key should do nothing')
	call wrefresh(win)

	call werase (subW)
	call box(subW, ACS_VLINE, ACS_HLINE)
	do i = 0, 4
		write( buffer, '(A,I1)' ) 'Time = ',i
		call mvwaddstr (subW, 1, 1, buffer(:8) )
		call wrefresh(subW)
		call delay_output(1000)
		call flushinp()
	end do

	call delwin (subW)
	call werase(win)
	call flash()
	call wrefresh(win)
	call delay_output(500)
	call flushinp()

	call mvwaddstr(win, 2, 1, 'Press a key, followed by ENTER')
	call wmove(win, 9, 10)
	call wrefresh(win)
	call echo()
	call noraw()
	call wgetnstr(win, buffer, 3)
	call flushinp()

	call wmove(win, 9, 10)
	call wdelch(win)
	call mvwaddstr(win, 4, 1, 'The character should now have been deleted')
	call Continue(win)

	call werase (win)
	call mvwaddstr(win, 2, 1, 'Press an arrow key')
	call wrefresh(win)
	call keypad(win, .TRUE.)
	call raw()
	c = wgetch(win)

	call nodelay(win, .TRUE.)
	i = wgetch(win)
	call nodelay(win, .FALSE.)

	call refresh()
	call wclear (win)
	call mvwaddstr(win, 3, 2, 'The window should have moved')
	call mvwaddstr(win, 4, 2, 'This text should have appeared without you pressing a key')
	write( buffer, '(A,I3)' ) 'Keycode = ',c
	call mvwaddstr(win, 2, 2, buffer(:13))
	call mvwaddstr(win, 6, 2, 'Enter a string')
	call mvwin(win, 2, 1)
	call wrefresh(win)
	call mvwgetstr(win, 7, 6, buffer)
	call mvwaddstr(win, 9, 3, 'You entered: ' )
	call waddstr(win, buffer)
	call Continue(win)

	return
	end

******************************************************************

	subroutine outputTest (win)

	implicit none

	include '../../curses.fh'

	integer*4 win

	integer*4 win1
	character Buffer*80
	integer ch
	integer by, bx

	call nl ()
	call werase (win)
	call mvwaddstr(win, 1, 1, 'You should now have a screen in the upper left corner, and this text should have wrapped')
	call wprintw(win, char(KEY_NEWLN) // 'This text should be down' )
	call waddstr(win,  'and broken into two here ^')
	call Continue(win)

	call werase(win)
	call wattron(win, A_BOLD)
	call mvwaddstr(win, 1, 1, 'A new window will appear with this text in it')
	call mvwaddstr(win, 8, 1, 'Press any key to continue')
	call wrefresh(win)
	call wgetch(win)

	call getbegyx(win, by, bx)

	win1 = newwin(10, 50, 15, 25)
	if(win1 .eq. NULL) then
		call endwin()
		return
	endif

	if (has_colors()) then
		call init_pair(COLOR_PAIRS-1,COLOR_BLUE,COLOR_WHITE)
		call wbkgd(win1, COLOR_PAIR(COLOR_PAIRS-1))
	else
		call wbkgd(win1, A_NORMAL)
	endif

	call werase (win1)
	call mvwaddstr(win1, 5, 1, 'This text should not appear')
	call copywin( win, win1, 0,0,  0,0, 9,49, .FALSE.)

	call box(win1,ACS_VLINE,ACS_HLINE)
	call wmove(win1, 8, 26)
	call wrefresh(win1)
	call wgetch(win1)

	call werase(win1)
	call wattron(win1, A_BLINK)
	call mvwaddstr(win1, 4, 1, 'This blinking text should appear in only the second window')
	call wattroff(win1, A_BLINK)
	call mvwin(win1, by, bx)
	call overlay(win, win1)
	call mvwin(win1,15,25)
	call wmove(win1, 8, 26)
	call wrefresh(win1)
	call wgetch(win1)
	call delwin(win1)

	call clear()
	call werase(win)
	call wrefresh(win)
	call mvwaddstr(win, 6, 2, 'This line shouldn''t appear')
	call mvwaddstr(win, 4, 2, 'Only half of the next line is visible')
	call mvwaddstr(win, 5, 2, 'Only half of the next line is visible')
	call wmove(win, 6, 1)
	call wclrtobot (win)
	call wmove(win, 5, 20)
	call wclrtoeol (win)
	call mvwaddstr(win, 8, 2, 'This line also shouldn''t appear')
	call wmove(win, 8, 1)
	call wdeleteln(win)
	call Continue(win)

	call wmove (win, 5, 9)
	ch = winch (win)

	call werase(win)
	call wmove (win, 6, 2)
	call waddstr (win, 'The next char should be l:  ')
	call winsch (win, ch)
	call Continue(win)

	call mvwinsstr( win, 6, 2, 'A1B2C3D4E5' )
	call Continue(win)

	call wmove(win, 5, 1)
	call winsertln (win)
	call mvwaddstr(win, 5, 2, 'The lines below should have moved down')
	call Continue(win)

	call werase(win)
	call mvwaddstr(win, 10, 1, 'Enter a string: ')
	call wrefresh(win)
	call wgetstr (win, Buffer)

	call mvaddstr(10, 1, 'Enter a string: ')
	call getstr (Buffer)

	call werase(win)
	call curs_set(2)
	call mvwaddstr(win, 1, 1, 'The cursor should appear as a block')
	call Continue(win)

	call werase(win)
	call curs_set(0)
	call mvwaddstr(win, 1, 1, 'The cursor should have disappeared')
	call Continue(win)

	call werase(win)
	call curs_set(1)
	call mvwaddstr(win, 1, 1, 'The cursor should be an underline')
	call Continue(win)

	if( has_colors() ) then
		call werase(win)
		call mvwaddstr(win, 1, 1, 'Colors should change after you press a key')
		call Continue(win)
		call init_pair(1, COLOR_RED, COLOR_WHITE)
		call wrefresh(win)
	endif

	call werase(win)
	call mvwaddstr(win, 1, 1, 'Information About Your Terminal')
	call mvwaddstr(win, 3, 1, termname())
	call mvwaddstr(win, 4, 1, longname())
	call Continue(win)

	call mvwaddnstr( win, 6,5, 'Have a nice day!ok', 16 )
	call wrefresh(win)

	call mvwinnstr( win, 6,5, Buffer, 18 )
	call mvaddstr( LINES-2, 10, Buffer )
	call refresh()

	return
	end

*********************************************************************

	subroutine padTest()

	implicit none

	include '../../curses.fh'

	integer*4 pad

	pad = newpad(50,100)
	call wattron(pad, A_REVERSE)
	call mvwaddstr(pad, 5, 2, 'This is a new pad')
	call wattrset(pad,0)
	call mvwaddstr(pad, 8, 0, 'The end of this line should be truncated here:abcd')
	call mvwaddstr(pad,11, 1, 'This line should not appear.')

	if( B_AND( termattrs(), A_BLINK ) .gt. 0 ) then
		call mvwaddstr(pad,7, 1, 'This terminal supports blinking.')
	endif

	call wmove(pad, 10, 1)
	call wclrtoeol(pad)
	call mvwaddstr(pad, 10, 1, ' Press any key to continue')
	call prefresh(pad,0,0,0,0,10,45)
	call keypad(pad, .TRUE.)
	call raw()
	call wgetch(pad)

	call mvwaddstr(pad, 35, 2, 'This is displayed at line 35 in the pad')
	call mvwaddstr(pad, 40, 1, ' Press any key to continue')
	call prefresh(pad,30,0,0,0,10,45)
	call keypad(pad, .TRUE.)
	call raw()
	call wgetch(pad)

	call delwin(pad)

	return
	end
